@@ -277,7 +277,7 @@ static DWORD WINAPI audin_server_thread_func(LPVOID arg)
 	                           &BytesReturned) == TRUE)
 	{
 		if (BytesReturned == sizeof(HANDLE))
-			CopyMemory(&ChannelEvent, buffer, sizeof(HANDLE));
+			ChannelEvent = *(HANDLE*)buffer;
 
 		WTSFreeMemory(buffer);
 	}
@@ -569,9 +569,7 @@ static UINT audin_server_packet_send(audin_server_context* context, wStream* s)
 	WINPR_ASSERT(s);
 
 	const size_t pos = Stream_GetPosition(s);
-	if (pos > UINT32_MAX)
-		return ERROR_INVALID_PARAMETER;
-
+	WINPR_ASSERT(pos <= UINT32_MAX);
 	if (!WTSVirtualChannelWrite(audin->audin_channel, Stream_BufferAs(s, char), (UINT32)pos,
 	                            &written))
 	{
@@ -885,7 +883,8 @@ BOOL audin_server_set_formats(audin_server_context* context, SSIZE_T count,
 	}
 	else
 	{
-		AUDIO_FORMAT* audin_server_formats = audio_formats_new(count);
+		const size_t scount = (size_t)count;
+		AUDIO_FORMAT* audin_server_formats = audio_formats_new(scount);
 		if (!audin_server_formats)
 			return count == 0;
 
@@ -893,7 +892,7 @@ BOOL audin_server_set_formats(audin_server_context* context, SSIZE_T count,
 		{
 			if (!audio_format_copy(&formats[x], &audin_server_formats[x]))
 			{
-				audio_formats_free(audin_server_formats, count);
+				audio_formats_free(audin_server_formats, scount);
 				return FALSE;
 			}
 		}
