@@ -59,17 +59,17 @@ CliprdrClientContext* cliprdr_get_client_interface(cliprdrPlugin* cliprdr)
  */
 static UINT cliprdr_packet_send(cliprdrPlugin* cliprdr, wStream* s)
 {
-	size_t pos = 0;
-	UINT32 dataLen = 0;
 	UINT status = CHANNEL_RC_OK;
 
 	WINPR_ASSERT(cliprdr);
 	WINPR_ASSERT(s);
 
-	pos = Stream_GetPosition(s);
-	dataLen = pos - 8;
+	const size_t pos = Stream_GetPosition(s);
+	const size_t dataLen = pos - 8;
+	WINPR_ASSERT(dataLen <= UINT32_MAX);
+
 	Stream_SetPosition(s, 4);
-	Stream_Write_UINT32(s, dataLen);
+	Stream_Write_UINT32(s, (UINT32)dataLen);
 	Stream_SetPosition(s, pos);
 
 	WLog_DBG(TAG, "Cliprdr Sending (%" PRIu32 " bytes)", dataLen + 8);
@@ -574,15 +574,15 @@ static UINT cliprdr_client_capabilities(CliprdrClientContext* context,
 	 * -> Update clipboard context feature state to what was agreed upon.
 	 */
 	if (!cliprdr->useLongFormatNames)
-		flags &= ~CB_USE_LONG_FORMAT_NAMES;
+		flags &= (uint32_t)~CB_USE_LONG_FORMAT_NAMES;
 	if (!cliprdr->streamFileClipEnabled)
-		flags &= ~CB_STREAM_FILECLIP_ENABLED;
+		flags &= (uint32_t)~CB_STREAM_FILECLIP_ENABLED;
 	if (!cliprdr->fileClipNoFilePaths)
-		flags &= ~CB_FILECLIP_NO_FILE_PATHS;
+		flags &= (uint32_t)~CB_FILECLIP_NO_FILE_PATHS;
 	if (!cliprdr->canLockClipData)
-		flags &= ~CB_CAN_LOCK_CLIPDATA;
+		flags &= (uint32_t)~CB_CAN_LOCK_CLIPDATA;
 	if (!cliprdr->hasHugeFileSupport)
-		flags &= ~CB_HUGE_FILE_SUPPORT_ENABLED;
+		flags &= (uint32_t)~CB_HUGE_FILE_SUPPORT_ENABLED;
 
 	cliprdr->useLongFormatNames = (flags & CB_USE_LONG_FORMAT_NAMES) ? TRUE : FALSE;
 	cliprdr->streamFileClipEnabled = (flags & CB_STREAM_FILECLIP_ENABLED) ? TRUE : FALSE;
@@ -967,6 +967,8 @@ static VOID VCAPITYPE cliprdr_virtual_channel_open_event_ex(LPVOID lpUserParam,
 
 		case CHANNEL_EVENT_USER:
 			break;
+		default:
+			break;
 	}
 
 	if (error && cliprdr && cliprdr->context->rdpcontext)
@@ -1086,7 +1088,8 @@ static VOID VCAPITYPE cliprdr_virtual_channel_init_event_ex(LPVOID lpUserParam,
 				WLog_ERR(TAG,
 				         "cliprdr_virtual_channel_event_terminated failed with error %" PRIu32 "!",
 				         error);
-
+			break;
+		default:
 			break;
 	}
 
