@@ -27,6 +27,7 @@
 
 #include <winpr/crt.h>
 #include <winpr/assert.h>
+#include <winpr/cast.h>
 #include <winpr/stream.h>
 
 #include <freerdp/client/channels.h>
@@ -85,7 +86,7 @@ static BOOL location_write_header(wStream* s, UINT16 pduType, UINT32 pduLength)
 	return Stream_EnsureRemainingCapacity(s, pduLength);
 }
 
-static BOOL location_read_server_ready_pdu(LOCATION_CALLBACK* callback, wStream* s, UINT16 pduSize)
+static BOOL location_read_server_ready_pdu(LOCATION_CALLBACK* callback, wStream* s, UINT32 pduSize)
 {
 	if (pduSize < 6 + 4)
 		return FALSE; // Short message
@@ -221,12 +222,12 @@ static UINT location_send_base_location3d(IWTSVirtualChannel* channel,
 
 	if (pdu->source)
 		WLog_DBG(TAG,
-		         "latitude=%lf, logitude=%lf, altitude=%" PRId32
+		         "latitude=%lf, longitude=%lf, altitude=%" PRId32
 		         ", speed=%lf, heading=%lf, haccuracy=%lf, source=%" PRIu8,
 		         pdu->latitude, pdu->longitude, pdu->altitude, pdu->speed, pdu->heading,
 		         pdu->horizontalAccuracy, *pdu->source);
 	else
-		WLog_DBG(TAG, "latitude=%lf, logitude=%lf, altitude=%" PRId32, pdu->latitude,
+		WLog_DBG(TAG, "latitude=%lf, longitude=%lf, altitude=%" PRId32, pdu->latitude,
 		         pdu->longitude, pdu->altitude);
 
 	if (!location_write_header(s, PDUTYPE_BASE_LOCATION3D, pdu->source ? 25 : 12))
@@ -244,7 +245,7 @@ static UINT location_send_base_location3d(IWTSVirtualChannel* channel,
 		    !freerdp_write_four_byte_float(s, *pdu->horizontalAccuracy))
 			return ERROR_INTERNAL_ERROR;
 
-		Stream_Write_UINT8(s, *pdu->source);
+		Stream_Write_UINT8(s, WINPR_ASSERTING_INT_CAST(UINT8, *pdu->source));
 	}
 
 	return location_channel_send(channel, s);
@@ -264,10 +265,10 @@ static UINT location_send_location2d_delta(IWTSVirtualChannel* channel,
 	const BOOL ext = pdu->speedDelta && pdu->headingDelta;
 
 	if (ext)
-		WLog_DBG(TAG, "latitude=%lf, logitude=%lf, speed=%lf, heading=%lf", pdu->latitudeDelta,
+		WLog_DBG(TAG, "latitude=%lf, longitude=%lf, speed=%lf, heading=%lf", pdu->latitudeDelta,
 		         pdu->longitudeDelta, pdu->speedDelta, pdu->headingDelta);
 	else
-		WLog_DBG(TAG, "latitude=%lf, logitude=%lf", pdu->latitudeDelta, pdu->longitudeDelta);
+		WLog_DBG(TAG, "latitude=%lf, longitude=%lf", pdu->latitudeDelta, pdu->longitudeDelta);
 
 	if (!location_write_header(s, PDUTYPE_LOCATION2D_DELTA, ext ? 16 : 8))
 		return ERROR_OUTOFMEMORY;
@@ -300,11 +301,11 @@ static UINT location_send_location3d_delta(IWTSVirtualChannel* channel,
 	const BOOL ext = pdu->speedDelta && pdu->headingDelta;
 
 	if (ext)
-		WLog_DBG(TAG, "latitude=%lf, logitude=%lf, altitude=%" PRId32 ", speed=%lf, heading=%lf",
+		WLog_DBG(TAG, "latitude=%lf, longitude=%lf, altitude=%" PRId32 ", speed=%lf, heading=%lf",
 		         pdu->latitudeDelta, pdu->longitudeDelta, pdu->altitudeDelta, pdu->speedDelta,
 		         pdu->headingDelta);
 	else
-		WLog_DBG(TAG, "latitude=%lf, logitude=%lf, altitude=%" PRId32, pdu->latitudeDelta,
+		WLog_DBG(TAG, "latitude=%lf, longitude=%lf, altitude=%" PRId32, pdu->latitudeDelta,
 		         pdu->longitudeDelta, pdu->altitudeDelta);
 
 	if (!location_write_header(s, PDUTYPE_LOCATION3D_DELTA, ext ? 20 : 12))
@@ -365,7 +366,7 @@ static UINT location_send(LocationClientContext* context, LOCATION_PDUTYPE type,
 					speed = va_arg(ap, double);
 					heading = va_arg(ap, double);
 					horizontalAccuracy = va_arg(ap, double);
-					source = va_arg(ap, int);
+					source = WINPR_ASSERTING_INT_CAST(LOCATIONSOURCE, va_arg(ap, int));
 					pdu.speed = &speed;
 					pdu.heading = &heading;
 					pdu.horizontalAccuracy = &horizontalAccuracy;
