@@ -17,6 +17,8 @@
  * limitations under the License.
  */
 
+#include <winpr/cast.h>
+
 #include <freerdp/config.h>
 
 #include <freerdp/freerdp.h>
@@ -176,7 +178,10 @@ static UINT device_server_recv_stream_list_response(CameraDeviceServerContext* c
 	if (!Stream_CheckAndLogRequiredLength(TAG, s, 5))
 		return ERROR_NO_DATA;
 
-	pdu.N_Descriptions = MIN(Stream_GetRemainingLength(s) / 5, 255);
+	pdu.N_Descriptions = 255;
+	const size_t len = Stream_GetRemainingLength(s) / 5;
+	if (len < 255)
+		pdu.N_Descriptions = (BYTE)len;
 
 	for (BYTE i = 0; i < pdu.N_Descriptions; ++i)
 	{
@@ -501,6 +506,8 @@ static UINT device_server_context_poll_int(CameraDeviceServerContext* context)
 		case CAMERA_DEVICE_OPENED:
 			error = device_process_message(device);
 			break;
+		default:
+			break;
 	}
 
 	return error;
@@ -518,7 +525,7 @@ static HANDLE device_server_get_channel_handle(device_server* device)
 	                           &BytesReturned) == TRUE)
 	{
 		if (BytesReturned == sizeof(HANDLE))
-			CopyMemory(&ChannelEvent, buffer, sizeof(HANDLE));
+			ChannelEvent = *(HANDLE*)buffer;
 
 		WTSFreeMemory(buffer);
 	}
@@ -568,6 +575,8 @@ static DWORD WINAPI device_server_thread_func(LPVOID arg)
 						break;
 				}
 				break;
+			default:
+				break;
 		}
 	}
 
@@ -704,8 +713,10 @@ static UINT device_server_packet_send(CameraDeviceServerContext* context, wStrea
 	WINPR_ASSERT(context);
 	WINPR_ASSERT(s);
 
-	if (!WTSVirtualChannelWrite(device->device_channel, Stream_BufferAs(s, char),
-	                            Stream_GetPosition(s), &written))
+	const size_t len = Stream_GetPosition(s);
+	WINPR_ASSERT(len <= UINT32_MAX);
+	if (!WTSVirtualChannelWrite(device->device_channel, Stream_BufferAs(s, char), (UINT32)len,
+	                            &written))
 	{
 		WLog_ERR(TAG, "WTSVirtualChannelWrite failed!");
 		error = ERROR_INTERNAL_ERROR;
@@ -819,14 +830,14 @@ device_send_start_streams_request_pdu(CameraDeviceServerContext* context,
 
 		Stream_Write_UINT8(s, info->StreamIndex);
 
-		Stream_Write_UINT8(s, description->Format);
+		Stream_Write_UINT8(s, WINPR_ASSERTING_INT_CAST(uint8_t, description->Format));
 		Stream_Write_UINT32(s, description->Width);
 		Stream_Write_UINT32(s, description->Height);
 		Stream_Write_UINT32(s, description->FrameRateNumerator);
 		Stream_Write_UINT32(s, description->FrameRateDenominator);
 		Stream_Write_UINT32(s, description->PixelAspectRatioNumerator);
 		Stream_Write_UINT32(s, description->PixelAspectRatioDenominator);
-		Stream_Write_UINT8(s, description->Flags);
+		Stream_Write_UINT8(s, WINPR_ASSERTING_INT_CAST(uint8_t, description->Flags));
 	}
 
 	return device_server_packet_send(context, s);
@@ -879,7 +890,7 @@ device_send_property_value_request_pdu(CameraDeviceServerContext* context,
 	if (!s)
 		return ERROR_NOT_ENOUGH_MEMORY;
 
-	Stream_Write_UINT8(s, propertyValueRequest->PropertySet);
+	Stream_Write_UINT8(s, WINPR_ASSERTING_INT_CAST(uint8_t, propertyValueRequest->PropertySet));
 	Stream_Write_UINT8(s, propertyValueRequest->PropertyId);
 
 	return device_server_packet_send(context, s);
@@ -899,10 +910,11 @@ static UINT device_send_set_property_value_request_pdu(
 	if (!s)
 		return ERROR_NOT_ENOUGH_MEMORY;
 
-	Stream_Write_UINT8(s, setPropertyValueRequest->PropertySet);
+	Stream_Write_UINT8(s, WINPR_ASSERTING_INT_CAST(uint8_t, setPropertyValueRequest->PropertySet));
 	Stream_Write_UINT8(s, setPropertyValueRequest->PropertyId);
 
-	Stream_Write_UINT8(s, setPropertyValueRequest->PropertyValue.Mode);
+	Stream_Write_UINT8(
+	    s, WINPR_ASSERTING_INT_CAST(uint8_t, setPropertyValueRequest->PropertyValue.Mode));
 	Stream_Write_INT32(s, setPropertyValueRequest->PropertyValue.Value);
 
 	return device_server_packet_send(context, s);
