@@ -28,6 +28,7 @@
 #include <freerdp/freerdp.h>
 
 #include "remdesk_main.h"
+#include "remdesk_common.h"
 
 /**
  * Function description
@@ -36,104 +37,14 @@
  */
 static UINT remdesk_virtual_channel_write(RemdeskServerContext* context, wStream* s)
 {
-	BOOL status = 0;
+	const size_t len = Stream_Length(s);
+	WINPR_ASSERT(len <= UINT32_MAX);
 	ULONG BytesWritten = 0;
-	status = WTSVirtualChannelWrite(context->priv->ChannelHandle, Stream_BufferAs(s, char),
-	                                Stream_Length(s), &BytesWritten);
+	BOOL status = WTSVirtualChannelWrite(context->priv->ChannelHandle, Stream_BufferAs(s, char),
+	                                     (UINT32)len, &BytesWritten);
 	return (status) ? CHANNEL_RC_OK : ERROR_INTERNAL_ERROR;
 }
 
-/**
- * Function description
- *
- * @return 0 on success, otherwise a Win32 error code
- */
-static UINT remdesk_read_channel_header(wStream* s, REMDESK_CHANNEL_HEADER* header)
-{
-	UINT32 ChannelNameLen = 0;
-
-	if (!Stream_CheckAndLogRequiredLength(TAG, s, 8))
-		return CHANNEL_RC_NO_MEMORY;
-
-	Stream_Read_UINT32(s, ChannelNameLen);     /* ChannelNameLen (4 bytes) */
-	Stream_Read_UINT32(s, header->DataLength); /* DataLen (4 bytes) */
-
-	if (ChannelNameLen > 64)
-	{
-		WLog_ERR(TAG, "ChannelNameLen > 64!");
-		return ERROR_INVALID_DATA;
-	}
-
-	if ((ChannelNameLen % 2) != 0)
-	{
-		WLog_ERR(TAG, "(ChannelNameLen %% 2) != 0!");
-		return ERROR_INVALID_DATA;
-	}
-
-	if (Stream_Read_UTF16_String_As_UTF8_Buffer(s, ChannelNameLen / sizeof(WCHAR),
-	                                            header->ChannelName,
-	                                            ARRAYSIZE(header->ChannelName)) < 0)
-		return ERROR_INVALID_DATA;
-
-	return CHANNEL_RC_OK;
-}
-
-/**
- * Function description
- *
- * @return 0 on success, otherwise a Win32 error code
- */
-static UINT remdesk_write_channel_header(wStream* s, REMDESK_CHANNEL_HEADER* header)
-{
-	UINT32 ChannelNameLen = 0;
-	WCHAR ChannelNameW[32] = { 0 };
-
-	for (size_t index = 0; index < 32; index++)
-	{
-		ChannelNameW[index] = (WCHAR)header->ChannelName[index];
-	}
-
-	ChannelNameLen = (strnlen(header->ChannelName, sizeof(header->ChannelName)) + 1) * 2;
-	Stream_Write_UINT32(s, ChannelNameLen);        /* ChannelNameLen (4 bytes) */
-	Stream_Write_UINT32(s, header->DataLength);    /* DataLen (4 bytes) */
-	Stream_Write(s, ChannelNameW, ChannelNameLen); /* ChannelName (variable) */
-	return CHANNEL_RC_OK;
-}
-
-/**
- * Function description
- *
- * @return 0 on success, otherwise a Win32 error code
- */
-static UINT remdesk_write_ctl_header(wStream* s, REMDESK_CTL_HEADER* ctlHeader)
-{
-	UINT error = 0;
-
-	if ((error = remdesk_write_channel_header(s, (REMDESK_CHANNEL_HEADER*)ctlHeader)))
-	{
-		WLog_ERR(TAG, "remdesk_write_channel_header failed with error %" PRIu32 "!", error);
-		return error;
-	}
-
-	Stream_Write_UINT32(s, ctlHeader->msgType); /* msgType (4 bytes) */
-	return CHANNEL_RC_OK;
-}
-
-/**
- * Function description
- *
- * @return 0 on success, otherwise a Win32 error code
- */
-static UINT remdesk_prepare_ctl_header(REMDESK_CTL_HEADER* ctlHeader, UINT32 msgType,
-                                       UINT32 msgSize)
-{
-	ctlHeader->msgType = msgType;
-	(void)sprintf_s(ctlHeader->ch.ChannelName, ARRAYSIZE(ctlHeader->ch.ChannelName),
-	                REMDESK_CHANNEL_CTL_NAME);
-	ctlHeader->ch.DataLength = 4 + msgSize;
-	return CHANNEL_RC_OK;
-}
-
 /**
  * Function description
  *
@@ -256,8 +167,7 @@ static UINT remdesk_recv_ctl_version_info_pdu(RemdeskServerContext* context, wSt
 static UINT remdesk_recv_ctl_remote_control_desktop_pdu(RemdeskServerContext* context, wStream* s,
                                                         REMDESK_CHANNEL_HEADER* header)
 {
-	SSIZE_T cchStringW = 0;
-	SSIZE_T cbRaConnectionStringW = 0;
+	size_t cchStringW = 0;
 	REMDESK_CTL_REMOTE_CONTROL_DESKTOP_PDU pdu = { 0 };
 	UINT error = 0;
 	UINT32 msgLength = header->DataLength - 4;
@@ -274,7 +184,7 @@ static UINT remdesk_recv_ctl_remote_control_desktop_pdu(RemdeskServerContext* co
 		return ERROR_INVALID_DATA;
 
 	cchStringW++;
-	cbRaConnectionStringW = cchStringW * 2;
+	const size_t cbRaConnectionStringW = cchStringW * 2;
 	pdu.raConnectionString =
 	    ConvertWCharNToUtf8Alloc(raConnectionStringW, cbRaConnectionStringW / sizeof(WCHAR), NULL);
 	if (!pdu.raConnectionString)
@@ -297,31 +207,28 @@ static UINT remdesk_recv_ctl_remote_control_desktop_pdu(RemdeskServerContext* co
 static UINT remdesk_recv_ctl_authenticate_pdu(RemdeskServerContext* context, wStream* s,
                                               REMDESK_CHANNEL_HEADER* header)
 {
-	int cchStringW = 0;
-	UINT32 msgLength = 0;
-	int cbExpertBlobW = 0;
+	size_t cchTmpStringW = 0;
 	const WCHAR* expertBlobW = NULL;
-	int cbRaConnectionStringW = 0;
 	REMDESK_CTL_AUTHENTICATE_PDU pdu = { 0 };
-	msgLength = header->DataLength - 4;
+	UINT32 msgLength = header->DataLength - 4;
 	const WCHAR* pStringW = Stream_ConstPointer(s);
 	const WCHAR* raConnectionStringW = pStringW;
 
-	while ((msgLength > 0) && pStringW[cchStringW])
+	while ((msgLength > 0) && pStringW[cchTmpStringW])
 	{
 		msgLength -= 2;
-		cchStringW++;
+		cchTmpStringW++;
 	}
 
-	if (pStringW[cchStringW] || !cchStringW)
+	if (pStringW[cchTmpStringW] || !cchTmpStringW)
 		return ERROR_INVALID_DATA;
 
-	cchStringW++;
-	cbRaConnectionStringW = cchStringW * 2;
-	pStringW += cchStringW;
+	cchTmpStringW++;
+	const size_t cbRaConnectionStringW = cchTmpStringW * sizeof(WCHAR);
+	pStringW += cchTmpStringW;
 	expertBlobW = pStringW;
-	cchStringW = 0;
 
+	size_t cchStringW = 0;
 	while ((msgLength > 0) && pStringW[cchStringW])
 	{
 		msgLength -= 2;
@@ -332,7 +239,7 @@ static UINT remdesk_recv_ctl_authenticate_pdu(RemdeskServerContext* context, wSt
 		return ERROR_INVALID_DATA;
 
 	cchStringW++;
-	cbExpertBlobW = cchStringW * 2;
+	const size_t cbExpertBlobW = cchStringW * 2;
 	pdu.raConnectionString =
 	    ConvertWCharNToUtf8Alloc(raConnectionStringW, cbRaConnectionStringW / sizeof(WCHAR), NULL);
 	if (!pdu.raConnectionString)
@@ -359,26 +266,27 @@ static UINT remdesk_recv_ctl_authenticate_pdu(RemdeskServerContext* context, wSt
 static UINT remdesk_recv_ctl_verify_password_pdu(RemdeskServerContext* context, wStream* s,
                                                  REMDESK_CHANNEL_HEADER* header)
 {
-	SSIZE_T cbExpertBlobW = 0;
-	REMDESK_CTL_VERIFY_PASSWORD_PDU pdu;
-	UINT error = 0;
+	REMDESK_CTL_VERIFY_PASSWORD_PDU pdu = { 0 };
 
 	if (!Stream_CheckAndLogRequiredLength(TAG, s, 8))
 		return ERROR_INVALID_DATA;
 
 	const WCHAR* expertBlobW = Stream_ConstPointer(s);
-	cbExpertBlobW = header->DataLength - 4;
+	if (header->DataLength < 4)
+		return ERROR_INVALID_PARAMETER;
+
+	const size_t cbExpertBlobW = header->DataLength - 4;
 
 	pdu.expertBlob = ConvertWCharNToUtf8Alloc(expertBlobW, cbExpertBlobW / sizeof(WCHAR), NULL);
-	if (pdu.expertBlob)
+	if (!pdu.expertBlob)
 		return ERROR_INTERNAL_ERROR;
 
 	WLog_INFO(TAG, "ExpertBlob: %s", pdu.expertBlob);
 
-	if ((error = remdesk_send_ctl_result_pdu(context, 0)))
-		WLog_ERR(TAG, "remdesk_send_ctl_result_pdu failed with error %" PRIu32 "!", error);
+	// TODO: Callback?
 
-	return error;
+	free(pdu.expertBlob);
+	return remdesk_send_ctl_result_pdu(context, 0);
 }
 
 /**
@@ -549,7 +457,7 @@ static DWORD WINAPI remdesk_server_thread(LPVOID arg)
 	                           &BytesReturned) == TRUE)
 	{
 		if (BytesReturned == sizeof(HANDLE))
-			CopyMemory(&ChannelEvent, buffer, sizeof(HANDLE));
+			ChannelEvent = *(HANDLE*)buffer;
 
 		WTSFreeMemory(buffer);
 	}
@@ -595,8 +503,14 @@ static DWORD WINAPI remdesk_server_thread(LPVOID arg)
 			break;
 		}
 
+		const size_t len = Stream_Capacity(s);
+		if (len > UINT32_MAX)
+		{
+			error = ERROR_INTERNAL_ERROR;
+			break;
+		}
 		if (WTSVirtualChannelRead(context->priv->ChannelHandle, 0, Stream_BufferAs(s, char),
-		                          Stream_Capacity(s), &BytesReturned))
+		                          (UINT32)len, &BytesReturned))
 		{
 			if (BytesReturned)
 				Stream_Seek(s, BytesReturned);
