@@ -22,6 +22,8 @@
 #include <stdlib.h>
 #include <string.h>
 
+#include <winpr/assert.h>
+#include <winpr/cast.h>
 #include <winpr/wtypes.h>
 #include <winpr/sysinfo.h>
 #include <winpr/collections.h>
@@ -265,7 +267,8 @@ static void LIBUSB_CALL func_iso_callback(struct libusb_transfer* transfer)
 			Stream_SetPosition(user_data->data,
 			                   40); /* TS_URB_ISOCH_TRANSFER_RESULT IsoPacket offset */
 
-			for (int i = 0; i < transfer->num_iso_packets; i++)
+			for (uint32_t i = 0; i < WINPR_ASSERTING_INT_CAST(uint32_t, transfer->num_iso_packets);
+			     i++)
 			{
 				const UINT32 act_len = transfer->iso_packet_desc[i].actual_length;
 				Stream_Write_UINT32(user_data->data, index);
@@ -307,8 +310,8 @@ static void LIBUSB_CALL func_iso_callback(struct libusb_transfer* transfer)
 					const UINT32 RequestID = streamID & INTERFACE_ID_MASK;
 					user_data->cb(user_data->idev, user_data->callback, user_data->data,
 					              InterfaceId, user_data->noack, user_data->MessageId, RequestID,
-					              transfer->num_iso_packets, transfer->status,
-					              user_data->StartFrame, user_data->ErrorCount,
+					              WINPR_ASSERTING_INT_CAST(uint32_t, transfer->num_iso_packets),
+					              transfer->status, user_data->StartFrame, user_data->ErrorCount,
 					              user_data->OutputBufferSize);
 					user_data->data = NULL;
 				}
@@ -369,9 +372,10 @@ static void LIBUSB_CALL func_bulk_transfer_cb(struct libusb_transfer* transfer)
 		const UINT32 RequestID = streamID & INTERFACE_ID_MASK;
 
 		user_data->cb(user_data->idev, user_data->callback, user_data->data, InterfaceId,
-		              user_data->noack, user_data->MessageId, RequestID, transfer->num_iso_packets,
+		              user_data->noack, user_data->MessageId, RequestID,
+		              WINPR_ASSERTING_INT_CAST(uint32_t, transfer->num_iso_packets),
 		              transfer->status, user_data->StartFrame, user_data->ErrorCount,
-		              transfer->actual_length);
+		              WINPR_ASSERTING_INT_CAST(uint32_t, transfer->actual_length));
 		user_data->data = NULL;
 		ArrayList_Remove(list, transfer);
 	}
@@ -655,8 +659,9 @@ libusb_udev_complete_msconfig_setup(IUDEVICE* idev, MSUSB_CONFIG_DESCRIPTOR* MsC
 	 * ||  bus_number  |  dev_number  |      bConfigurationValue    ||
 	 * ---------------------------------------------------------------
 	 * ***********************/
-	MsConfig->ConfigurationHandle =
-	    MsConfig->bConfigurationValue | (pdev->bus_number << 24) | (pdev->dev_number << 16);
+	MsConfig->ConfigurationHandle = (uint32_t)MsConfig->bConfigurationValue |
+	                                ((uint32_t)pdev->bus_number << 24) |
+	                                (((uint32_t)pdev->dev_number << 16) & 0xFF0000);
 	MsInterfaces = MsConfig->MsInterfaces;
 
 	for (UINT32 inum = 0; inum < MsConfig->NumInterfaces; inum++)
@@ -672,10 +677,12 @@ libusb_udev_complete_msconfig_setup(IUDEVICE* idev, MSUSB_CONFIG_DESCRIPTOR* MsC
 		 * ||  bus_number  |  dev_number  |  altsetting  | interfaceNum ||
 		 * ---------------------------------------------------------------
 		 * ***********************/
-		MsInterface->InterfaceHandle = LibusbAltsetting->bInterfaceNumber |
-		                               (LibusbAltsetting->bAlternateSetting << 8) |
-		                               (pdev->dev_number << 16) | (pdev->bus_number << 24);
-		MsInterface->Length = 16 + (MsInterface->NumberOfPipes * 20);
+		MsInterface->InterfaceHandle =
+		    WINPR_ASSERTING_INT_CAST(UINT32, (LibusbAltsetting->bInterfaceNumber |
+		                                      (LibusbAltsetting->bAlternateSetting << 8) |
+		                                      (pdev->dev_number << 16) | (pdev->bus_number << 24)));
+		const size_t len = 16 + (MsInterface->NumberOfPipes * 20);
+		MsInterface->Length = WINPR_ASSERTING_INT_CAST(UINT16, len);
 		MsInterface->bInterfaceClass = LibusbAltsetting->bInterfaceClass;
 		MsInterface->bInterfaceSubClass = LibusbAltsetting->bInterfaceSubClass;
 		MsInterface->bInterfaceProtocol = LibusbAltsetting->bInterfaceProtocol;
@@ -695,10 +702,11 @@ libusb_udev_complete_msconfig_setup(IUDEVICE* idev, MSUSB_CONFIG_DESCRIPTOR* MsC
 			 * ||  bus_number  |  dev_number  |      bEndpointAddress       ||
 			 * ---------------------------------------------------------------
 			 * ***********************/
-			MsPipe->PipeHandle = LibusbEndpoint->bEndpointAddress | (pdev->dev_number << 16) |
-			                     (pdev->bus_number << 24);
+			MsPipe->PipeHandle = LibusbEndpoint->bEndpointAddress |
+			                     (((uint32_t)pdev->dev_number << 16) & 0xFF0000) |
+			                     (((uint32_t)pdev->bus_number << 24) & 0xFF000000);
 			/* count endpoint max packet size */
-			int max = LibusbEndpoint->wMaxPacketSize & 0x07ff;
+			unsigned max = LibusbEndpoint->wMaxPacketSize & 0x07ff;
 			BYTE attr = LibusbEndpoint->bmAttributes;
 
 			if ((attr & 0x3) == 1 || (attr & 0x3) == 3)
@@ -706,7 +714,7 @@ libusb_udev_complete_msconfig_setup(IUDEVICE* idev, MSUSB_CONFIG_DESCRIPTOR* MsC
 				max *= (1 + ((LibusbEndpoint->wMaxPacketSize >> 11) & 3));
 			}
 
-			MsPipe->MaximumPacketSize = max;
+			MsPipe->MaximumPacketSize = WINPR_ASSERTING_INT_CAST(uint16_t, max);
 			MsPipe->bEndpointAddress = LibusbEndpoint->bEndpointAddress;
 			MsPipe->bInterval = LibusbEndpoint->bInterval;
 			MsPipe->PipeType = attr & 0x3;
@@ -714,7 +722,7 @@ libusb_udev_complete_msconfig_setup(IUDEVICE* idev, MSUSB_CONFIG_DESCRIPTOR* MsC
 		}
 	}
 
-	MsConfig->MsOutSize = MsOutSize;
+	MsConfig->MsOutSize = WINPR_ASSERTING_INT_CAST(int, MsOutSize);
 	MsConfig->InitCompleted = 1;
 
 	/* replace device's MsConfig */
@@ -756,7 +764,8 @@ static int libusb_udev_select_configuration(IUDEVICE* idev, UINT32 bConfiguratio
 	if (bConfigurationValue == 0)
 		ret = libusb_set_configuration(libusb_handle, -1);
 	else
-		ret = libusb_set_configuration(libusb_handle, bConfigurationValue);
+		ret = libusb_set_configuration(libusb_handle,
+		                               WINPR_ASSERTING_INT_CAST(int, bConfigurationValue));
 
 	if (log_libusb_result(urbdrc->log, WLOG_ERROR, "libusb_set_configuration", ret))
 	{
@@ -784,6 +793,7 @@ static int libusb_udev_control_pipe_request(IUDEVICE* idev, UINT32 RequestId,
 	int error = 0;
 	UDEVICE* pdev = (UDEVICE*)idev;
 
+	WINPR_ASSERT(EndpointAddress <= UINT8_MAX);
 	/*
 	pdev->request_queue->register_request(pdev->request_queue, RequestId, NULL, 0);
 	*/
@@ -794,14 +804,19 @@ static int libusb_udev_control_pipe_request(IUDEVICE* idev, UINT32 RequestId,
 			idev->cancel_all_transfer_request(idev);
 			// dummy_wait_s_obj(1);
 			/** set feature to ep (set halt)*/
-			error = libusb_control_transfer(
-			    pdev->libusb_handle, LIBUSB_ENDPOINT_OUT | LIBUSB_RECIPIENT_ENDPOINT,
-			    LIBUSB_REQUEST_SET_FEATURE, ENDPOINT_HALT, EndpointAddress, NULL, 0, 1000);
+			/*
+			uint8_t request_type, uint8_t bRequest,
+			*/
+			error = libusb_control_transfer(pdev->libusb_handle,
+			                                (uint8_t)LIBUSB_ENDPOINT_OUT |
+			                                    (uint8_t)LIBUSB_RECIPIENT_ENDPOINT,
+			                                LIBUSB_REQUEST_SET_FEATURE, ENDPOINT_HALT,
+			                                (uint16_t)EndpointAddress, NULL, 0, 1000);
 			break;
 
 		case PIPE_RESET:
 			idev->cancel_all_transfer_request(idev);
-			error = libusb_clear_halt(pdev->libusb_handle, EndpointAddress);
+			error = libusb_clear_halt(pdev->libusb_handle, (uint8_t)EndpointAddress);
 			// func_set_usbd_status(pdev, UsbdStatus, error);
 			break;
 
@@ -945,7 +960,8 @@ static int libusb_udev_os_feature_descriptor_request(IUDEVICE* idev, UINT32 Requ
 		const BYTE bMS_Vendorcode = ms_string_desc[16];
 		/** get os descriptor */
 		error = libusb_control_transfer(
-		    pdev->libusb_handle, LIBUSB_ENDPOINT_IN | LIBUSB_REQUEST_TYPE_VENDOR | Recipient,
+		    pdev->libusb_handle,
+		    (uint8_t)LIBUSB_ENDPOINT_IN | (uint8_t)LIBUSB_REQUEST_TYPE_VENDOR | Recipient,
 		    bMS_Vendorcode, (UINT16)((InterfaceNumber << 8) | Ms_PageIndex), Ms_featureDescIndex,
 		    Buffer, (UINT16)*BufferSize, Timeout);
 		log_libusb_result(pdev->urbdrc->log, WLOG_DEBUG, "libusb_control_transfer", error);
@@ -1185,7 +1201,8 @@ static int libusb_udev_query_device_port_status(IUDEVICE* idev, UINT32* UsbdStat
 	{
 		ret = idev->control_transfer(
 		    idev, 0xffff, 0, 0,
-		    LIBUSB_ENDPOINT_IN | LIBUSB_REQUEST_TYPE_CLASS | LIBUSB_RECIPIENT_OTHER,
+		    (uint8_t)LIBUSB_ENDPOINT_IN | (uint8_t)LIBUSB_REQUEST_TYPE_CLASS |
+		        (uint8_t)LIBUSB_RECIPIENT_OTHER,
 		    LIBUSB_REQUEST_GET_STATUS, 0, pdev->port_number, UsbdStatus, BufferSize, Buffer, 1000);
 
 		if (log_libusb_result(urbdrc->log, WLOG_DEBUG, "libusb_control_transfer", ret))
@@ -1251,9 +1268,10 @@ static int libusb_udev_isoch_transfer(IUDEVICE* idev, GENERIC_CHANNEL_CALLBACK*
 	}
 
 	/**  process URB_FUNCTION_IOSCH_TRANSFER */
-	libusb_fill_iso_transfer(iso_transfer, pdev->libusb_handle, EndpointAddress,
-	                         Stream_Pointer(user_data->data), BufferSize, NumberOfPackets,
-	                         func_iso_callback, user_data, Timeout);
+	libusb_fill_iso_transfer(
+	    iso_transfer, pdev->libusb_handle, WINPR_ASSERTING_INT_CAST(uint8_t, EndpointAddress),
+	    Stream_Pointer(user_data->data), WINPR_ASSERTING_INT_CAST(int, BufferSize),
+	    WINPR_ASSERTING_INT_CAST(int, NumberOfPackets), func_iso_callback, user_data, Timeout);
 	set_stream_id_for_buffer(iso_transfer, streamID);
 	libusb_set_iso_packet_lengths(iso_transfer, iso_packet_size);
 
@@ -1354,16 +1372,18 @@ static int libusb_udev_bulk_or_interrupt_transfer(IUDEVICE* idev,
 	{
 		case BULK_TRANSFER:
 			/** Bulk Transfer */
-			libusb_fill_bulk_transfer(transfer, pdev->libusb_handle, EndpointAddress,
-			                          Stream_Pointer(user_data->data), BufferSize,
-			                          func_bulk_transfer_cb, user_data, Timeout);
+			libusb_fill_bulk_transfer(
+			    transfer, pdev->libusb_handle, WINPR_ASSERTING_INT_CAST(uint8_t, EndpointAddress),
+			    Stream_Pointer(user_data->data), WINPR_ASSERTING_INT_CAST(int, BufferSize),
+			    func_bulk_transfer_cb, user_data, Timeout);
 			break;
 
 		case INTERRUPT_TRANSFER:
 			/**  Interrupt Transfer */
-			libusb_fill_interrupt_transfer(transfer, pdev->libusb_handle, EndpointAddress,
-			                               Stream_Pointer(user_data->data), BufferSize,
-			                               func_bulk_transfer_cb, user_data, Timeout);
+			libusb_fill_interrupt_transfer(
+			    transfer, pdev->libusb_handle, WINPR_ASSERTING_INT_CAST(uint8_t, EndpointAddress),
+			    Stream_Pointer(user_data->data), WINPR_ASSERTING_INT_CAST(int, BufferSize),
+			    func_bulk_transfer_cb, user_data, Timeout);
 			break;
 
 		default:
@@ -1461,7 +1481,7 @@ BASIC_STATE_FUNC_DEFINED(channelID, UINT32)
 BASIC_STATE_FUNC_DEFINED(ReqCompletion, UINT32)
 BASIC_STATE_FUNC_DEFINED(bus_number, BYTE)
 BASIC_STATE_FUNC_DEFINED(dev_number, BYTE)
-BASIC_STATE_FUNC_DEFINED(port_number, int)
+BASIC_STATE_FUNC_DEFINED(port_number, UINT8)
 BASIC_STATE_FUNC_DEFINED(MsConfig, MSUSB_CONFIG_DESCRIPTOR*)
 
 BASIC_POINT_FUNC_DEFINED(udev, void*)
@@ -1608,9 +1628,10 @@ static int udev_get_device_handle(URBDRC_PLUGIN* urbdrc, libusb_context* ctx, UD
 
 			pdev->port_number = port_numbers[(error - 1)];
 			error = 0;
-			WLog_Print(urbdrc->log, WLOG_DEBUG, "  Port: %d", pdev->port_number);
+			WLog_Print(urbdrc->log, WLOG_DEBUG, "  Port: %" PRIu8, pdev->port_number);
 			/* gen device path */
-			(void)sprintf(pdev->path, "%" PRIu16 "-%d", bus_number, pdev->port_number);
+			(void)_snprintf(pdev->path, sizeof(pdev->path), "%" PRIu16 "-%d", bus_number,
+			                pdev->port_number);
 
 			WLog_Print(urbdrc->log, WLOG_DEBUG, "  DevPath: %s", pdev->path);
 		}
@@ -1638,7 +1659,7 @@ static int udev_get_hub_handle(URBDRC_PLUGIN* urbdrc, libusb_context* ctx, UDEVI
 		LIBUSB_DEVICE* dev = libusb_list[i];
 
 		if ((bus_number != libusb_get_bus_number(dev)) ||
-		    (1 != libusb_get_device_address(dev))) /* Root hub allways first on bus. */
+		    (1 != libusb_get_device_address(dev))) /* Root hub always first on bus. */
 			libusb_unref_device(dev);
 		else
 		{
