@@ -32,28 +32,55 @@
 #define TAG CLIENT_TAG("sdl.cliprdr")
 
 #define mime_text_plain "text/plain"
-#define mime_text_utf8 mime_text_plain ";charset=utf-8"
+// NOLINTNEXTLINE(bugprone-suspicious-missing-comma)
+const char mime_text_utf8[] = mime_text_plain ";charset=utf-8";
 
-static const std::vector<const char*> mime_text = { mime_text_plain, mime_text_utf8, "UTF8_STRING",
-	                                                "COMPOUND_TEXT", "TEXT",         "STRING" };
+static const std::vector<const char*>& s_mime_text()
+{
+	static std::vector<const char*> values;
+	if (values.empty())
+	{
+		values = std::vector<const char*>(
+		    { mime_text_plain, mime_text_utf8, "UTF8_STRING", "COMPOUND_TEXT", "TEXT", "STRING" });
+	}
+	return values;
+}
 
-static const char mime_png[] = "image/png";
-static const char mime_webp[] = "image/webp";
-static const char mime_jpg[] = "image/jpeg";
-static const char mime_tiff[] = "image/tiff";
-static const char mime_uri_list[] = "text/uri-list";
-static const char mime_html[] = "text/html";
+static const char s_mime_png[] = "image/png";
+static const char s_mime_webp[] = "image/webp";
+static const char s_mime_jpg[] = "image/jpeg";
+static const char s_mime_tiff[] = "image/tiff";
+static const char s_mime_uri_list[] = "text/uri-list";
+static const char s_mime_html[] = "text/html";
 
 #define BMP_MIME_LIST "image/bmp", "image/x-bmp", "image/x-MS-bmp", "image/x-win-bitmap"
-static const std::vector<const char*> mime_bitmap = { BMP_MIME_LIST };
-static const std::vector<const char*> mime_image = { mime_png, mime_webp, mime_jpg, mime_tiff,
-	                                                 BMP_MIME_LIST };
 
-static const char mime_gnome_copied_files[] = "x-special/gnome-copied-files";
-static const char mime_mate_copied_files[] = "x-special/mate-copied-files";
+static const std::vector<const char*>& s_mime_bitmap()
+{
+	static std::vector<const char*> values;
+	if (values.empty())
+	{
+		values = std::vector<const char*>({ BMP_MIME_LIST });
+	}
+	return values;
+}
 
-static const char* type_HtmlFormat = "HTML Format";
-static const char* type_FileGroupDescriptorW = "FileGroupDescriptorW";
+static const std::vector<const char*>& s_mime_image()
+{
+	static std::vector<const char*> values;
+	if (values.empty())
+	{
+		values = std::vector<const char*>(
+		    { s_mime_png, s_mime_webp, s_mime_jpg, s_mime_tiff, BMP_MIME_LIST });
+	}
+	return values;
+}
+
+static const char s_mime_gnome_copied_files[] = "x-special/gnome-copied-files";
+static const char s_mime_mate_copied_files[] = "x-special/mate-copied-files";
+
+static const char* s_type_HtmlFormat = "HTML Format";
+static const char* s_type_FileGroupDescriptorW = "FileGroupDescriptorW";
 
 class ClipboardLockGuard
 {
@@ -132,11 +159,22 @@ BOOL sdlClip::uninit(CliprdrClientContext* clip)
 bool sdlClip::handle_update(const SDL_ClipboardEvent& ev)
 {
 	if (!_ctx || !_sync || ev.owner)
+	{
+		/* TODO: Hack to identify our own updates */
+		if (ev.owner && (ev.reserved == 0x42))
+		{
+			_cache_data.clear();
+			auto rc =
+			    SDL_SetClipboardData(sdlClip::ClipDataCb, sdlClip::ClipCleanCb, this, ev.mime_types,
+			                         WINPR_ASSERTING_INT_CAST(size_t, ev.num_mime_types));
+			_current_mimetypes.clear();
+			return rc;
+		}
 		return true;
+	}
 
 	clearServerFormats();
 
-	std::string mime_uri_list = "text/uri-list";
 	std::string mime_html = "text/html";
 
 	std::vector<std::string> mime_bitmap = { "image/bmp", "image/x-bmp", "image/x-MS-bmp",
@@ -150,7 +188,11 @@ bool sdlClip::handle_update(const SDL_ClipboardEvent& ev)
 	std::vector<std::string> clientFormatNames;
 	std::vector<CLIPRDR_FORMAT> clientFormats;
 
-	size_t nformats = ev.n_mime_types;
+#if SDL_VERSION_ATLEAST(3, 1, 8)
+	size_t nformats = WINPR_ASSERTING_INT_CAST(size_t, ev.num_mime_types);
+#else
+	size_t nformats = WINPR_ASSERTING_INT_CAST(size_t, ev.n_mime_types);
+#endif
 	const char** clipboard_mime_formats = ev.mime_types;
 
 	WLog_Print(_log, WLOG_TRACE, "SDL has %d formats", nformats);
@@ -163,7 +205,8 @@ bool sdlClip::handle_update(const SDL_ClipboardEvent& ev)
 		std::string local_mime = clipboard_mime_formats[i];
 		WLog_Print(_log, WLOG_TRACE, " - %s", local_mime.c_str());
 
-		if (std::find(mime_text.begin(), mime_text.end(), local_mime) != mime_text.end())
+		if (std::find(s_mime_text().begin(), s_mime_text().end(), local_mime) !=
+		    s_mime_text().end())
 		{
 			/* text formats */
 			if (!textPushed)
@@ -176,7 +219,7 @@ bool sdlClip::handle_update(const SDL_ClipboardEvent& ev)
 		}
 		else if (local_mime == mime_html)
 			/* html */
-			clientFormatNames.emplace_back(type_HtmlFormat);
+			clientFormatNames.emplace_back(s_type_HtmlFormat);
 		else if (std::find(mime_bitmap.begin(), mime_bitmap.end(), local_mime) != mime_bitmap.end())
 		{
 			/* image formats */
@@ -207,9 +250,9 @@ bool sdlClip::handle_update(const SDL_ClipboardEvent& ev)
 	clientFormats.erase(u, clientFormats.end());
 
 	const CLIPRDR_FORMAT_LIST formatList = {
-		.common = { .msgType = CB_FORMAT_LIST, .msgFlags = 0 },
-		.numFormats = static_cast<UINT32>(clientFormats.size()),
-		.formats = clientFormats.data(),
+		{ CB_FORMAT_LIST, 0, 0 },
+		static_cast<UINT32>(clientFormats.size()),
+		clientFormats.data(),
 	};
 
 	WLog_Print(_log, WLOG_TRACE,
@@ -242,8 +285,7 @@ UINT sdlClip::MonitorReady(CliprdrClientContext* context, const CLIPRDR_MONITOR_
 		return ret;
 
 	clipboard->_sync = true;
-	SDL_ClipboardEvent ev = { SDL_EVENT_CLIPBOARD_UPDATE, 0, 0, false, 0, nullptr };
-	if (!clipboard->handle_update(ev))
+	if (!sdl_push_user_event(SDL_EVENT_CLIPBOARD_UPDATE))
 		return ERROR_INTERNAL_ERROR;
 
 	return CHANNEL_RC_OK;
@@ -252,14 +294,11 @@ UINT sdlClip::MonitorReady(CliprdrClientContext* context, const CLIPRDR_MONITOR_
 UINT sdlClip::SendClientCapabilities()
 {
 	CLIPRDR_GENERAL_CAPABILITY_SET generalCapabilitySet = {
-		.capabilitySetType = CB_CAPSTYPE_GENERAL,
-		.capabilitySetLength = 12,
-		.version = CB_CAPS_VERSION_2,
-		.generalFlags = CB_USE_LONG_FORMAT_NAMES | cliprdr_file_context_current_flags(_file)
+		CB_CAPSTYPE_GENERAL, 12, CB_CAPS_VERSION_2,
+		CB_USE_LONG_FORMAT_NAMES | cliprdr_file_context_current_flags(_file)
 	};
-	CLIPRDR_CAPABILITIES capabilities = {
-		.cCapabilitiesSets = 1,
-		.capabilitySets = reinterpret_cast<CLIPRDR_CAPABILITY_SET*>(&generalCapabilitySet)
+	const CLIPRDR_CAPABILITIES capabilities = {
+		{ CB_TYPE_NONE, 0, 0 }, 1, reinterpret_cast<CLIPRDR_CAPABILITY_SET*>(&generalCapabilitySet)
 	};
 
 	WINPR_ASSERT(_ctx);
@@ -270,15 +309,15 @@ UINT sdlClip::SendClientCapabilities()
 void sdlClip::clearServerFormats()
 {
 	_serverFormats.clear();
+	_cache_data.clear();
 	cliprdr_file_context_clear(_file);
 }
 
 UINT sdlClip::SendFormatListResponse(BOOL status)
 {
 	const CLIPRDR_FORMAT_LIST_RESPONSE formatListResponse = {
-		.common = { .msgType = CB_FORMAT_LIST_RESPONSE,
-		            .msgFlags = static_cast<UINT16>(status ? CB_RESPONSE_OK : CB_RESPONSE_FAIL),
-		            .dataLen = 0 }
+		{ CB_FORMAT_LIST_RESPONSE, static_cast<UINT16>(status ? CB_RESPONSE_OK : CB_RESPONSE_FAIL),
+		  0 }
 	};
 	WINPR_ASSERT(_ctx);
 	WINPR_ASSERT(_ctx->ClientFormatListResponse);
@@ -303,9 +342,9 @@ UINT sdlClip::SendDataResponse(const BYTE* data, size_t size)
 
 UINT sdlClip::SendDataRequest(uint32_t formatID, const std::string& mime)
 {
-	CLIPRDR_FORMAT_DATA_REQUEST request = { .requestedFormatId = formatID };
+	const CLIPRDR_FORMAT_DATA_REQUEST request = { { CB_TYPE_NONE, 0, 0 }, formatID };
 
-	_request_queue.push({ formatID, mime });
+	_request_queue.emplace(formatID, mime);
 
 	WINPR_ASSERT(_ctx);
 	WINPR_ASSERT(_ctx->ClientFormatDataRequest);
@@ -338,9 +377,9 @@ uint32_t sdlClip::serverIdForMime(const std::string& mime)
 {
 	std::string cmp = mime;
 	if (mime_is_html(mime))
-		cmp = type_HtmlFormat;
+		cmp = s_type_HtmlFormat;
 	if (mime_is_file(mime))
-		cmp = type_FileGroupDescriptorW;
+		cmp = s_type_FileGroupDescriptorW;
 
 	for (auto& format : _serverFormats)
 	{
@@ -417,12 +456,12 @@ UINT sdlClip::ReceiveServerFormatList(CliprdrClientContext* context,
 
 		if (format->formatName)
 		{
-			if (strcmp(format->formatName, type_HtmlFormat) == 0)
+			if (strcmp(format->formatName, s_type_HtmlFormat) == 0)
 			{
 				text = TRUE;
 				html = TRUE;
 			}
-			else if (strcmp(format->formatName, type_FileGroupDescriptorW) == 0)
+			else if (strcmp(format->formatName, s_type_FileGroupDescriptorW) == 0)
 			{
 				file = TRUE;
 				text = TRUE;
@@ -448,29 +487,39 @@ UINT sdlClip::ReceiveServerFormatList(CliprdrClientContext* context,
 		}
 	}
 
-	std::vector<const char*> mimetypes;
+	clipboard->_current_mimetypes.clear();
 	if (text)
 	{
-		mimetypes.insert(mimetypes.end(), mime_text.begin(), mime_text.end());
+		clipboard->_current_mimetypes.insert(clipboard->_current_mimetypes.end(),
+		                                     s_mime_text().begin(), s_mime_text().end());
 	}
 	if (image)
 	{
-		mimetypes.insert(mimetypes.end(), mime_bitmap.begin(), mime_bitmap.end());
-		mimetypes.insert(mimetypes.end(), mime_image.begin(), mime_image.end());
+		clipboard->_current_mimetypes.insert(clipboard->_current_mimetypes.end(),
+		                                     s_mime_bitmap().begin(), s_mime_bitmap().end());
+		clipboard->_current_mimetypes.insert(clipboard->_current_mimetypes.end(),
+		                                     s_mime_image().begin(), s_mime_image().end());
 	}
 	if (html)
 	{
-		mimetypes.push_back(mime_html);
+		clipboard->_current_mimetypes.push_back(s_mime_html);
 	}
 	if (file)
 	{
-		mimetypes.push_back(mime_uri_list);
-		mimetypes.push_back(mime_gnome_copied_files);
-		mimetypes.push_back(mime_mate_copied_files);
+		clipboard->_current_mimetypes.push_back(s_mime_uri_list);
+		clipboard->_current_mimetypes.push_back(s_mime_gnome_copied_files);
+		clipboard->_current_mimetypes.push_back(s_mime_mate_copied_files);
 	}
 
-	const bool rc = SDL_SetClipboardData(sdlClip::ClipDataCb, sdlClip::ClipCleanCb, clipboard,
-	                                     mimetypes.data(), mimetypes.size());
+	SDL_Event ev = { SDL_EVENT_CLIPBOARD_UPDATE };
+	ev.clipboard.owner = true;
+	ev.clipboard.num_mime_types =
+	    WINPR_ASSERTING_INT_CAST(Sint32, clipboard->_current_mimetypes.size());
+	ev.clipboard.mime_types = clipboard->_current_mimetypes.data();
+
+	/* TODO: Hack to identify our own updates */
+	ev.clipboard.reserved = 0x42;
+	auto rc = (SDL_PushEvent(&ev) == 1);
 	return clipboard->SendFormatListResponse(rc);
 }
 
@@ -504,8 +553,9 @@ std::shared_ptr<BYTE> sdlClip::ReceiveFormatDataRequestHandle(
 	ClipboardLockGuard give_me_a_name(clipboard->_system);
 	std::lock_guard<CriticalSection> lock(clipboard->_lock);
 
-	const UINT32 fileFormatId = ClipboardGetFormatId(clipboard->_system, type_FileGroupDescriptorW);
-	const UINT32 htmlFormatId = ClipboardGetFormatId(clipboard->_system, type_HtmlFormat);
+	const UINT32 fileFormatId =
+	    ClipboardGetFormatId(clipboard->_system, s_type_FileGroupDescriptorW);
+	const UINT32 htmlFormatId = ClipboardGetFormatId(clipboard->_system, s_type_HtmlFormat);
 
 	switch (formatId)
 	{
@@ -518,23 +568,23 @@ std::shared_ptr<BYTE> sdlClip::ReceiveFormatDataRequestHandle(
 
 		case CF_DIB:
 		case CF_DIBV5:
-			mime = mime_bitmap[0];
+			mime = s_mime_bitmap()[0];
 			break;
 
 		case CF_TIFF:
-			mime = mime_tiff;
+			mime = s_mime_tiff;
 			break;
 
 		default:
 			if (formatId == fileFormatId)
 			{
-				localFormatId = ClipboardGetFormatId(clipboard->_system, mime_uri_list);
-				mime = mime_uri_list;
+				localFormatId = ClipboardGetFormatId(clipboard->_system, s_mime_uri_list);
+				mime = s_mime_uri_list;
 			}
 			else if (formatId == htmlFormatId)
 			{
-				localFormatId = ClipboardGetFormatId(clipboard->_system, mime_html);
-				mime = mime_html;
+				localFormatId = ClipboardGetFormatId(clipboard->_system, s_mime_html);
+				mime = s_mime_html;
 			}
 			else
 				return data;
@@ -658,18 +708,18 @@ UINT sdlClip::ReceiveFormatDataResponse(CliprdrClientContext* context,
 				auto name = clipboard->getServerFormat(request.format());
 				if (!name.empty())
 				{
-					if (name == type_FileGroupDescriptorW)
+					if (name == s_type_FileGroupDescriptorW)
 					{
 						srcFormatId =
-						    ClipboardGetFormatId(clipboard->_system, type_FileGroupDescriptorW);
+						    ClipboardGetFormatId(clipboard->_system, s_type_FileGroupDescriptorW);
 
 						if (!cliprdr_file_context_update_server_data(
 						        clipboard->_file, clipboard->_system, data, size))
 							return ERROR_INTERNAL_ERROR;
 					}
-					else if (name == type_HtmlFormat)
+					else if (name == s_type_HtmlFormat)
 					{
-						srcFormatId = ClipboardGetFormatId(clipboard->_system, type_HtmlFormat);
+						srcFormatId = ClipboardGetFormatId(clipboard->_system, s_type_HtmlFormat);
 					}
 				}
 			}
@@ -713,6 +763,8 @@ const void* sdlClip::ClipDataCb(void* userdata, const char* mime_type, size_t* s
 		}
 
 		uint32_t formatID = clip->serverIdForMime(mime_type);
+		WLog_Print(clip->_log, WLOG_INFO, "requesting format %s [0x%08" PRIx32 "]", mime_type,
+		           formatID);
 		if (clip->SendDataRequest(formatID, mime_type))
 			return nullptr;
 	}
@@ -720,16 +772,8 @@ const void* sdlClip::ClipDataCb(void* userdata, const char* mime_type, size_t* s
 	{
 		HANDLE hdl[2] = { freerdp_abort_event(clip->_sdl->context()), clip->_event };
 
-		// Unlock the sdl->critical lock or we'll deadlock with the FreeRDP thread
-		// when it pushes events (like end_paint).
-		// we can safely do that here as we're called from the SDL thread
-		SdlContext* sdl = clip->_sdl;
-		sdl->critical.unlock();
-
 		DWORD status = WaitForMultipleObjects(ARRAYSIZE(hdl), hdl, FALSE, 10 * 1000);
 
-		sdl->critical.lock();
-
 		if (status != WAIT_OBJECT_0 + 1)
 		{
 			std::lock_guard<CriticalSection> lock(clip->_lock);
@@ -749,7 +793,7 @@ const void* sdlClip::ClipDataCb(void* userdata, const char* mime_type, size_t* s
 		auto request = clip->_request_queue.front();
 		clip->_request_queue.pop();
 
-		if (!clip->_request_queue.size())
+		if (clip->_request_queue.empty())
 			(void)ResetEvent(clip->_event);
 
 		if (request.success())
@@ -779,25 +823,25 @@ void sdlClip::ClipCleanCb(void* userdata)
 	ClipboardLockGuard give_me_a_name(clip->_system);
 	std::lock_guard<CriticalSection> lock(clip->_lock);
 	ClipboardEmpty(clip->_system);
-	clip->_cache_data.clear();
 }
 
 bool sdlClip::mime_is_file(const std::string& mime)
 {
-	if (strncmp(mime_uri_list, mime.c_str(), sizeof(mime_uri_list)) == 0)
+	if (strncmp(s_mime_uri_list, mime.c_str(), sizeof(s_mime_uri_list)) == 0)
 		return true;
-	if (strncmp(mime_gnome_copied_files, mime.c_str(), sizeof(mime_gnome_copied_files)) == 0)
+	if (strncmp(s_mime_gnome_copied_files, mime.c_str(), sizeof(s_mime_gnome_copied_files)) == 0)
 		return true;
-	if (strncmp(mime_mate_copied_files, mime.c_str(), sizeof(mime_mate_copied_files)) == 0)
+	if (strncmp(s_mime_mate_copied_files, mime.c_str(), sizeof(s_mime_mate_copied_files)) == 0)
 		return true;
 	return false;
 }
 
 bool sdlClip::mime_is_text(const std::string& mime)
 {
-	for (size_t x = 0; x < ARRAYSIZE(mime_text); x++)
+	for (const auto& tmime : s_mime_text())
 	{
-		if (mime == mime_text[x])
+		assert(tmime != nullptr);
+		if (mime == tmime)
 			return true;
 	}
 
@@ -806,9 +850,10 @@ bool sdlClip::mime_is_text(const std::string& mime)
 
 bool sdlClip::mime_is_image(const std::string& mime)
 {
-	for (size_t x = 0; x < ARRAYSIZE(mime_image); x++)
+	for (const auto& imime : s_mime_image())
 	{
-		if (mime == mime_image[x])
+		assert(imime != nullptr);
+		if (mime == imime)
 			return true;
 	}
 
@@ -817,7 +862,7 @@ bool sdlClip::mime_is_image(const std::string& mime)
 
 bool sdlClip::mime_is_html(const std::string& mime)
 {
-	return mime.compare(mime_html) == 0;
+	return mime.compare(s_mime_html) == 0;
 }
 
 ClipRequest::ClipRequest(UINT32 format, const std::string& mime)
