@@ -387,8 +387,10 @@ static BOOL xf_event_Expose(xfContext* xfc, const XExposeEvent* event, BOOL app)
 	{
 		x = 0;
 		y = 0;
-		w = freerdp_settings_get_uint32(settings, FreeRDP_DesktopWidth);
-		h = freerdp_settings_get_uint32(settings, FreeRDP_DesktopHeight);
+		w = WINPR_ASSERTING_INT_CAST(int,
+		                             freerdp_settings_get_uint32(settings, FreeRDP_DesktopWidth));
+		h = WINPR_ASSERTING_INT_CAST(int,
+		                             freerdp_settings_get_uint32(settings, FreeRDP_DesktopHeight));
 	}
 	else
 	{
@@ -402,7 +404,9 @@ static BOOL xf_event_Expose(xfContext* xfc, const XExposeEvent* event, BOOL app)
 	{
 		if (xfc->common.context.gdi->gfx)
 		{
-			xf_OutputExpose(xfc, x, y, w, h);
+			xf_OutputExpose(
+			    xfc, WINPR_ASSERTING_INT_CAST(uint32_t, x), WINPR_ASSERTING_INT_CAST(uint32_t, y),
+			    WINPR_ASSERTING_INT_CAST(uint32_t, w), WINPR_ASSERTING_INT_CAST(uint32_t, h));
 			return TRUE;
 		}
 		xf_draw_screen(xfc, x, y, w, h);
@@ -485,7 +489,8 @@ static BOOL xf_event_MotionNotify(xfContext* xfc, const XMotionEvent* event, BOO
 	    (xfc->common.mouse_grabbed && freerdp_client_use_relative_mouse_events(&xfc->common)))
 		return TRUE;
 
-	return xf_generic_MotionNotify(xfc, event->x, event->y, event->state, event->window, app);
+	return xf_generic_MotionNotify(xfc, event->x, event->y,
+	                               WINPR_ASSERTING_INT_CAST(int, event->state), event->window, app);
 }
 
 BOOL xf_generic_ButtonEvent(xfContext* xfc, int x, int y, int button, Window window, BOOL app,
@@ -495,12 +500,14 @@ BOOL xf_generic_ButtonEvent(xfContext* xfc, int x, int y, int button, Window win
 	Window childWindow = None;
 
 	WINPR_ASSERT(xfc);
+	if (button < 0)
+		return FALSE;
 
 	for (size_t i = 0; i < ARRAYSIZE(xfc->button_map); i++)
 	{
 		const button_map* cur = &xfc->button_map[i];
 
-		if (cur->button == button)
+		if (cur->button == (UINT32)button)
 		{
 			flags = cur->flags;
 			break;
@@ -591,7 +598,9 @@ static BOOL xf_event_ButtonPress(xfContext* xfc, const XButtonEvent* event, BOOL
 	if (xfc->xi_event ||
 	    (xfc->common.mouse_grabbed && freerdp_client_use_relative_mouse_events(&xfc->common)))
 		return TRUE;
-	return xf_generic_ButtonEvent(xfc, event->x, event->y, event->button, event->window, app, TRUE);
+	return xf_generic_ButtonEvent(xfc, event->x, event->y,
+	                              WINPR_ASSERTING_INT_CAST(int, event->button), event->window, app,
+	                              TRUE);
 }
 
 static BOOL xf_event_ButtonRelease(xfContext* xfc, const XButtonEvent* event, BOOL app)
@@ -601,7 +610,8 @@ static BOOL xf_event_ButtonRelease(xfContext* xfc, const XButtonEvent* event, BO
 	if (xfc->xi_event ||
 	    (xfc->common.mouse_grabbed && freerdp_client_use_relative_mouse_events(&xfc->common)))
 		return TRUE;
-	return xf_generic_ButtonEvent(xfc, event->x, event->y, event->button, event->window, app,
+	return xf_generic_ButtonEvent(xfc, event->x, event->y,
+	                              WINPR_ASSERTING_INT_CAST(int, event->button), event->window, app,
 	                              FALSE);
 }
 
@@ -735,7 +745,7 @@ static BOOL xf_event_ClientMessage(xfContext* xfc, const XClientMessageEvent* ev
 			xfAppWindow* appWindow = xf_AppWindowFromX11Window(xfc, event->window);
 
 			if (appWindow)
-				xf_rail_send_client_system_command(xfc, appWindow->windowId, SC_CLOSE);
+				return xf_rail_send_client_system_command(xfc, appWindow->windowId, SC_CLOSE);
 
 			return TRUE;
 		}
@@ -833,14 +843,19 @@ static BOOL xf_event_ConfigureNotify(xfContext* xfc, const XConfigureEvent* even
 			{
 				xfc->scaledWidth = xfc->window->width;
 				xfc->scaledHeight = xfc->window->height;
-				xf_draw_screen(xfc, 0, 0,
-				               freerdp_settings_get_uint32(settings, FreeRDP_DesktopWidth),
-				               freerdp_settings_get_uint32(settings, FreeRDP_DesktopHeight));
+				xf_draw_screen(
+				    xfc, 0, 0,
+				    WINPR_ASSERTING_INT_CAST(
+				        int32_t, freerdp_settings_get_uint32(settings, FreeRDP_DesktopWidth)),
+				    WINPR_ASSERTING_INT_CAST(
+				        int32_t, freerdp_settings_get_uint32(settings, FreeRDP_DesktopHeight)));
 			}
 			else
 			{
-				xfc->scaledWidth = freerdp_settings_get_uint32(settings, FreeRDP_DesktopWidth);
-				xfc->scaledHeight = freerdp_settings_get_uint32(settings, FreeRDP_DesktopHeight);
+				xfc->scaledWidth = WINPR_ASSERTING_INT_CAST(
+				    int, freerdp_settings_get_uint32(settings, FreeRDP_DesktopWidth));
+				xfc->scaledHeight = WINPR_ASSERTING_INT_CAST(
+				    int, freerdp_settings_get_uint32(settings, FreeRDP_DesktopHeight));
 			}
 
 #endif
@@ -1038,7 +1053,8 @@ static BOOL xf_event_PropertyNotify(xfContext* xfc, const XPropertyEvent* event,
 				if (appWindow->rail_state != WINDOW_SHOW_MAXIMIZED)
 				{
 					appWindow->rail_state = WINDOW_SHOW_MAXIMIZED;
-					xf_rail_send_client_system_command(xfc, appWindow->windowId, SC_MAXIMIZE);
+					return xf_rail_send_client_system_command(xfc, appWindow->windowId,
+					                                          SC_MAXIMIZE);
 				}
 			}
 			else if (appWindow->minimized)
@@ -1046,7 +1062,8 @@ static BOOL xf_event_PropertyNotify(xfContext* xfc, const XPropertyEvent* event,
 				if (appWindow->rail_state != WINDOW_SHOW_MINIMIZED)
 				{
 					appWindow->rail_state = WINDOW_SHOW_MINIMIZED;
-					xf_rail_send_client_system_command(xfc, appWindow->windowId, SC_MINIMIZE);
+					return xf_rail_send_client_system_command(xfc, appWindow->windowId,
+					                                          SC_MINIMIZE);
 				}
 			}
 			else
@@ -1054,7 +1071,7 @@ static BOOL xf_event_PropertyNotify(xfContext* xfc, const XPropertyEvent* event,
 				if (appWindow->rail_state != WINDOW_SHOW && appWindow->rail_state != WINDOW_HIDE)
 				{
 					appWindow->rail_state = WINDOW_SHOW;
-					xf_rail_send_client_system_command(xfc, appWindow->windowId, SC_RESTORE);
+					return xf_rail_send_client_system_command(xfc, appWindow->windowId, SC_RESTORE);
 				}
 			}
 		}
@@ -1149,6 +1166,8 @@ static BOOL xf_event_suppress_events(xfContext* xfc, xfAppWindow* appWindow, con
 		case LMS_TERMINATING:
 			/* Already sent RDP end move to server. Allow events to pass. */
 			break;
+		default:
+			break;
 	}
 
 	return FALSE;
@@ -1305,14 +1324,14 @@ BOOL xf_generic_RawButtonEvent(xfContext* xfc, int button, BOOL app, BOOL down)
 {
 	UINT16 flags = 0;
 
-	if (app)
+	if (app || (button < 0))
 		return FALSE;
 
 	for (size_t i = 0; i < ARRAYSIZE(xfc->button_map); i++)
 	{
 		const button_map* cur = &xfc->button_map[i];
 
-		if (cur->button == button)
+		if (cur->button == (UINT32)button)
 		{
 			flags = cur->flags;
 			break;
