@@ -750,9 +750,11 @@ static BOOL request_file_size_async(CliprdrFileContext* file_context, CliprdrFus
 		HashTable_Remove(file_context->request_table, (void*)(uintptr_t)fuse_request->stream_id);
 		return FALSE;
 	}
+	// NOLINTNEXTLINE(clang-analyzer-unix.Malloc)
 	DEBUG_CLIPRDR(file_context->log, "Requested file size for file \"%s\" with stream id %u",
 	              fuse_file->filename, fuse_request->stream_id);
 
+	// NOLINTNEXTLINE(clang-analyzer-unix.Malloc)
 	return TRUE;
 }
 
@@ -773,7 +775,7 @@ static void write_file_attributes(CliprdrFuseFile* fuse_file, struct stat* attr)
 	{
 		attr->st_mode = S_IFREG | (fuse_file->is_readonly ? 0444 : 0644);
 		attr->st_nlink = 1;
-		attr->st_size = fuse_file->size;
+		attr->st_size = WINPR_ASSERTING_INT_CAST(__off_t, fuse_file->size);
 	}
 	attr->st_uid = getuid();
 	attr->st_gid = getgid();
@@ -1109,8 +1111,9 @@ static void cliprdr_file_fuse_readdir(fuse_req_t fuse_req, fuse_ino_t fuse_ino,
 		child = ArrayList_GetItem(fuse_file->children, j);
 
 		write_file_attributes(child, &attr);
-		entry_size = fuse_add_direntry(fuse_req, buf + written_size, max_size - written_size,
-		                               child->filename, &attr, i + 1);
+		entry_size =
+		    fuse_add_direntry(fuse_req, buf + written_size, max_size - written_size,
+		                      child->filename, &attr, WINPR_ASSERTING_INT_CAST(off_t, i + 1));
 		if (entry_size > max_size - written_size)
 			break;
 
@@ -1466,7 +1469,7 @@ static UINT cliprdr_file_context_server_file_range_request(
 	if (!rfile)
 		goto fail;
 
-	if (_fseeki64(rfile->fp, offset, SEEK_SET) < 0)
+	if (_fseeki64(rfile->fp, WINPR_ASSERTING_INT_CAST(int64_t, offset), SEEK_SET) < 0)
 		goto fail;
 
 	data = malloc(fileContentsRequest->cbRequested);
@@ -1755,18 +1758,15 @@ static CliprdrFuseFile* clip_data_dir_new(CliprdrFileContext* file_context, BOOL
 
 static char* get_parent_path(const char* filepath)
 {
-	char* base = NULL;
-	size_t parent_path_length = 0;
-	char* parent_path = NULL;
-
-	base = strrchr(filepath, '/');
+	const char* base = strrchr(filepath, '/');
 	WINPR_ASSERT(base);
 
-	while (base > filepath && *base == '/')
+	while ((base > filepath) && (*base == '/'))
 		--base;
 
-	parent_path_length = 1 + base - filepath;
-	parent_path = calloc(parent_path_length + 1, sizeof(char));
+	WINPR_ASSERT(base >= filepath);
+	const size_t parent_path_length = 1ULL + (size_t)(base - filepath);
+	char* parent_path = calloc(parent_path_length + 1, sizeof(char));
 	if (!parent_path)
 		return NULL;
 
