@@ -25,6 +25,7 @@
 
 #include <winpr/string.h>
 #include <winpr/file.h>
+#include <winpr/cast.h>
 
 #include <freerdp/client.h>
 #include <freerdp/client/file.h>
@@ -756,10 +757,8 @@ static SSIZE_T freerdp_client_rdp_file_add_line(rdpFile* file)
 
 	while ((file->lineCount + 1) > file->lineSize)
 	{
-		size_t new_size = 0;
-		rdpFileLine* new_line = NULL;
-		new_size = file->lineSize * 2;
-		new_line = (rdpFileLine*)realloc(file->lines, new_size * sizeof(rdpFileLine));
+		size_t new_size = file->lineCount + 2048;
+		rdpFileLine* new_line = (rdpFileLine*)realloc(file->lines, new_size * sizeof(rdpFileLine));
 
 		if (!new_line)
 			return -1;
@@ -1136,18 +1135,23 @@ BOOL freerdp_client_populate_rdp_file_from_settings(rdpFile* file, const rdpSett
 	file->DesktopHeight = freerdp_settings_get_uint32(settings, FreeRDP_DesktopHeight);
 	file->SessionBpp = freerdp_settings_get_uint32(settings, FreeRDP_ColorDepth);
 	file->DesktopScaleFactor = freerdp_settings_get_uint32(settings, FreeRDP_DesktopScaleFactor);
-	file->DynamicResolution = freerdp_settings_get_bool(settings, FreeRDP_DynamicResolutionUpdate);
-	file->VideoPlaybackMode = freerdp_settings_get_bool(settings, FreeRDP_SupportVideoOptimized);
+	file->DynamicResolution = WINPR_ASSERTING_INT_CAST(
+	    UINT32, freerdp_settings_get_bool(settings, FreeRDP_DynamicResolutionUpdate));
+	file->VideoPlaybackMode = WINPR_ASSERTING_INT_CAST(
+	    UINT32, freerdp_settings_get_bool(settings, FreeRDP_SupportVideoOptimized));
 
 	// TODO file->MaximizeToCurrentDisplays;
 	// TODO file->SingleMonInWindowedMode;
 	// TODO file->EncodeRedirectedVideoCapture;
 	// TODO file->RedirectedVideoCaptureEncodingQuality;
-	file->ConnectToConsole = freerdp_settings_get_bool(settings, FreeRDP_ConsoleSession);
-	file->NegotiateSecurityLayer =
-	    freerdp_settings_get_bool(settings, FreeRDP_NegotiateSecurityLayer);
-	file->EnableCredSSPSupport = freerdp_settings_get_bool(settings, FreeRDP_NlaSecurity);
-	file->EnableRdsAadAuth = freerdp_settings_get_bool(settings, FreeRDP_AadSecurity);
+	file->ConnectToConsole = WINPR_ASSERTING_INT_CAST(
+	    UINT32, freerdp_settings_get_bool(settings, FreeRDP_ConsoleSession));
+	file->NegotiateSecurityLayer = WINPR_ASSERTING_INT_CAST(
+	    UINT32, freerdp_settings_get_bool(settings, FreeRDP_NegotiateSecurityLayer));
+	file->EnableCredSSPSupport =
+	    WINPR_ASSERTING_INT_CAST(UINT32, freerdp_settings_get_bool(settings, FreeRDP_NlaSecurity));
+	file->EnableRdsAadAuth =
+	    WINPR_ASSERTING_INT_CAST(UINT32, freerdp_settings_get_bool(settings, FreeRDP_AadSecurity));
 
 	if (freerdp_settings_get_bool(settings, FreeRDP_RemoteApplicationMode))
 		index = FreeRDP_RemoteApplicationWorkingDir;
@@ -1224,19 +1228,22 @@ BOOL freerdp_client_populate_rdp_file_from_settings(rdpFile* file, const rdpSett
 	if (!rdp_opt_duplicate(settings, FreeRDP_GatewayAvdActivityhint, &file->activityhint))
 		return FALSE;
 
-	file->AudioCaptureMode = freerdp_settings_get_bool(settings, FreeRDP_AudioCapture);
-	file->BitmapCachePersistEnable =
-	    freerdp_settings_get_bool(settings, FreeRDP_BitmapCachePersistEnabled);
-	file->Compression = freerdp_settings_get_bool(settings, FreeRDP_CompressionEnabled);
+	file->AudioCaptureMode =
+	    WINPR_ASSERTING_INT_CAST(UINT32, freerdp_settings_get_bool(settings, FreeRDP_AudioCapture));
+	file->BitmapCachePersistEnable = WINPR_ASSERTING_INT_CAST(
+	    UINT32, freerdp_settings_get_bool(settings, FreeRDP_BitmapCachePersistEnabled));
+	file->Compression = WINPR_ASSERTING_INT_CAST(
+	    UINT32, freerdp_settings_get_bool(settings, FreeRDP_CompressionEnabled));
 	file->AuthenticationLevel = freerdp_settings_get_uint32(settings, FreeRDP_AuthenticationLevel);
 	file->GatewayUsageMethod = freerdp_settings_get_uint32(settings, FreeRDP_GatewayUsageMethod);
 	file->GatewayCredentialsSource =
 	    freerdp_settings_get_uint32(settings, FreeRDP_GatewayCredentialsSource);
-	file->PromptCredentialOnce =
-	    freerdp_settings_get_bool(settings, FreeRDP_GatewayUseSameCredentials);
-	file->PromptForCredentials = freerdp_settings_get_bool(settings, FreeRDP_PromptForCredentials);
-	file->RemoteApplicationMode =
-	    freerdp_settings_get_bool(settings, FreeRDP_RemoteApplicationMode);
+	file->PromptCredentialOnce = WINPR_ASSERTING_INT_CAST(
+	    UINT32, freerdp_settings_get_bool(settings, FreeRDP_GatewayUseSameCredentials));
+	file->PromptForCredentials = WINPR_ASSERTING_INT_CAST(
+	    UINT32, freerdp_settings_get_bool(settings, FreeRDP_PromptForCredentials));
+	file->RemoteApplicationMode = WINPR_ASSERTING_INT_CAST(
+	    UINT32, freerdp_settings_get_bool(settings, FreeRDP_RemoteApplicationMode));
 	if (!FILE_POPULATE_STRING(&file->GatewayAccessToken, settings, FreeRDP_GatewayAccessToken) ||
 	    !FILE_POPULATE_STRING(&file->RemoteApplicationProgram, settings,
 	                          FreeRDP_RemoteApplicationProgram) ||
@@ -1251,26 +1258,34 @@ BOOL freerdp_client_populate_rdp_file_from_settings(rdpFile* file, const rdpSett
 	    !FILE_POPULATE_STRING(&file->RemoteApplicationCmdLine, settings,
 	                          FreeRDP_RemoteApplicationCmdLine))
 		return FALSE;
-	file->SpanMonitors = freerdp_settings_get_bool(settings, FreeRDP_SpanMonitors);
-	file->UseMultiMon = freerdp_settings_get_bool(settings, FreeRDP_UseMultimon);
-	file->AllowDesktopComposition =
-	    freerdp_settings_get_bool(settings, FreeRDP_AllowDesktopComposition);
-	file->AllowFontSmoothing = freerdp_settings_get_bool(settings, FreeRDP_AllowFontSmoothing);
-	file->DisableWallpaper = freerdp_settings_get_bool(settings, FreeRDP_DisableWallpaper);
-	file->DisableFullWindowDrag =
-	    freerdp_settings_get_bool(settings, FreeRDP_DisableFullWindowDrag);
-	file->DisableMenuAnims = freerdp_settings_get_bool(settings, FreeRDP_DisableMenuAnims);
-	file->DisableThemes = freerdp_settings_get_bool(settings, FreeRDP_DisableThemes);
+	file->SpanMonitors =
+	    WINPR_ASSERTING_INT_CAST(UINT32, freerdp_settings_get_bool(settings, FreeRDP_SpanMonitors));
+	file->UseMultiMon =
+	    WINPR_ASSERTING_INT_CAST(UINT32, freerdp_settings_get_bool(settings, FreeRDP_UseMultimon));
+	file->AllowDesktopComposition = WINPR_ASSERTING_INT_CAST(
+	    UINT32, freerdp_settings_get_bool(settings, FreeRDP_AllowDesktopComposition));
+	file->AllowFontSmoothing = WINPR_ASSERTING_INT_CAST(
+	    UINT32, freerdp_settings_get_bool(settings, FreeRDP_AllowFontSmoothing));
+	file->DisableWallpaper = WINPR_ASSERTING_INT_CAST(
+	    UINT32, freerdp_settings_get_bool(settings, FreeRDP_DisableWallpaper));
+	file->DisableFullWindowDrag = WINPR_ASSERTING_INT_CAST(
+	    UINT32, freerdp_settings_get_bool(settings, FreeRDP_DisableFullWindowDrag));
+	file->DisableMenuAnims = WINPR_ASSERTING_INT_CAST(
+	    UINT32, freerdp_settings_get_bool(settings, FreeRDP_DisableMenuAnims));
+	file->DisableThemes = WINPR_ASSERTING_INT_CAST(
+	    UINT32, freerdp_settings_get_bool(settings, FreeRDP_DisableThemes));
 	file->BandwidthAutoDetect = (freerdp_settings_get_uint32(settings, FreeRDP_ConnectionType) >=
 	                             CONNECTION_TYPE_AUTODETECT)
 	                                ? TRUE
 	                                : FALSE;
 	file->NetworkAutoDetect =
 	    freerdp_settings_get_bool(settings, FreeRDP_NetworkAutoDetect) ? 1 : 0;
-	file->AutoReconnectionEnabled =
-	    freerdp_settings_get_bool(settings, FreeRDP_AutoReconnectionEnabled);
-	file->RedirectSmartCards = freerdp_settings_get_bool(settings, FreeRDP_RedirectSmartCards);
-	file->RedirectWebauthN = freerdp_settings_get_bool(settings, FreeRDP_RedirectWebAuthN);
+	file->AutoReconnectionEnabled = WINPR_ASSERTING_INT_CAST(
+	    UINT32, freerdp_settings_get_bool(settings, FreeRDP_AutoReconnectionEnabled));
+	file->RedirectSmartCards = WINPR_ASSERTING_INT_CAST(
+	    UINT32, freerdp_settings_get_bool(settings, FreeRDP_RedirectSmartCards));
+	file->RedirectWebauthN = WINPR_ASSERTING_INT_CAST(
+	    UINT32, freerdp_settings_get_bool(settings, FreeRDP_RedirectWebAuthN));
 
 	redirectCameras =
 	    freerdp_client_channel_args_to_string(settings, RDPECAM_DVC_CHANNEL_NAME, "device:");
@@ -1285,7 +1300,7 @@ BOOL freerdp_client_populate_rdp_file_from_settings(rdpFile* file, const rdpSett
 			errno = 0;
 			val = strtoul(str, NULL, 0);
 			if ((val < UINT32_MAX) && (errno == 0))
-				file->EncodeRedirectedVideoCapture = val;
+				file->EncodeRedirectedVideoCapture = (UINT32)val;
 		}
 		free(str);
 
@@ -1298,7 +1313,7 @@ BOOL freerdp_client_populate_rdp_file_from_settings(rdpFile* file, const rdpSett
 			val = strtoul(str, NULL, 0);
 			if ((val <= 2) && (errno == 0))
 			{
-				file->RedirectedVideoCaptureEncodingQuality = val;
+				file->RedirectedVideoCaptureEncodingQuality = (UINT32)val;
 			}
 		}
 		free(str);
@@ -1655,11 +1670,11 @@ static ADDIN_ARGV* rdp_file_to_args(const char* channel, const char* values)
 		if (!rc)
 			goto fail;
 	}
-	free(p);
+	CommandLineParserFree(p);
 	return args;
 
 fail:
-	free(p);
+	CommandLineParserFree(p);
 	freerdp_addin_argv_free(args);
 	return NULL;
 }
@@ -2308,15 +2323,11 @@ BOOL freerdp_client_populate_settings_from_rdp_file(const rdpFile* file, rdpSett
 	if (~file->RedirectLocation && (file->RedirectLocation != 0))
 	{
 		size_t count = 0;
-		union
-		{
-			void* pv;
-			char** str;
-			const char** cstr;
-		} cnv;
-		cnv.str = CommandLineParseCommaSeparatedValuesEx(LOCATION_CHANNEL_NAME, NULL, &count);
-		const BOOL rc = freerdp_client_add_dynamic_channel(settings, count, cnv.cstr);
-		free(cnv.pv);
+
+		char** ptr = CommandLineParseCommaSeparatedValuesEx(LOCATION_CHANNEL_NAME, NULL, &count);
+		const BOOL rc =
+		    freerdp_client_add_dynamic_channel(settings, count, (const char* const*)ptr);
+		CommandLineParserFree(ptr);
 		if (!rc)
 			return FALSE;
 	}
@@ -2369,7 +2380,7 @@ BOOL freerdp_client_populate_settings_from_rdp_file(const rdpFile* file, rdpSett
 		union
 		{
 			char** c;
-			const char** cc;
+			const char* const* cc;
 		} cnv;
 		ADDIN_ARGV* args = rdp_file_to_args(RDPECAM_DVC_CHANNEL_NAME, file->RedirectCameras);
 		if (!args)
@@ -2395,7 +2406,8 @@ BOOL freerdp_client_populate_settings_from_rdp_file(const rdpFile* file, rdpSett
 
 		cnv.c = args->argv;
 		if (status)
-			status = freerdp_client_add_dynamic_channel(settings, args->argc, cnv.cc);
+			status = freerdp_client_add_dynamic_channel(
+			    settings, WINPR_ASSERTING_INT_CAST(size_t, args->argc), cnv.cc);
 		freerdp_addin_argv_free(args);
 		if (!status)
 			return FALSE;
@@ -2413,13 +2425,14 @@ BOOL freerdp_client_populate_settings_from_rdp_file(const rdpFile* file, rdpSett
 		union
 		{
 			char** c;
-			const char** cc;
+			const char* const* cc;
 		} cnv;
 		ADDIN_ARGV* args = rdp_file_to_args(URBDRC_CHANNEL_NAME, file->UsbDevicesToRedirect);
 		if (!args)
 			return FALSE;
 		cnv.c = args->argv;
-		const BOOL status = freerdp_client_add_dynamic_channel(settings, args->argc, cnv.cc);
+		const BOOL status = freerdp_client_add_dynamic_channel(
+		    settings, WINPR_ASSERTING_INT_CAST(size_t, args->argc), cnv.cc);
 		freerdp_addin_argv_free(args);
 		if (!status)
 			return FALSE;
@@ -2439,34 +2452,34 @@ BOOL freerdp_client_populate_settings_from_rdp_file(const rdpFile* file, rdpSett
 	if (~(size_t)file->SelectedMonitors)
 	{
 		size_t count = 0;
-		char** args = CommandLineParseCommaSeparatedValues(file->SelectedMonitors, &count);
+		char** ptr = CommandLineParseCommaSeparatedValues(file->SelectedMonitors, &count);
 		UINT32* list = NULL;
 
 		if (!freerdp_settings_set_pointer_len(settings, FreeRDP_MonitorIds, NULL, count))
 		{
-			free(args);
+			CommandLineParserFree(ptr);
 			return FALSE;
 		}
 		list = freerdp_settings_get_pointer_writable(settings, FreeRDP_MonitorIds);
 		if (!list && (count > 0))
 		{
-			free(args);
+			CommandLineParserFree(ptr);
 			return FALSE;
 		}
 		for (size_t x = 0; x < count; x++)
 		{
 			unsigned long val = 0;
 			errno = 0;
-			val = strtoul(args[x], NULL, 0);
+			val = strtoul(ptr[x], NULL, 0);
 			if ((val >= UINT32_MAX) && (errno != 0))
 			{
-				free(args);
+				CommandLineParserFree(ptr);
 				free(list);
 				return FALSE;
 			}
-			list[x] = val;
+			list[x] = (UINT32)val;
 		}
-		free(args);
+		CommandLineParserFree(ptr);
 	}
 
 	if (~file->DynamicResolution)
@@ -2620,7 +2633,7 @@ int freerdp_client_rdp_file_get_integer_option(const rdpFile* file, const char*
 	if (freerdp_client_rdp_file_find_integer_entry(wfile, name, &value, &line))
 	{
 		if (value && ~(*value))
-			return *value;
+			return WINPR_ASSERTING_INT_CAST(int, *value);
 		if (line)
 			return (int)line->iValue;
 	}
