@@ -23,6 +23,7 @@
 
 #include <winpr/crt.h>
 #include <winpr/assert.h>
+#include <winpr/cast.h>
 
 #include <freerdp/freerdp.h>
 #include <winpr/stream.h>
@@ -62,10 +63,10 @@ static UINT32 update_glyph_offset(const BYTE* data, size_t length, UINT32 index,
 		}
 
 		if (flAccel & SO_VERTICAL)
-			*y += offset;
+			*y += WINPR_ASSERTING_INT_CAST(int32_t, offset);
 
 		if (flAccel & SO_HORIZONTAL)
-			*x += offset;
+			*x += WINPR_ASSERTING_INT_CAST(int32_t, offset);
 	}
 
 	return index;
@@ -109,8 +110,8 @@ static BOOL update_process_glyph(rdpContext* context, const BYTE* data, UINT32 c
 
 	if ((dx <= (bound->x + bound->width)) && (dy <= (bound->y + bound->height)))
 	{
-		INT32 dw = glyph->cx - sx;
-		INT32 dh = glyph->cy - sy;
+		INT32 dw = WINPR_ASSERTING_INT_CAST(int32_t, glyph->cx) - sx;
+		INT32 dh = WINPR_ASSERTING_INT_CAST(int32_t, glyph->cy) - sy;
 
 		if ((dw + dx) > (bound->x + bound->width))
 			dw = (bound->x + bound->width) - (dw + dx);
@@ -126,7 +127,7 @@ static BOOL update_process_glyph(rdpContext* context, const BYTE* data, UINT32 c
 	}
 
 	if (flAccel & SO_CHAR_INC_EQUAL_BM_BASE)
-		*x += glyph->cx;
+		*x += WINPR_ASSERTING_INT_CAST(int32_t, glyph->cx);
 
 	return TRUE;
 }
@@ -195,7 +196,8 @@ static BOOL update_process_glyph_fragments(rdpContext* context, const BYTE* data
 	if (bkHeight < 0)
 		bkHeight = 0;
 
-	if (opX + opWidth > (INT64)freerdp_settings_get_uint32(context->settings, FreeRDP_DesktopWidth))
+	const UINT32 w = freerdp_settings_get_uint32(context->settings, FreeRDP_DesktopWidth);
+	if (opX + opWidth > (INT64)w)
 	{
 		/**
 		 * Some Microsoft servers send erroneous high values close to the
@@ -206,10 +208,10 @@ static BOOL update_process_glyph_fragments(rdpContext* context, const BYTE* data
 		 * a RDP session to Windows XP Professional SP3.
 		 * This workaround prevents resulting problems in the UI callbacks.
 		 */
-		opWidth = freerdp_settings_get_uint32(context->settings, FreeRDP_DesktopWidth) - opX;
+		opWidth = WINPR_ASSERTING_INT_CAST(int, w) - opX;
 	}
 
-	if (bkX + bkWidth > (INT64)freerdp_settings_get_uint32(context->settings, FreeRDP_DesktopWidth))
+	if (bkX + bkWidth > (INT64)w)
 	{
 		/**
 		 * Some Microsoft servers send erroneous high values close to the
@@ -220,13 +222,13 @@ static BOOL update_process_glyph_fragments(rdpContext* context, const BYTE* data
 		 * a RDP session to Windows XP Professional SP3.
 		 * This workaround prevents resulting problems in the UI callbacks.
 		 */
-		bkWidth = freerdp_settings_get_uint32(context->settings, FreeRDP_DesktopWidth) - bkX;
+		bkWidth = WINPR_ASSERTING_INT_CAST(int, w) - bkX;
 	}
 
-	bound.x = bkX;
-	bound.y = bkY;
-	bound.width = bkWidth;
-	bound.height = bkHeight;
+	bound.x = WINPR_ASSERTING_INT_CAST(INT16, bkX);
+	bound.y = WINPR_ASSERTING_INT_CAST(INT16, bkY);
+	bound.width = WINPR_ASSERTING_INT_CAST(INT16, bkWidth);
+	bound.height = WINPR_ASSERTING_INT_CAST(INT16, bkHeight);
 
 	if (!glyph->BeginDraw(context, opX, opY, opWidth, opHeight, bgcolor, fgcolor, fOpRedundant))
 		return FALSE;
@@ -250,7 +252,7 @@ static BOOL update_process_glyph_fragments(rdpContext* context, const BYTE* data
 				if (fragments == NULL)
 					return FALSE;
 
-				for (size_t n = 0; n < size;)
+				for (UINT32 n = 0; n < size;)
 				{
 					const UINT32 fop = fragments[n++];
 					n = update_glyph_offset(fragments, size, n, &x, &y, ulCharInc, flAccel);
@@ -311,7 +313,8 @@ static BOOL update_gdi_glyph_index(rdpContext* context, GLYPH_INDEX_ORDER* glyph
 	    context, glyphIndex->data, glyphIndex->cbData, glyphIndex->cacheId, glyphIndex->ulCharInc,
 	    glyphIndex->flAccel, glyphIndex->backColor, glyphIndex->foreColor, glyphIndex->x,
 	    glyphIndex->y, glyphIndex->bkLeft, glyphIndex->bkTop, bkWidth, bkHeight, glyphIndex->opLeft,
-	    glyphIndex->opTop, opWidth, opHeight, glyphIndex->fOpRedundant);
+	    glyphIndex->opTop, opWidth, opHeight,
+	    WINPR_ASSERTING_INT_CAST(int32_t, glyphIndex->fOpRedundant));
 }
 
 static BOOL update_gdi_fast_index(rdpContext* context, const FAST_INDEX_ORDER* fastIndex)
@@ -491,23 +494,17 @@ static BOOL update_gdi_fast_glyph(rdpContext* context, const FAST_GLYPH_ORDER* f
 
 static BOOL update_gdi_cache_glyph(rdpContext* context, const CACHE_GLYPH_ORDER* cacheGlyph)
 {
-	rdpCache* cache = NULL;
-
 	if (!context || !cacheGlyph || !context->cache)
 		return FALSE;
 
-	cache = context->cache;
+	rdpCache* cache = context->cache;
 
 	for (size_t i = 0; i < cacheGlyph->cGlyphs; i++)
 	{
 		const GLYPH_DATA* glyph_data = &cacheGlyph->glyphData[i];
-		rdpGlyph* glyph = NULL;
-
-		if (!glyph_data)
-			return FALSE;
-
-		if (!(glyph = Glyph_Alloc(context, glyph_data->x, glyph_data->y, glyph_data->cx,
-		                          glyph_data->cy, glyph_data->cb, glyph_data->aj)))
+		rdpGlyph* glyph = Glyph_Alloc(context, glyph_data->x, glyph_data->y, glyph_data->cx,
+		                              glyph_data->cy, glyph_data->cb, glyph_data->aj);
+		if (!glyph)
 			return FALSE;
 
 		if (!glyph_cache_put(cache->glyph, cacheGlyph->cacheId, glyph_data->cacheIndex, glyph))
@@ -522,23 +519,16 @@ static BOOL update_gdi_cache_glyph(rdpContext* context, const CACHE_GLYPH_ORDER*
 
 static BOOL update_gdi_cache_glyph_v2(rdpContext* context, const CACHE_GLYPH_V2_ORDER* cacheGlyphV2)
 {
-	rdpCache* cache = NULL;
-
 	if (!context || !cacheGlyphV2 || !context->cache)
 		return FALSE;
 
-	cache = context->cache;
+	rdpCache* cache = context->cache;
 
 	for (size_t i = 0; i < cacheGlyphV2->cGlyphs; i++)
 	{
 		const GLYPH_DATA_V2* glyphData = &cacheGlyphV2->glyphData[i];
-		rdpGlyph* glyph = NULL;
-
-		if (!glyphData)
-			return FALSE;
-
-		glyph = Glyph_Alloc(context, glyphData->x, glyphData->y, glyphData->cx, glyphData->cy,
-		                    glyphData->cb, glyphData->aj);
+		rdpGlyph* glyph = Glyph_Alloc(context, glyphData->x, glyphData->y, glyphData->cx,
+		                              glyphData->cy, glyphData->cb, glyphData->aj);
 
 		if (!glyph)
 			return FALSE;
@@ -753,7 +743,7 @@ void glyph_cache_free(rdpGlyphCache* glyphCache)
 				}
 			}
 
-			free(entries);
+			free((void*)entries);
 			cache[i].entries = NULL;
 		}
 
