@@ -466,7 +466,7 @@ BOOL nego_send_preconnection_pdu(rdpNego* nego)
 			free(wszPCB);
 			return FALSE;
 		}
-		cchPCB = len;
+		cchPCB = (UINT16)len;
 		cchPCB += 1; /* zero-termination */
 		cbSize += cchPCB * sizeof(WCHAR);
 	}
@@ -842,6 +842,8 @@ int nego_recv(rdpTransport* transport, wStream* s, void* extra)
 				if (!nego_process_negotiation_failure(nego, s))
 					return -1;
 				break;
+			default:
+				return -1;
 		}
 	}
 	else if (li == 6)
@@ -1295,9 +1297,9 @@ static const char* nego_rdp_neg_rsp_flags_str(UINT32 flags)
 		winpr_str_append("RESTRICTED_ADMIN_MODE_SUPPORTED", buffer, sizeof(buffer), "|");
 	if (flags & REDIRECTED_AUTHENTICATION_MODE_SUPPORTED)
 		winpr_str_append("REDIRECTED_AUTHENTICATION_MODE_SUPPORTED", buffer, sizeof(buffer), "|");
-	if ((flags &
-	     ~(EXTENDED_CLIENT_DATA_SUPPORTED | DYNVC_GFX_PROTOCOL_SUPPORTED | RDP_NEGRSP_RESERVED |
-	       RESTRICTED_ADMIN_MODE_SUPPORTED | REDIRECTED_AUTHENTICATION_MODE_SUPPORTED)))
+	if ((flags & (uint32_t)~(EXTENDED_CLIENT_DATA_SUPPORTED | DYNVC_GFX_PROTOCOL_SUPPORTED |
+	                         RDP_NEGRSP_RESERVED | RESTRICTED_ADMIN_MODE_SUPPORTED |
+	                         REDIRECTED_AUTHENTICATION_MODE_SUPPORTED)))
 		winpr_str_append("UNKNOWN", buffer, sizeof(buffer), "|");
 
 	return buffer;
@@ -1507,7 +1509,7 @@ BOOL nego_send_negotiation_response(rdpNego* nego)
 				    ENCRYPTION_LEVEL_NONE)
 				{
 					/**
-					 * If the server implementation did not explicitely set a
+					 * If the server implementation did not explicitly set a
 					 * encryption level we default to client compatible
 					 */
 					if (!freerdp_settings_set_uint32(settings, FreeRDP_EncryptionLevel,
@@ -1815,7 +1817,7 @@ void nego_enable_aad(rdpNego* nego, BOOL enable_aad)
  * Set routing token.
  * @param nego A pointer to the NEGO struct
  * @param RoutingToken A pointer to the routing token
- * @param RoutingTokenLength The lenght of the routing token
+ * @param RoutingTokenLength The length of the routing token
  *
  * @return \b TRUE for success, \b FALSE otherwise
  */
