@@ -26,6 +26,8 @@
 #include <winpr/string.h>
 #include <winpr/synch.h>
 #include <winpr/assert.h>
+#include <winpr/cast.h>
+#include <winpr/json.h>
 
 #include "rdp.h"
 
@@ -42,6 +44,14 @@
 
 #define RDP_TAG FREERDP_TAG("core.rdp")
 
+typedef struct
+{
+	const char* file;
+	const char* fkt;
+	size_t line;
+	DWORD level;
+} log_line_t;
+
 static const char* DATA_PDU_TYPE_STRINGS[80] = {
 	"?",
 	"?",      /* 0x00 - 0x01 */
@@ -297,7 +307,10 @@ BOOL rdp_read_share_control_header(rdpRdp* rdp, wStream* s, UINT16* tpktLength,
 	WLog_Print(rdp->log, WLOG_DEBUG, "type=%s, tpktLength=%" PRIuz ", remainingLength=%" PRIuz,
 	           pdu_type_to_str(*type, buffer, sizeof(buffer)), len, remLen);
 	if (remainingLength)
-		*remainingLength = remLen;
+	{
+		WINPR_ASSERT(remLen <= UINT16_MAX);
+		*remainingLength = (UINT16)remLen;
+	}
 	return Stream_CheckAndLogRequiredLengthWLog(rdp->log, s, remLen);
 }
 
@@ -315,9 +328,9 @@ BOOL rdp_write_share_control_header(rdpRdp* rdp, wStream* s, size_t length, UINT
 		return FALSE;
 	length -= RDP_PACKET_HEADER_MAX_LENGTH;
 	/* Share Control Header */
-	Stream_Write_UINT16(s, length);      /* totalLength */
-	Stream_Write_UINT16(s, type | 0x10); /* pduType */
-	Stream_Write_UINT16(s, channel_id);  /* pduSource */
+	Stream_Write_UINT16(s, WINPR_ASSERTING_INT_CAST(uint16_t, length));      /* totalLength */
+	Stream_Write_UINT16(s, WINPR_ASSERTING_INT_CAST(uint16_t, type | 0x10)); /* pduType */
+	Stream_Write_UINT16(s, WINPR_ASSERTING_INT_CAST(uint16_t, channel_id));  /* pduSource */
 	return TRUE;
 }
 
@@ -361,7 +374,8 @@ BOOL rdp_write_share_data_header(rdpRdp* rdp, wStream* s, size_t length, BYTE ty
 	Stream_Write_UINT32(s, share_id);  /* shareId (4 bytes) */
 	Stream_Write_UINT8(s, 0);          /* pad1 (1 byte) */
 	Stream_Write_UINT8(s, STREAM_LOW); /* streamId (1 byte) */
-	Stream_Write_UINT16(s, length);    /* uncompressedLength (2 bytes) */
+	Stream_Write_UINT16(
+	    s, WINPR_ASSERTING_INT_CAST(uint16_t, length)); /* uncompressedLength (2 bytes) */
 	Stream_Write_UINT8(s, type);       /* pduType2, Data PDU Type (1 byte) */
 	Stream_Write_UINT8(s, 0);          /* compressedType (1 byte) */
 	Stream_Write_UINT16(s, 0);         /* compressedLength (2 bytes) */
@@ -526,7 +540,6 @@ fail:
 BOOL rdp_read_header(rdpRdp* rdp, wStream* s, UINT16* length, UINT16* channelId)
 {
 	BYTE li = 0;
-	BYTE byte = 0;
 	BYTE code = 0;
 	BYTE choice = 0;
 	UINT16 initiator = 0;
@@ -631,7 +644,7 @@ BOOL rdp_read_header(rdpRdp* rdp, wStream* s, UINT16* length, UINT16* channelId)
 	if (!per_read_integer16(s, channelId, 0)) /* channelId */
 		return FALSE;
 
-	Stream_Read_UINT8(s, byte); /* dataPriority + Segmentation (0x70) */
+	const uint8_t byte = Stream_Get_UINT8(s); /* dataPriority + Segmentation (0x70) */
 
 	if (!per_read_length(s, length)) /* userData (OCTET_STRING) */
 		return FALSE;
@@ -667,14 +680,14 @@ BOOL rdp_write_header(rdpRdp* rdp, wStream* s, size_t length, UINT16 channelId)
 	if ((rdp->sec_flags & SEC_ENCRYPT) &&
 	    (rdp->settings->EncryptionMethods == ENCRYPTION_METHOD_FIPS))
 	{
-		const UINT16 body_length = length - RDP_PACKET_HEADER_MAX_LENGTH;
+		const UINT16 body_length = (UINT16)length - RDP_PACKET_HEADER_MAX_LENGTH;
 		const UINT16 pad = 8 - (body_length % 8);
 
 		if (pad != 8)
 			length += pad;
 	}
 
-	if (!mcs_write_domain_mcspdu_header(s, MCSPDU, length, 0))
+	if (!mcs_write_domain_mcspdu_header(s, MCSPDU, (UINT16)length, 0))
 		return FALSE;
 	if (!per_write_integer16(s, rdp->mcs->userId, MCS_BASE_CHANNEL_ID)) /* initiator */
 		return FALSE;
@@ -690,7 +703,8 @@ BOOL rdp_write_header(rdpRdp* rdp, wStream* s, size_t length, UINT16 channelId)
 	 * the data first and then store the header.
 	 */
 	length = (length - RDP_PACKET_HEADER_MAX_LENGTH) | 0x8000;
-	Stream_Write_UINT16_BE(s, length); /* userData (OCTET_STRING) */
+	Stream_Write_UINT16_BE(
+	    s, WINPR_ASSERTING_INT_CAST(uint16_t, length)); /* userData (OCTET_STRING) */
 	return TRUE;
 }
 
@@ -707,7 +721,8 @@ static BOOL rdp_security_stream_out(rdpRdp* rdp, wStream* s, size_t length, UINT
 
 	if (sec_flags != 0)
 	{
-		if (!rdp_write_security_header(rdp, s, sec_flags))
+		WINPR_ASSERT(sec_flags <= UINT16_MAX);
+		if (!rdp_write_security_header(rdp, s, (UINT16)sec_flags))
 			return FALSE;
 
 		if (sec_flags & SEC_ENCRYPT)
@@ -719,7 +734,7 @@ static BOOL rdp_security_stream_out(rdpRdp* rdp, wStream* s, size_t length, UINT
 			if (rdp->settings->EncryptionMethods == ENCRYPTION_METHOD_FIPS)
 			{
 				BYTE* data = Stream_PointerAs(s, BYTE) + 12;
-				const size_t size = (data - Stream_Buffer(s));
+				const size_t size = WINPR_ASSERTING_INT_CAST(size_t, (data - Stream_Buffer(s)));
 				if (size > length)
 					goto unlock;
 
@@ -736,7 +751,7 @@ static BOOL rdp_security_stream_out(rdpRdp* rdp, wStream* s, size_t length, UINT
 				if (*pad)
 					memset(data + length, 0, *pad);
 
-				Stream_Write_UINT8(s, *pad);
+				Stream_Write_UINT8(s, WINPR_ASSERTING_INT_CAST(uint8_t, *pad));
 
 				if (!Stream_CheckAndLogRequiredCapacityWLog(rdp->log, s, 8))
 					goto unlock;
@@ -758,11 +773,11 @@ static BOOL rdp_security_stream_out(rdpRdp* rdp, wStream* s, size_t length, UINT
 				if (!Stream_CheckAndLogRequiredCapacityWLog(rdp->log, s, 8))
 					goto unlock;
 				if (sec_flags & SEC_SECURE_CHECKSUM)
-					status = security_salted_mac_signature(rdp, data, length, TRUE,
+					status = security_salted_mac_signature(rdp, data, (UINT32)length, TRUE,
 					                                       Stream_Pointer(s), 8);
 				else
-					status =
-					    security_mac_signature(rdp, data, length, Stream_PointerAs(s, BYTE), 8);
+					status = security_mac_signature(rdp, data, (UINT32)length,
+					                                Stream_PointerAs(s, BYTE), 8);
 
 				if (!status)
 					goto unlock;
@@ -965,9 +980,6 @@ static BOOL rdp_recv_server_shutdown_denied_pdu(rdpRdp* rdp, wStream* s)
 
 static BOOL rdp_recv_server_set_keyboard_indicators_pdu(rdpRdp* rdp, wStream* s)
 {
-	UINT16 unitId = 0;
-	UINT16 ledFlags = 0;
-
 	WINPR_ASSERT(rdp);
 	WINPR_ASSERT(s);
 
@@ -978,8 +990,8 @@ static BOOL rdp_recv_server_set_keyboard_indicators_pdu(rdpRdp* rdp, wStream* s)
 	if (!Stream_CheckAndLogRequiredLengthWLog(rdp->log, s, 4))
 		return FALSE;
 
-	Stream_Read_UINT16(s, unitId);   /* unitId (2 bytes) */
-	Stream_Read_UINT16(s, ledFlags); /* ledFlags (2 bytes) */
+	const uint16_t unitId = Stream_Get_UINT16(s); /* unitId (2 bytes) */
+	const UINT16 ledFlags = Stream_Get_UINT16(s); /* ledFlags (2 bytes) */
 	return IFCALLRESULT(TRUE, context->update->SetKeyboardIndicators, context, ledFlags);
 }
 
@@ -1441,7 +1453,6 @@ BOOL rdp_decrypt(rdpRdp* rdp, wStream* s, UINT16* pLength, UINT16 securityFlags)
 	BYTE cmac[8] = { 0 };
 	BYTE wmac[8] = { 0 };
 	BOOL status = FALSE;
-	INT32 length = 0;
 
 	WINPR_ASSERT(rdp);
 	WINPR_ASSERT(rdp->settings);
@@ -1451,7 +1462,7 @@ BOOL rdp_decrypt(rdpRdp* rdp, wStream* s, UINT16* pLength, UINT16 securityFlags)
 	if (!security_lock(rdp))
 		return FALSE;
 
-	length = *pLength;
+	INT32 length = *pLength;
 	if (rdp->settings->EncryptionMethods == ENCRYPTION_METHOD_NONE)
 		return TRUE;
 
@@ -1485,10 +1496,10 @@ BOOL rdp_decrypt(rdpRdp* rdp, wStream* s, UINT16* pLength, UINT16 securityFlags)
 			goto unlock;
 		}
 
-		if (!security_fips_decrypt(Stream_Pointer(s), length, rdp))
+		if (!security_fips_decrypt(Stream_Pointer(s), (size_t)length, rdp))
 			goto unlock;
 
-		if (!security_fips_check_signature(Stream_ConstPointer(s), length - pad, sig, 8, rdp))
+		if (!security_fips_check_signature(Stream_ConstPointer(s), (size_t)padLength, sig, 8, rdp))
 			goto unlock;
 
 		Stream_SetLength(s, Stream_Length(s) - pad);
@@ -1508,15 +1519,15 @@ BOOL rdp_decrypt(rdpRdp* rdp, wStream* s, UINT16* pLength, UINT16 securityFlags)
 			goto unlock;
 		}
 
-		if (!security_decrypt(Stream_PointerAs(s, BYTE), length, rdp))
+		if (!security_decrypt(Stream_PointerAs(s, BYTE), (size_t)length, rdp))
 			goto unlock;
 
 		if (securityFlags & SEC_SECURE_CHECKSUM)
-			status = security_salted_mac_signature(rdp, Stream_ConstPointer(s), length, FALSE, cmac,
-			                                       sizeof(cmac));
+			status = security_salted_mac_signature(rdp, Stream_ConstPointer(s), (UINT32)length,
+			                                       FALSE, cmac, sizeof(cmac));
 		else
-			status =
-			    security_mac_signature(rdp, Stream_ConstPointer(s), length, cmac, sizeof(cmac));
+			status = security_mac_signature(rdp, Stream_ConstPointer(s), (UINT32)length, cmac,
+			                                sizeof(cmac));
 
 		if (!status)
 			goto unlock;
@@ -1534,7 +1545,7 @@ BOOL rdp_decrypt(rdpRdp* rdp, wStream* s, UINT16* pLength, UINT16 securityFlags)
 			// return FALSE;
 		}
 
-		*pLength = length;
+		*pLength = (UINT16)length;
 	}
 	res = TRUE;
 unlock:
@@ -2392,6 +2403,7 @@ static void rdp_reset_free(rdpRdp* rdp)
 {
 	WINPR_ASSERT(rdp);
 
+	(void)security_lock(rdp);
 	rdp_free_rc4_decrypt_keys(rdp);
 	rdp_free_rc4_encrypt_keys(rdp);
 
@@ -2399,6 +2411,7 @@ static void rdp_reset_free(rdpRdp* rdp)
 	winpr_Cipher_Free(rdp->fips_decrypt);
 	rdp->fips_encrypt = NULL;
 	rdp->fips_decrypt = NULL;
+	(void)security_unlock(rdp);
 
 	mcs_free(rdp->mcs);
 	nego_free(rdp->nego);
@@ -2494,7 +2507,6 @@ void rdp_free(rdpRdp* rdp)
 {
 	if (rdp)
 	{
-		DeleteCriticalSection(&rdp->critical);
 		rdp_reset_free(rdp);
 
 		freerdp_settings_free(rdp->settings);
@@ -2514,6 +2526,8 @@ void rdp_free(rdpRdp* rdp)
 		if (rdp->abortEvent)
 			(void)CloseHandle(rdp->abortEvent);
 		aad_free(rdp->aad);
+		WINPR_JSON_Delete(rdp->wellknown);
+		DeleteCriticalSection(&rdp->critical);
 		free(rdp);
 	}
 }
@@ -2565,11 +2579,12 @@ void* rdp_get_io_callback_context(rdpRdp* rdp)
 const char* rdp_finalize_flags_to_str(UINT32 flags, char* buffer, size_t size)
 {
 	char number[32] = { 0 };
-	const UINT32 mask = ~(FINALIZE_SC_SYNCHRONIZE_PDU | FINALIZE_SC_CONTROL_COOPERATE_PDU |
-	                      FINALIZE_SC_CONTROL_GRANTED_PDU | FINALIZE_SC_FONT_MAP_PDU |
-	                      FINALIZE_CS_SYNCHRONIZE_PDU | FINALIZE_CS_CONTROL_COOPERATE_PDU |
-	                      FINALIZE_CS_CONTROL_REQUEST_PDU | FINALIZE_CS_PERSISTENT_KEY_LIST_PDU |
-	                      FINALIZE_CS_FONT_LIST_PDU | FINALIZE_DEACTIVATE_REACTIVATE);
+	const UINT32 mask =
+	    (uint32_t)~(FINALIZE_SC_SYNCHRONIZE_PDU | FINALIZE_SC_CONTROL_COOPERATE_PDU |
+	                FINALIZE_SC_CONTROL_GRANTED_PDU | FINALIZE_SC_FONT_MAP_PDU |
+	                FINALIZE_CS_SYNCHRONIZE_PDU | FINALIZE_CS_CONTROL_COOPERATE_PDU |
+	                FINALIZE_CS_CONTROL_REQUEST_PDU | FINALIZE_CS_PERSISTENT_KEY_LIST_PDU |
+	                FINALIZE_CS_FONT_LIST_PDU | FINALIZE_DEACTIVATE_REACTIVATE);
 
 	if (flags & FINALIZE_SC_SYNCHRONIZE_PDU)
 		winpr_str_append("FINALIZE_SC_SYNCHRONIZE_PDU", buffer, size, "|");
@@ -2782,6 +2797,20 @@ static BOOL parse_on_off_option(const char* value)
 
 #define STR(x) #x
 
+static BOOL option_is_runtime_checks(wLog* log, const char* tok)
+{
+	const char* experimental[] = { STR(WITH_VERBOSE_WINPR_ASSERT) };
+	for (size_t x = 0; x < ARRAYSIZE(experimental); x++)
+	{
+		const char* opt = experimental[x];
+		if (starts_with(tok, opt))
+		{
+			return parse_on_off_option(tok);
+		}
+	}
+	return FALSE;
+}
+
 static BOOL option_is_experimental(wLog* log, const char* tok)
 {
 	const char* experimental[] = { STR(WITH_DSP_EXPERIMENTAL), STR(WITH_VAAPI) };
@@ -2903,13 +2932,161 @@ static void log_build_warn(rdpRdp* rdp, const char* what, const char* msg,
 	free(list);
 }
 
+#define print_first_line(log, firstLine, what) \
+	print_first_line_int((log), (firstLine), (what), __FILE__, __func__, __LINE__)
+static void print_first_line_int(wLog* log, log_line_t* firstLine, const char* what,
+                                 const char* file, const char* fkt, size_t line)
+{
+	WINPR_ASSERT(firstLine);
+	if (!firstLine->fkt)
+	{
+		const DWORD level = WLOG_WARN;
+		if (WLog_IsLevelActive(log, level))
+		{
+			WLog_PrintMessage(log, WLOG_MESSAGE_TEXT, level, line, file, fkt,
+			                  "*************************************************");
+			WLog_PrintMessage(log, WLOG_MESSAGE_TEXT, level, line, file, fkt,
+			                  "[SSL] {%s} build or configuration missing:", what);
+		}
+		firstLine->line = line;
+		firstLine->file = file;
+		firstLine->fkt = fkt;
+		firstLine->level = level;
+	}
+}
+
+static void print_last_line(wLog* log, const log_line_t* firstLine)
+{
+	WINPR_ASSERT(firstLine);
+	if (firstLine->fkt)
+	{
+		if (WLog_IsLevelActive(log, firstLine->level))
+			WLog_PrintMessage(log, WLOG_MESSAGE_TEXT, firstLine->level, firstLine->line,
+			                  firstLine->file, firstLine->fkt,
+			                  "*************************************************");
+	}
+}
+
+static void log_build_warn_cipher(rdpRdp* rdp, log_line_t* firstLine, WINPR_CIPHER_TYPE md,
+                                  const char* what)
+{
+	BOOL haveCipher = FALSE;
+
+	char key[WINPR_CIPHER_MAX_KEY_LENGTH] = { 0 };
+	char iv[WINPR_CIPHER_MAX_IV_LENGTH] = { 0 };
+
+	/* RC4 only exists in the compatibility functions winpr_RC4_*
+	 * winpr_Cipher_* does not support that. */
+	if (md == WINPR_CIPHER_ARC4_128)
+	{
+		WINPR_RC4_CTX* enc = winpr_RC4_New(key, sizeof(key));
+		haveCipher = enc != NULL;
+		winpr_RC4_Free(enc);
+	}
+	else
+	{
+		WINPR_CIPHER_CTX* enc =
+		    winpr_Cipher_NewEx(md, WINPR_ENCRYPT, key, sizeof(key), iv, sizeof(iv));
+		WINPR_CIPHER_CTX* dec =
+		    winpr_Cipher_NewEx(md, WINPR_DECRYPT, key, sizeof(key), iv, sizeof(iv));
+		if (enc && dec)
+			haveCipher = TRUE;
+
+		winpr_Cipher_Free(enc);
+		winpr_Cipher_Free(dec);
+	}
+
+	if (!haveCipher)
+	{
+		print_first_line(rdp->log, firstLine, "Cipher");
+		WLog_Print(rdp->log, WLOG_WARN, "* %s: %s", winpr_cipher_type_to_string(md), what);
+	}
+}
+
+static void log_build_warn_hmac(rdpRdp* rdp, log_line_t* firstLine, WINPR_MD_TYPE md,
+                                const char* what)
+{
+	BOOL haveHmacX = FALSE;
+	WINPR_HMAC_CTX* hmac = winpr_HMAC_New();
+	if (hmac)
+	{
+		/* We need some key length, but there is no real limit here.
+		 * just take the cipher maximum key length as we already have that available.
+		 */
+		char key[WINPR_CIPHER_MAX_KEY_LENGTH] = { 0 };
+		haveHmacX = winpr_HMAC_Init(hmac, md, key, sizeof(key));
+	}
+	winpr_HMAC_Free(hmac);
+
+	if (!haveHmacX)
+	{
+		print_first_line(rdp->log, firstLine, "HMAC");
+		WLog_Print(rdp->log, WLOG_WARN, " * %s: %s", winpr_md_type_to_string(md), what);
+	}
+}
+
+static void log_build_warn_hash(rdpRdp* rdp, log_line_t* firstLine, WINPR_MD_TYPE md,
+                                const char* what)
+{
+	BOOL haveDigestX = FALSE;
+
+	WINPR_DIGEST_CTX* digest = winpr_Digest_New();
+	if (digest)
+		haveDigestX = winpr_Digest_Init(digest, md);
+	winpr_Digest_Free(digest);
+
+	if (!haveDigestX)
+	{
+		print_first_line(rdp->log, firstLine, "Digest");
+		WLog_Print(rdp->log, WLOG_WARN, " * %s: %s", winpr_md_type_to_string(md), what);
+	}
+}
+
+static void log_build_warn_ssl(rdpRdp* rdp)
+{
+	WINPR_ASSERT(rdp);
+
+	log_line_t firstHashLine = { 0 };
+	log_build_warn_hash(rdp, &firstHashLine, WINPR_MD_MD4, "NTLM support not available");
+	log_build_warn_hash(rdp, &firstHashLine, WINPR_MD_MD5,
+	                    "NTLM, assistance files with encrypted passwords, autoreconnect cookies, "
+	                    "licensing and RDP security will not work");
+	log_build_warn_hash(rdp, &firstHashLine, WINPR_MD_SHA1,
+	                    "assistance files with encrypted passwords, Kerberos, Smartcard Logon, RDP "
+	                    "security support not available");
+	log_build_warn_hash(
+	    rdp, &firstHashLine, WINPR_MD_SHA256,
+	    "file clipboard, AAD gateway, NLA security and certificates might not work");
+	print_last_line(rdp->log, &firstHashLine);
+
+	log_line_t firstHmacLine = { 0 };
+	log_build_warn_hmac(rdp, &firstHmacLine, WINPR_MD_MD5, "Autoreconnect cookie not supported");
+	log_build_warn_hmac(rdp, &firstHmacLine, WINPR_MD_SHA1, "RDP security not supported");
+	print_last_line(rdp->log, &firstHmacLine);
+
+	log_line_t firstCipherLine = { 0 };
+	log_build_warn_cipher(rdp, &firstCipherLine, WINPR_CIPHER_ARC4_128,
+	                      "assistance files with encrypted passwords, NTLM, RDP licensing and RDP "
+	                      "security will not work");
+	log_build_warn_cipher(rdp, &firstCipherLine, WINPR_CIPHER_DES_EDE3_CBC,
+	                      "RDP security FIPS mode will not work");
+	log_build_warn_cipher(
+	    rdp, &firstCipherLine, WINPR_CIPHER_AES_128_CBC,
+	    "assistance file encrypted LHTicket will not work and ARM gateway might not");
+	log_build_warn_cipher(rdp, &firstCipherLine, WINPR_CIPHER_AES_192_CBC,
+	                      "ARM gateway might not work");
+	log_build_warn_cipher(rdp, &firstCipherLine, WINPR_CIPHER_AES_256_CBC,
+	                      "ARM gateway might not work");
+	print_last_line(rdp->log, &firstCipherLine);
+}
+
 void rdp_log_build_warnings(rdpRdp* rdp)
 {
 	static unsigned count = 0;
 
 	WINPR_ASSERT(rdp);
 	/* Since this function is called in context creation routines stop logging
-	 * this issue repetedly. This is required for proxy, which would otherwise
+	 * this issue repeatedly. This is required for proxy, which would otherwise
 	 * spam the log with these. */
 	if (count > 0)
 		return;
@@ -2917,4 +3094,7 @@ void rdp_log_build_warnings(rdpRdp* rdp)
 	log_build_warn(rdp, "experimental", "might crash the application", option_is_experimental);
 	log_build_warn(rdp, "debug", "might leak sensitive information (credentials, ...)",
 	               option_is_debug);
+	log_build_warn(rdp, "runtime-check", "might slow down the application",
+	               option_is_runtime_checks);
+	log_build_warn_ssl(rdp);
 }
