@@ -24,10 +24,14 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <stdint.h>
 
+#include <winpr/wtypes.h>
 #include <winpr/crt.h>
 #include <winpr/synch.h>
 #include <winpr/stream.h>
+#include <winpr/assert.h>
+#include <winpr/cast.h>
 
 #include <freerdp/log.h>
 #include <freerdp/constants.h>
@@ -79,7 +83,8 @@ static BOOL wts_queue_receive_data(rdpPeerChannel* channel, const BYTE* Buffer,
 	if (!messageCtx)
 		return FALSE;
 
-	messageCtx->channelId = channel->channelId;
+	WINPR_ASSERT(channel->channelId <= UINT16_MAX);
+	messageCtx->channelId = (UINT16)channel->channelId;
 	messageCtx->length = Length;
 	messageCtx->offset = 0;
 	buffer = (BYTE*)(messageCtx + 1);
@@ -91,17 +96,19 @@ static BOOL wts_queue_send_item(rdpPeerChannel* channel, BYTE* Buffer, UINT32 Le
 {
 	BYTE* buffer = NULL;
 	UINT32 length = 0;
-	UINT16 channelId = 0;
+
 	WINPR_ASSERT(channel);
 	WINPR_ASSERT(channel->vcm);
 	buffer = Buffer;
 	length = Length;
-	channelId = channel->channelId;
+
+	WINPR_ASSERT(channel->channelId <= UINT16_MAX);
+	const UINT16 channelId = (UINT16)channel->channelId;
 	return MessageQueue_Post(channel->vcm->queue, (void*)(UINT_PTR)channelId, 0, (void*)buffer,
 	                         (void*)(UINT_PTR)length);
 }
 
-static int wts_read_variable_uint(wStream* s, int cbLen, UINT32* val)
+static unsigned wts_read_variable_uint(wStream* s, int cbLen, UINT32* val)
 {
 	WINPR_ASSERT(s);
 	WINPR_ASSERT(val);
@@ -198,15 +205,16 @@ static BOOL wts_read_drdynvc_create_response(rdpPeerChannel* channel, wStream* s
 static BOOL wts_read_drdynvc_data_first(rdpPeerChannel* channel, wStream* s, int cbLen,
                                         UINT32 length)
 {
-	int value = 0;
 	WINPR_ASSERT(channel);
 	WINPR_ASSERT(s);
-	value = wts_read_variable_uint(s, cbLen, &channel->dvc_total_length);
+	const UINT32 value = wts_read_variable_uint(s, cbLen, &channel->dvc_total_length);
 
 	if (value == 0)
 		return FALSE;
-
-	length -= value;
+	if (value > length)
+		length = 0;
+	else
+		length -= value;
 
 	if (length > channel->dvc_total_length)
 		return FALSE;
@@ -264,8 +272,6 @@ static void wts_read_drdynvc_close_response(rdpPeerChannel* channel)
 
 static BOOL wts_read_drdynvc_pdu(rdpPeerChannel* channel)
 {
-	UINT32 length = 0;
-	UINT8 value = 0;
 	UINT8 Cmd = 0;
 	UINT8 Sp = 0;
 	UINT8 cbChId = 0;
@@ -275,20 +281,20 @@ static BOOL wts_read_drdynvc_pdu(rdpPeerChannel* channel)
 	WINPR_ASSERT(channel);
 	WINPR_ASSERT(channel->vcm);
 
-	length = Stream_GetPosition(channel->receiveData);
+	size_t length = Stream_GetPosition(channel->receiveData);
 
-	if (length < 1)
+	if ((length < 1) || (length > UINT32_MAX))
 		return FALSE;
 
 	Stream_SetPosition(channel->receiveData, 0);
-	Stream_Read_UINT8(channel->receiveData, value);
+	const UINT8 value = Stream_Get_UINT8(channel->receiveData);
 	length--;
 	Cmd = (value & 0xf0) >> 4;
 	Sp = (value & 0x0c) >> 2;
 	cbChId = (value & 0x03) >> 0;
 
 	if (Cmd == CAPABILITY_REQUEST_PDU)
-		return wts_read_drdynvc_capabilities_response(channel, length);
+		return wts_read_drdynvc_capabilities_response(channel, (UINT32)length);
 
 	if (channel->vcm->drdynvc_state == DRDYNVC_STATE_READY)
 	{
@@ -306,13 +312,13 @@ static BOOL wts_read_drdynvc_pdu(rdpPeerChannel* channel)
 
 		if (haveChannelId)
 		{
-			value = wts_read_variable_uint(channel->receiveData, cbChId, &ChannelId);
-			if (value == 0)
+			const unsigned val = wts_read_variable_uint(channel->receiveData, cbChId, &ChannelId);
+			if (val == 0)
 				return FALSE;
 
-			length -= value;
+			length -= val;
 
-			DEBUG_DVC("Cmd %s ChannelId %" PRIu32 " length %" PRIu32 "",
+			DEBUG_DVC("Cmd %s ChannelId %" PRIu32 " length %" PRIuz "",
 			          drdynvc_get_packet_type(Cmd), ChannelId, length);
 			dvc = wts_get_dvc_channel_by_id(channel->vcm, ChannelId);
 			if (!dvc)
@@ -325,7 +331,7 @@ static BOOL wts_read_drdynvc_pdu(rdpPeerChannel* channel)
 		switch (Cmd)
 		{
 			case CREATE_REQUEST_PDU:
-				return wts_read_drdynvc_create_response(dvc, channel->receiveData, length);
+				return wts_read_drdynvc_create_response(dvc, channel->receiveData, (UINT32)length);
 
 			case DATA_FIRST_PDU:
 				if (dvc->dvc_open_state != DVC_OPEN_STATE_SUCCEEDED)
@@ -337,7 +343,7 @@ static BOOL wts_read_drdynvc_pdu(rdpPeerChannel* channel)
 					return TRUE;
 				}
 
-				return wts_read_drdynvc_data_first(dvc, channel->receiveData, Sp, length);
+				return wts_read_drdynvc_data_first(dvc, channel->receiveData, Sp, (UINT32)length);
 
 			case DATA_PDU:
 				if (dvc->dvc_open_state != DVC_OPEN_STATE_SUCCEEDED)
@@ -349,7 +355,7 @@ static BOOL wts_read_drdynvc_pdu(rdpPeerChannel* channel)
 					return TRUE;
 				}
 
-				return wts_read_drdynvc_data(dvc, channel->receiveData, length);
+				return wts_read_drdynvc_data(dvc, channel->receiveData, (UINT32)length);
 
 			case CLOSE_REQUEST_PDU:
 				wts_read_drdynvc_close_response(dvc);
@@ -390,12 +396,12 @@ static int wts_write_variable_uint(wStream* s, UINT32 val)
 	if (val <= 0xFF)
 	{
 		cb = 0;
-		Stream_Write_UINT8(s, val);
+		Stream_Write_UINT8(s, WINPR_ASSERTING_INT_CAST(uint8_t, val));
 	}
 	else if (val <= 0xFFFF)
 	{
 		cb = 1;
-		Stream_Write_UINT16(s, val);
+		Stream_Write_UINT16(s, WINPR_ASSERTING_INT_CAST(uint16_t, val));
 	}
 	else
 	{
@@ -416,7 +422,7 @@ static void wts_write_drdynvc_header(wStream* s, BYTE Cmd, UINT32 ChannelId)
 	Stream_GetPointer(s, bm);
 	Stream_Seek_UINT8(s);
 	cbChId = wts_write_variable_uint(s, ChannelId);
-	*bm = ((Cmd & 0x0F) << 4) | cbChId;
+	*bm = (((Cmd & 0x0F) << 4) | cbChId) & 0xFF;
 }
 
 static BOOL wts_write_drdynvc_create_request(wStream* s, UINT32 ChannelId, const char* ChannelName)
@@ -470,8 +476,12 @@ static BOOL WTSProcessChannelData(rdpPeerChannel* channel, UINT16 channelId, con
 		}
 		else
 		{
-			ret = wts_queue_receive_data(channel, Stream_Buffer(channel->receiveData),
-			                             Stream_GetPosition(channel->receiveData));
+			const size_t pos = Stream_GetPosition(channel->receiveData);
+			if (pos > UINT32_MAX)
+				ret = FALSE;
+			else
+				ret = wts_queue_receive_data(channel, Stream_Buffer(channel->receiveData),
+				                             (UINT32)pos);
 		}
 
 		Stream_SetPosition(channel->receiveData, 0);
@@ -561,8 +571,8 @@ BOOL WTSVirtualChannelManagerOpen(HANDLE hServer)
 
 		if (channel)
 		{
-			BYTE capaBuffer[12];
-			wStream staticS;
+			BYTE capaBuffer[12] = { 0 };
+			wStream staticS = { 0 };
 			wStream* s = Stream_StaticInit(&staticS, capaBuffer, sizeof(capaBuffer));
 
 			vcm->drdynvc_channel = channel;
@@ -573,9 +583,10 @@ BOOL WTSVirtualChannelManagerOpen(HANDLE hServer)
 
 			/* TODO: shall implement version 2 and 3 */
 
+			const size_t pos = Stream_GetPosition(s);
+			WINPR_ASSERT(pos <= UINT32_MAX);
 			ULONG written = 0;
-			if (!WTSVirtualChannelWrite(channel, (PCHAR)capaBuffer, Stream_GetPosition(s),
-			                            &written))
+			if (!WTSVirtualChannelWrite(channel, (PCHAR)capaBuffer, (UINT32)pos, &written))
 				return FALSE;
 		}
 	}
@@ -1314,7 +1325,9 @@ HANDLE WINAPI FreeRDP_WTSVirtualChannelOpen(HANDLE hServer, DWORD SessionId, LPS
 	{
 		const UINT32 VCChunkSize =
 		    freerdp_settings_get_uint32(context->settings, FreeRDP_VCChunkSize);
-		channel = channel_new(vcm, client, joined_channel->ChannelId, index,
+
+		WINPR_ASSERT(index <= UINT16_MAX);
+		channel = channel_new(vcm, client, joined_channel->ChannelId, (UINT16)index,
 		                      RDP_PEER_CHANNEL_TYPE_SVC, VCChunkSize, pVirtualName);
 
 		if (!channel)
@@ -1395,7 +1408,8 @@ HANDLE WINAPI FreeRDP_WTSVirtualChannelOpenEx(DWORD SessionId, LPSTR pVirtualNam
 		return NULL;
 	}
 
-	channel->channelId = InterlockedIncrement(&vcm->dvc_channel_id_seq);
+	const LONG hdl = InterlockedIncrement(&vcm->dvc_channel_id_seq);
+	channel->channelId = WINPR_ASSERTING_INT_CAST(uint32_t, hdl);
 
 	if (!HashTable_Insert(vcm->dynamicVirtualChannels, &channel->channelId, channel))
 	{
@@ -1411,8 +1425,10 @@ HANDLE WINAPI FreeRDP_WTSVirtualChannelOpenEx(DWORD SessionId, LPSTR pVirtualNam
 	if (!wts_write_drdynvc_create_request(s, channel->channelId, pVirtualName))
 		goto fail;
 
-	if (!WTSVirtualChannelWrite(vcm->drdynvc_channel, Stream_BufferAs(s, char),
-	                            Stream_GetPosition(s), &written))
+	const size_t pos = Stream_GetPosition(s);
+	WINPR_ASSERT(pos <= UINT32_MAX);
+	if (!WTSVirtualChannelWrite(vcm->drdynvc_channel, Stream_BufferAs(s, char), (UINT32)pos,
+	                            &written))
 		goto fail;
 
 	Stream_Free(s, TRUE);
@@ -1469,8 +1485,11 @@ BOOL WINAPI FreeRDP_WTSVirtualChannelClose(HANDLE hChannelHandle)
 				else
 				{
 					wts_write_drdynvc_header(s, CLOSE_REQUEST_PDU, channel->channelId);
+
+					const size_t pos = Stream_GetPosition(s);
+					WINPR_ASSERT(pos <= UINT32_MAX);
 					ret = WTSVirtualChannelWrite(vcm->drdynvc_channel, Stream_BufferAs(s, char),
-					                             Stream_GetPosition(s), &written);
+					                             (UINT32)pos, &written);
 					Stream_Free(s, TRUE);
 				}
 			}
@@ -1534,8 +1553,6 @@ BOOL WINAPI FreeRDP_WTSVirtualChannelWrite(HANDLE hChannelHandle, PCHAR Buffer,
 	int cbChId = 0;
 	int first = 0;
 	BYTE* buffer = NULL;
-	UINT32 length = 0;
-	UINT32 written = 0;
 	UINT32 totalWritten = 0;
 	rdpPeerChannel* channel = (rdpPeerChannel*)hChannelHandle;
 	BOOL ret = FALSE;
@@ -1547,7 +1564,7 @@ BOOL WINAPI FreeRDP_WTSVirtualChannelWrite(HANDLE hChannelHandle, PCHAR Buffer,
 	WINPR_ASSERT(channel->vcm);
 	if (channel->channelType == RDP_PEER_CHANNEL_TYPE_SVC)
 	{
-		length = Length;
+		const ULONG length = Length;
 		buffer = (BYTE*)malloc(length);
 
 		if (!buffer)
@@ -1592,26 +1609,28 @@ BOOL WINAPI FreeRDP_WTSVirtualChannelWrite(HANDLE hChannelHandle, PCHAR Buffer,
 			if (first && (Length > Stream_GetRemainingLength(s)))
 			{
 				cbLen = wts_write_variable_uint(s, Length);
-				buffer[0] = (DATA_FIRST_PDU << 4) | (cbLen << 2) | cbChId;
+				buffer[0] = ((DATA_FIRST_PDU << 4) | (cbLen << 2) | cbChId) & 0xFF;
 			}
 			else
 			{
-				buffer[0] = (DATA_PDU << 4) | cbChId;
+				buffer[0] = ((DATA_PDU << 4) | cbChId) & 0xFF;
 			}
 
 			first = FALSE;
-			written = Stream_GetRemainingLength(s);
+			size_t written = Stream_GetRemainingLength(s);
 
 			if (written > Length)
 				written = Length;
 
 			Stream_Write(s, Buffer, written);
-			length = Stream_GetPosition(s);
+			const size_t length = Stream_GetPosition(s);
 			Stream_Free(s, FALSE);
+			if (length > UINT32_MAX)
+				goto fail;
 			Length -= written;
 			Buffer += written;
 			totalWritten += written;
-			if (!wts_queue_send_item(channel->vcm->drdynvc_channel, buffer, length))
+			if (!wts_queue_send_item(channel->vcm->drdynvc_channel, buffer, (UINT32)length))
 				goto fail;
 		}
 	}
@@ -1668,7 +1687,7 @@ BOOL WINAPI FreeRDP_WTSVirtualChannelQuery(HANDLE hChannelHandle, WTS_VIRTUAL_CL
 			}
 			else
 			{
-				CopyMemory(*ppBuffer, &fds[0], sizeof(void*));
+				CopyMemory(*ppBuffer, (void*)&fds[0], sizeof(void*));
 				*pBytesReturned = sizeof(void*);
 				status = TRUE;
 			}
@@ -1686,7 +1705,7 @@ BOOL WINAPI FreeRDP_WTSVirtualChannelQuery(HANDLE hChannelHandle, WTS_VIRTUAL_CL
 			}
 			else
 			{
-				CopyMemory(*ppBuffer, &(hEvent), sizeof(HANDLE));
+				CopyMemory(*ppBuffer, (void*)&hEvent, sizeof(HANDLE));
 				*pBytesReturned = sizeof(void*);
 				status = TRUE;
 			}
