@@ -478,6 +478,8 @@ static int transport_bio_buffered_write(BIO* bio, const char* buf, int num)
 
 	WINPR_ASSERT(bio);
 	WINPR_ASSERT(ptr);
+	if (num < 0)
+		return num;
 
 	ptr->writeBlocked = FALSE;
 	BIO_clear_flags(bio, BIO_FLAGS_WRITE);
@@ -485,7 +487,7 @@ static int transport_bio_buffered_write(BIO* bio, const char* buf, int num)
 	/* we directly append extra bytes in the xmit buffer, this could be prevented
 	 * but for now it makes the code more simple.
 	 */
-	if (buf && num && !ringbuffer_write(&ptr->xmitBuffer, (const BYTE*)buf, num))
+	if (buf && (num > 0) && !ringbuffer_write(&ptr->xmitBuffer, (const BYTE*)buf, (size_t)num))
 	{
 		WLog_ERR(TAG, "an error occurred when writing (num: %d)", num);
 		return -1;
@@ -499,7 +501,9 @@ static int transport_bio_buffered_write(BIO* bio, const char* buf, int num)
 		while (chunks[i].size)
 		{
 			ERR_clear_error();
-			const int status = BIO_write(next_bio, chunks[i].data, chunks[i].size);
+
+			const size_t wr = MIN(INT32_MAX, chunks[i].size);
+			const int status = BIO_write(next_bio, chunks[i].data, (int)wr);
 
 			if (status <= 0)
 			{
@@ -589,7 +593,7 @@ static long transport_bio_buffered_ctrl(BIO* bio, int cmd, long arg1, void* arg2
 			break;
 
 		case BIO_CTRL_WPENDING:
-			status = ringbuffer_used(&ptr->xmitBuffer);
+			status = WINPR_ASSERTING_INT_CAST(long, ringbuffer_used(&ptr->xmitBuffer));
 			break;
 
 		case BIO_CTRL_PENDING:
@@ -727,7 +731,7 @@ char* freerdp_tcp_get_peer_address(SOCKET sockfd)
 	struct sockaddr_storage saddr = { 0 };
 	socklen_t length = sizeof(struct sockaddr_storage);
 
-	if (getpeername(sockfd, (struct sockaddr*)&saddr, &length) != 0)
+	if (getpeername((int)sockfd, (struct sockaddr*)&saddr, &length) != 0)
 	{
 		return NULL;
 	}
@@ -811,9 +815,8 @@ static BOOL freerdp_tcp_connect_timeout(rdpContext* context, int sockfd, struct
                                         socklen_t addrlen, UINT32 timeout)
 {
 	BOOL rc = FALSE;
-	HANDLE handles[2];
-	int status = 0;
-	int count = 0;
+	HANDLE handles[2] = { 0 };
+	DWORD count = 0;
 	u_long arg = 0;
 	DWORD tout = (timeout > 0) ? timeout : INFINITE;
 
@@ -822,22 +825,23 @@ static BOOL freerdp_tcp_connect_timeout(rdpContext* context, int sockfd, struct
 	if (!handles[count])
 		return FALSE;
 
-	status = WSAEventSelect(sockfd, handles[count++], FD_READ | FD_WRITE | FD_CONNECT | FD_CLOSE);
+	const int wsastatus = WSAEventSelect((SOCKET)sockfd, handles[count++],
+	                                     FD_READ | FD_WRITE | FD_CONNECT | FD_CLOSE);
 
-	if (status < 0)
+	if (wsastatus < 0)
 	{
 		WLog_ERR(TAG, "WSAEventSelect failed with %d", WSAGetLastError());
 		goto fail;
 	}
 
 	handles[count++] = utils_get_abort_event(context->rdp);
-	status = _connect(sockfd, addr, addrlen);
+	const int constatus = _connect((SOCKET)sockfd, addr, WINPR_ASSERTING_INT_CAST(int, addrlen));
 
-	if (status < 0)
+	if (constatus < 0)
 	{
-		status = WSAGetLastError();
+		const int estatus = WSAGetLastError();
 
-		switch (status)
+		switch (estatus)
 		{
 			case WSAEINPROGRESS:
 			case WSAEWOULDBLOCK:
@@ -848,20 +852,20 @@ static BOOL freerdp_tcp_connect_timeout(rdpContext* context, int sockfd, struct
 		}
 	}
 
-	status = WaitForMultipleObjects(count, handles, FALSE, tout);
+	const DWORD wstatus = WaitForMultipleObjects(count, handles, FALSE, tout);
 
-	if (WAIT_OBJECT_0 != status)
+	if (WAIT_OBJECT_0 != wstatus)
 		goto fail;
 
-	status = recv(sockfd, NULL, 0, 0);
+	const SSIZE_T res = recv(sockfd, NULL, 0, 0);
 
-	if (status == SOCKET_ERROR)
+	if (res == SOCKET_ERROR)
 	{
 		if (WSAGetLastError() == WSAECONNRESET)
 			goto fail;
 	}
 
-	status = WSAEventSelect(sockfd, handles[0], 0);
+	const int status = WSAEventSelect((SOCKET)sockfd, handles[0], 0);
 
 	if (status < 0)
 	{
@@ -869,7 +873,7 @@ static BOOL freerdp_tcp_connect_timeout(rdpContext* context, int sockfd, struct
 		goto fail;
 	}
 
-	if (_ioctlsocket(sockfd, FIONBIO, &arg) != 0)
+	if (_ioctlsocket((SOCKET)sockfd, FIONBIO, &arg) != 0)
 		goto fail;
 
 	rc = TRUE;
@@ -899,19 +903,17 @@ static int freerdp_tcp_connect_multi(rdpContext* context, char** hostnames, cons
                                      UINT32 count, UINT16 port, UINT32 timeout)
 {
 	UINT32 sindex = count;
-	int status = -1;
 	SOCKET sockfd = INVALID_SOCKET;
-	HANDLE* events = NULL;
 	struct addrinfo* addr = NULL;
 	struct addrinfo* result = NULL;
-	t_peer* peers = NULL;
-	events = (HANDLE*)calloc(count + 1, sizeof(HANDLE));
-	peers = (t_peer*)calloc(count, sizeof(t_peer));
+
+	HANDLE* events = (HANDLE*)calloc(count + 1, sizeof(HANDLE));
+	t_peer* peers = (t_peer*)calloc(count, sizeof(t_peer));
 
 	if (!peers || !events || (count < 1))
 	{
 		free(peers);
-		free(events);
+		free((void*)events);
 		return -1;
 	}
 
@@ -920,7 +922,7 @@ static int freerdp_tcp_connect_multi(rdpContext* context, char** hostnames, cons
 		int curPort = port;
 
 		if (ports)
-			curPort = ports[index];
+			curPort = WINPR_ASSERTING_INT_CAST(int, ports[index]);
 
 		result = freerdp_tcp_resolve_host(hostnames[index], curPort, 0);
 
@@ -962,9 +964,10 @@ static int freerdp_tcp_connect_multi(rdpContext* context, char** hostnames, cons
 			continue;
 
 		/* blocking tcp connect */
-		status = _connect(sockfd, addr->ai_addr, addr->ai_addrlen);
+		const int rc =
+		    _connect(sockfd, addr->ai_addr, WINPR_ASSERTING_INT_CAST(int, addr->ai_addrlen));
 
-		if (status >= 0)
+		if (rc >= 0)
 		{
 			/* connection success */
 			sindex = index;
@@ -984,8 +987,8 @@ static int freerdp_tcp_connect_multi(rdpContext* context, char** hostnames, cons
 		peer_free(&peers[index]);
 
 	free(peers);
-	free(events);
-	return sockfd;
+	free((void*)events);
+	return (int)sockfd;
 }
 
 BOOL freerdp_tcp_set_keep_alive_mode(const rdpSettings* settings, int sockfd)
@@ -1099,8 +1102,6 @@ static int get_next_addrinfo(rdpContext* context, struct addrinfo* input, struct
 			const int family = (IPvX == 4) ? AF_INET : AF_INET6;
 			while (addr && (addr->ai_family != family))
 				addr = addr->ai_next;
-			if (!addr)
-				goto fail;
 		}
 		break;
 		default:
@@ -1149,7 +1150,6 @@ int freerdp_tcp_default_connect(rdpContext* context, rdpSettings* settings, cons
 		if (sockfd < 0)
 		{
 			freerdp_set_last_error_if_not(context, FREERDP_ERROR_CONNECT_FAILED);
-
 			return -1;
 		}
 	}
@@ -1160,10 +1160,19 @@ int freerdp_tcp_default_connect(rdpContext* context, rdpSettings* settings, cons
 #if defined(HAVE_AF_VSOCK_H)
 		hostname = vsock;
 		sockfd = socket(AF_VSOCK, SOCK_STREAM, 0);
+		if (sockfd < 0)
+		{
+			char buffer[256] = { 0 };
+			WLog_WARN(TAG, "socket(AF_VSOCK, SOCK_STREAM, 0) failed with %s [%d]",
+			          winpr_strerror(errno, buffer, sizeof(buffer)));
+			freerdp_set_last_error_if_not(context, FREERDP_ERROR_CONNECT_FAILED);
+			return -1;
+		}
+
 		struct sockaddr_vm addr = { 0 };
 
 		addr.svm_family = AF_VSOCK;
-		addr.svm_port = port;
+		addr.svm_port = WINPR_ASSERTING_INT_CAST(typeof(addr.svm_port), port);
 
 		errno = 0;
 		char* ptr = NULL;
@@ -1175,7 +1184,7 @@ int freerdp_tcp_default_connect(rdpContext* context, rdpSettings* settings, cons
 			         winpr_strerror(errno, ebuffer, sizeof(ebuffer)));
 			return -1;
 		}
-		addr.svm_cid = val;
+		addr.svm_cid = WINPR_ASSERTING_INT_CAST(typeof(addr.svm_cid), val);
 		if (addr.svm_cid == 2)
 		{
 			addr.svm_flags = VMADDR_FLAG_TO_HOST;
@@ -1201,9 +1210,10 @@ int freerdp_tcp_default_connect(rdpContext* context, rdpSettings* settings, cons
 			{
 				if (settings->TargetNetAddressCount > 0)
 				{
+					WINPR_ASSERT(port <= UINT16_MAX);
 					sockfd = freerdp_tcp_connect_multi(
 					    context, settings->TargetNetAddresses, settings->TargetNetPorts,
-					    settings->TargetNetAddressCount, port, timeout);
+					    settings->TargetNetAddressCount, (UINT16)port, timeout);
 				}
 			}
 		}
@@ -1357,7 +1367,7 @@ static int freerdp_tcp_layer_read(void* userContext, void* data, int bytes)
 	int status = 0;
 
 	(void)WSAResetEvent(tcpLayer->hEvent);
-	status = _recv(tcpLayer->sockfd, data, bytes, 0);
+	status = _recv((SOCKET)tcpLayer->sockfd, data, bytes, 0);
 
 	if (status > 0)
 		return status;
@@ -1392,7 +1402,7 @@ static int freerdp_tcp_layer_write(void* userContext, const void* data, int byte
 	int error = 0;
 	int status = 0;
 
-	status = _send(tcpLayer->sockfd, data, bytes, 0);
+	status = _send((SOCKET)tcpLayer->sockfd, data, bytes, 0);
 
 	if (status <= 0)
 	{
@@ -1420,7 +1430,7 @@ static BOOL freerdp_tcp_layer_close(void* userContext)
 	rdpTcpLayer* tcpLayer = (rdpTcpLayer*)userContext;
 
 	if (tcpLayer->sockfd >= 0)
-		closesocket(tcpLayer->sockfd);
+		closesocket((SOCKET)tcpLayer->sockfd);
 	if (tcpLayer->hEvent)
 		(void)CloseHandle(tcpLayer->hEvent);
 
@@ -1517,7 +1527,7 @@ rdpTransportLayer* freerdp_tcp_connect_layer(rdpContext* context, const char* ho
 		goto fail;
 
 	/* WSAEventSelect automatically sets the socket in non-blocking mode */
-	if (WSAEventSelect(sockfd, tcpLayer->hEvent, FD_READ | FD_ACCEPT | FD_CLOSE))
+	if (WSAEventSelect((SOCKET)sockfd, tcpLayer->hEvent, FD_READ | FD_ACCEPT | FD_CLOSE))
 	{
 		WLog_ERR(TAG, "WSAEventSelect returned 0x%08X", WSAGetLastError());
 		goto fail;
@@ -1529,7 +1539,7 @@ rdpTransportLayer* freerdp_tcp_connect_layer(rdpContext* context, const char* ho
 
 fail:
 	if (sockfd >= 0)
-		closesocket(sockfd);
+		closesocket((SOCKET)sockfd);
 	transport_layer_free(layer);
 	return NULL;
 }
