@@ -26,6 +26,8 @@
 #include <winpr/synch.h>
 #include <winpr/thread.h>
 #include <winpr/collections.h>
+#include <winpr/assert.h>
+#include <winpr/cast.h>
 
 #include "settings.h"
 #include "update.h"
@@ -171,35 +173,39 @@ static BOOL update_write_bitmap_data(rdpUpdate* update_pub, wStream* s, BITMAP_D
 		}
 	}
 
-	Stream_Write_UINT16(s, bitmapData->destLeft);
-	Stream_Write_UINT16(s, bitmapData->destTop);
-	Stream_Write_UINT16(s, bitmapData->destRight);
-	Stream_Write_UINT16(s, bitmapData->destBottom);
-	Stream_Write_UINT16(s, bitmapData->width);
-	Stream_Write_UINT16(s, bitmapData->height);
-	Stream_Write_UINT16(s, bitmapData->bitsPerPixel);
-	Stream_Write_UINT16(s, bitmapData->flags);
-	Stream_Write_UINT16(s, bitmapData->bitmapLength);
+	Stream_Write_UINT16(s, WINPR_ASSERTING_INT_CAST(uint16_t, bitmapData->destLeft));
+	Stream_Write_UINT16(s, WINPR_ASSERTING_INT_CAST(uint16_t, bitmapData->destTop));
+	Stream_Write_UINT16(s, WINPR_ASSERTING_INT_CAST(uint16_t, bitmapData->destRight));
+	Stream_Write_UINT16(s, WINPR_ASSERTING_INT_CAST(uint16_t, bitmapData->destBottom));
+	Stream_Write_UINT16(s, WINPR_ASSERTING_INT_CAST(uint16_t, bitmapData->width));
+	Stream_Write_UINT16(s, WINPR_ASSERTING_INT_CAST(uint16_t, bitmapData->height));
+	Stream_Write_UINT16(s, WINPR_ASSERTING_INT_CAST(uint16_t, bitmapData->bitsPerPixel));
+	Stream_Write_UINT16(s, WINPR_ASSERTING_INT_CAST(uint16_t, bitmapData->flags));
+	Stream_Write_UINT16(s, WINPR_ASSERTING_INT_CAST(uint16_t, bitmapData->bitmapLength));
 
 	if (bitmapData->flags & BITMAP_COMPRESSION)
 	{
 		if (!(bitmapData->flags & NO_BITMAP_COMPRESSION_HDR))
 		{
-			Stream_Write_UINT16(s,
-			                    bitmapData->cbCompFirstRowSize); /* cbCompFirstRowSize (2 bytes) */
-			Stream_Write_UINT16(s,
-			                    bitmapData->cbCompMainBodySize); /* cbCompMainBodySize (2 bytes) */
-			Stream_Write_UINT16(s, bitmapData->cbScanWidth);     /* cbScanWidth (2 bytes) */
-			Stream_Write_UINT16(s,
-			                    bitmapData->cbUncompressedSize); /* cbUncompressedSize (2 bytes) */
+			Stream_Write_UINT16(
+			    s,
+			    WINPR_ASSERTING_INT_CAST(
+			        uint16_t, bitmapData->cbCompFirstRowSize)); /* cbCompFirstRowSize (2 bytes) */
+			Stream_Write_UINT16(
+			    s,
+			    WINPR_ASSERTING_INT_CAST(
+			        uint16_t, bitmapData->cbCompMainBodySize)); /* cbCompMainBodySize (2 bytes) */
+			Stream_Write_UINT16(
+			    s, WINPR_ASSERTING_INT_CAST(uint16_t,
+			                                bitmapData->cbScanWidth)); /* cbScanWidth (2 bytes) */
+			Stream_Write_UINT16(
+			    s,
+			    WINPR_ASSERTING_INT_CAST(
+			        uint16_t, bitmapData->cbUncompressedSize)); /* cbUncompressedSize (2 bytes) */
 		}
+	}
 
-		Stream_Write(s, bitmapData->bitmapDataStream, bitmapData->bitmapLength);
-	}
-	else
-	{
-		Stream_Write(s, bitmapData->bitmapDataStream, bitmapData->bitmapLength);
-	}
+	Stream_Write(s, bitmapData->bitmapDataStream, bitmapData->bitmapLength);
 
 	return TRUE;
 }
@@ -249,7 +255,8 @@ static BOOL update_write_bitmap_update(rdpUpdate* update, wStream* s,
 		return FALSE;
 
 	Stream_Write_UINT16(s, UPDATE_TYPE_BITMAP);   /* updateType */
-	Stream_Write_UINT16(s, bitmapUpdate->number); /* numberRectangles (2 bytes) */
+	Stream_Write_UINT16(s, WINPR_ASSERTING_INT_CAST(
+	                           uint16_t, bitmapUpdate->number)); /* numberRectangles (2 bytes) */
 
 	/* rectangles */
 	for (UINT32 i = 0; i < bitmapUpdate->number; i++)
@@ -676,8 +683,9 @@ POINTER_NEW_UPDATE* update_read_pointer_new(rdpUpdate* update, wStream* s)
 		goto fail;
 	}
 
+	WINPR_ASSERT(pointer_new->xorBpp <= UINT8_MAX);
 	if (!s_update_read_pointer_color(
-	        s, &pointer_new->colorPtrAttr, pointer_new->xorBpp,
+	        s, &pointer_new->colorPtrAttr, (UINT8)pointer_new->xorBpp,
 	        update->context->settings->LargePointerFlag)) /* colorPtrAttr */
 		goto fail;
 
@@ -1041,7 +1049,7 @@ static BOOL s_update_end_paint(rdpContext* context)
 	return TRUE;
 }
 
-static void update_flush(rdpContext* context)
+static BOOL update_flush(rdpContext* context)
 {
 	rdp_update_internal* update = NULL;
 
@@ -1050,39 +1058,42 @@ static void update_flush(rdpContext* context)
 
 	if (update->numberOrders > 0)
 	{
-		update_end_paint(&update->common);
-		update_begin_paint(&update->common);
+		if (!update_end_paint(&update->common))
+			return FALSE;
+
+		if (!update_begin_paint(&update->common))
+			return FALSE;
 	}
+	return TRUE;
 }
 
-static void update_force_flush(rdpContext* context)
+static BOOL update_force_flush(rdpContext* context)
 {
-	update_flush(context);
+	return update_flush(context);
 }
 
 static BOOL update_check_flush(rdpContext* context, size_t size)
 {
-	wStream* s = NULL;
-	rdp_update_internal* update = NULL;
-
 	WINPR_ASSERT(context);
-	update = update_cast(context->update);
+	rdp_update_internal* update = update_cast(context->update);
 
-	s = update->us;
+	wStream* s = update->us;
 
-	if (!update->us)
+	if (!s)
 	{
-		update_begin_paint(&update->common);
-		return FALSE;
+		if (!update_begin_paint(&update->common))
+			return FALSE;
+		s = update->us;
 	}
 
-	if (Stream_GetPosition(s) + size + 64 >= 0x3FFF)
+	if (Stream_GetPosition(s) + size + 64 >= FASTPATH_MAX_PACKET_SIZE)
 	{
-		update_flush(context);
-		return TRUE;
+		// Too big for the current packet. Flush first
+		if (!update_flush(context))
+			return FALSE;
 	}
 
-	return FALSE;
+	return TRUE;
 }
 
 static BOOL update_set_bounds(rdpContext* context, const rdpBounds* bounds)
@@ -1124,9 +1135,9 @@ static BOOL update_bounds_equals(rdpBounds* bounds1, rdpBounds* bounds2)
 	return FALSE;
 }
 
-static int update_prepare_bounds(rdpContext* context, ORDER_INFO* orderInfo)
+static size_t update_prepare_bounds(rdpContext* context, ORDER_INFO* orderInfo)
 {
-	int length = 0;
+	size_t length = 0;
 	rdp_update_internal* update = NULL;
 
 	WINPR_ASSERT(context);
@@ -1182,10 +1193,9 @@ static int update_prepare_bounds(rdpContext* context, ORDER_INFO* orderInfo)
 	return length;
 }
 
-static int update_prepare_order_info(rdpContext* context, ORDER_INFO* orderInfo, UINT32 orderType)
+static size_t update_prepare_order_info(rdpContext* context, ORDER_INFO* orderInfo,
+                                        UINT32 orderType)
 {
-	int length = 1;
-
 	WINPR_ASSERT(context);
 	WINPR_ASSERT(orderInfo);
 
@@ -1193,29 +1203,31 @@ static int update_prepare_order_info(rdpContext* context, ORDER_INFO* orderInfo,
 	orderInfo->orderType = orderType;
 	orderInfo->controlFlags = ORDER_STANDARD;
 	orderInfo->controlFlags |= ORDER_TYPE_CHANGE;
-	length += 1;
+	size_t length = 2;
 	length += get_primary_drawing_order_field_bytes(orderInfo->orderType, NULL);
 	length += update_prepare_bounds(context, orderInfo);
 	return length;
 }
 
-static int update_write_order_info(rdpContext* context, wStream* s, ORDER_INFO* orderInfo,
+static int update_write_order_info(rdpContext* context, wStream* s, const ORDER_INFO* orderInfo,
                                    size_t offset)
 {
-	size_t position = 0;
-
 	WINPR_UNUSED(context);
 	WINPR_ASSERT(orderInfo);
+	WINPR_ASSERT(orderInfo->controlFlags <= UINT8_MAX);
+
+	const size_t position = Stream_GetPosition(s);
+	const UINT8 controlFlags = (UINT8)orderInfo->controlFlags;
 
-	position = Stream_GetPosition(s);
 	Stream_SetPosition(s, offset);
-	Stream_Write_UINT8(s, orderInfo->controlFlags); /* controlFlags (1 byte) */
+	Stream_Write_UINT8(s, controlFlags); /* controlFlags (1 byte) */
 
 	if (orderInfo->controlFlags & ORDER_TYPE_CHANGE)
-		Stream_Write_UINT8(s, orderInfo->orderType); /* orderType (1 byte) */
+		Stream_Write_UINT8(
+		    s, WINPR_ASSERTING_INT_CAST(uint8_t, orderInfo->orderType)); /* orderType (1 byte) */
 
 	if (!update_write_field_flags(
-	        s, orderInfo->fieldFlags, orderInfo->controlFlags,
+	        s, orderInfo->fieldFlags, controlFlags,
 	        get_primary_drawing_order_field_bytes(orderInfo->orderType, NULL)))
 		return -1;
 	if (!update_write_bounds(s, orderInfo))
@@ -1338,7 +1350,8 @@ static BOOL update_send_surface_bits(rdpContext* context,
 	WINPR_ASSERT(surfaceBitsCommand);
 	WINPR_ASSERT(rdp);
 
-	update_force_flush(context);
+	if (!update_force_flush(context))
+		return FALSE;
 	s = fastpath_update_pdu_init(rdp->fastpath);
 
 	if (!s)
@@ -1351,8 +1364,7 @@ static BOOL update_send_surface_bits(rdpContext* context,
 	                              surfaceBitsCommand->skipCompression))
 		goto out_fail;
 
-	update_force_flush(context);
-	ret = TRUE;
+	ret = update_force_flush(context);
 out_fail:
 	Stream_Release(s);
 	return ret;
@@ -1365,7 +1377,8 @@ static BOOL update_send_surface_frame_marker(rdpContext* context,
 	WINPR_ASSERT(context);
 	rdpRdp* rdp = context->rdp;
 	BOOL ret = FALSE;
-	update_force_flush(context);
+	if (!update_force_flush(context))
+		return FALSE;
 
 	WINPR_ASSERT(rdp);
 	s = fastpath_update_pdu_init(rdp->fastpath);
@@ -1373,13 +1386,13 @@ static BOOL update_send_surface_frame_marker(rdpContext* context,
 	if (!s)
 		return FALSE;
 
-	if (!update_write_surfcmd_frame_marker(s, surfaceFrameMarker->frameAction,
+	WINPR_ASSERT(surfaceFrameMarker->frameAction <= UINT16_MAX);
+	if (!update_write_surfcmd_frame_marker(s, (UINT16)surfaceFrameMarker->frameAction,
 	                                       surfaceFrameMarker->frameId) ||
 	    !fastpath_send_update_pdu(rdp->fastpath, FASTPATH_UPDATETYPE_SURFCMDS, s, FALSE))
 		goto out_fail;
 
-	update_force_flush(context);
-	ret = TRUE;
+	ret = update_force_flush(context);
 out_fail:
 	Stream_Release(s);
 	return ret;
@@ -1394,7 +1407,8 @@ static BOOL update_send_surface_frame_bits(rdpContext* context, const SURFACE_BI
 	rdpRdp* rdp = context->rdp;
 	BOOL ret = FALSE;
 
-	update_force_flush(context);
+	if (!update_force_flush(context))
+		return FALSE;
 
 	WINPR_ASSERT(rdp);
 	s = fastpath_update_pdu_init(rdp->fastpath);
@@ -1419,7 +1433,10 @@ static BOOL update_send_surface_frame_bits(rdpContext* context, const SURFACE_BI
 
 	ret = fastpath_send_update_pdu(rdp->fastpath, FASTPATH_UPDATETYPE_SURFCMDS, s,
 	                               cmd->skipCompression);
-	update_force_flush(context);
+	if (!ret)
+		goto out_fail;
+
+	ret = update_force_flush(context);
 out_fail:
 	Stream_Release(s);
 	return ret;
@@ -1479,7 +1496,8 @@ static BOOL update_send_bitmap_update(rdpContext* context, const BITMAP_UPDATE*
 	rdpUpdate* update = context->update;
 	BOOL ret = TRUE;
 
-	update_force_flush(context);
+	if (!update_force_flush(context))
+		return FALSE;
 
 	WINPR_ASSERT(rdp);
 	s = fastpath_update_pdu_init(rdp->fastpath);
@@ -1495,7 +1513,8 @@ static BOOL update_send_bitmap_update(rdpContext* context, const BITMAP_UPDATE*
 		goto out_fail;
 	}
 
-	update_force_flush(context);
+	ret = update_force_flush(context);
+
 out_fail:
 	Stream_Release(s);
 	return ret;
@@ -1538,9 +1557,11 @@ static BOOL update_send_dstblt(rdpContext* context, const DSTBLT_ORDER* dstblt)
 
 	rdp_update_internal* update = update_cast(context->update);
 
-	const int headerLength = update_prepare_order_info(context, &orderInfo, ORDER_TYPE_DSTBLT);
+	const size_t headerLength = update_prepare_order_info(context, &orderInfo, ORDER_TYPE_DSTBLT);
 	const size_t inf = update_approximate_dstblt_order(&orderInfo, dstblt);
-	update_check_flush(context, headerLength + inf);
+	if (!update_check_flush(context, headerLength + inf))
+		return FALSE;
+
 	wStream* s = update->us;
 
 	if (!s)
@@ -1563,19 +1584,19 @@ static BOOL update_send_dstblt(rdpContext* context, const DSTBLT_ORDER* dstblt)
 
 static BOOL update_send_patblt(rdpContext* context, PATBLT_ORDER* patblt)
 {
-	wStream* s = NULL;
 	size_t offset = 0;
-	int headerLength = 0;
-	ORDER_INFO orderInfo;
-	rdp_update_internal* update = NULL;
+	ORDER_INFO orderInfo = { 0 };
 
 	WINPR_ASSERT(context);
 	WINPR_ASSERT(patblt);
-	update = update_cast(context->update);
+	rdp_update_internal* update = update_cast(context->update);
 
-	headerLength = update_prepare_order_info(context, &orderInfo, ORDER_TYPE_PATBLT);
-	update_check_flush(context, headerLength + update_approximate_patblt_order(&orderInfo, patblt));
-	s = update->us;
+	const size_t headerLength = update_prepare_order_info(context, &orderInfo, ORDER_TYPE_PATBLT);
+	if (!update_check_flush(context,
+	                        headerLength + update_approximate_patblt_order(&orderInfo, patblt)))
+		return FALSE;
+
+	wStream* s = update->us;
 
 	if (!s)
 		return FALSE;
@@ -1600,9 +1621,11 @@ static BOOL update_send_scrblt(rdpContext* context, const SCRBLT_ORDER* scrblt)
 	WINPR_ASSERT(scrblt);
 	rdp_update_internal* update = update_cast(context->update);
 
-	const int headerLength = update_prepare_order_info(context, &orderInfo, ORDER_TYPE_SCRBLT);
+	const size_t headerLength = update_prepare_order_info(context, &orderInfo, ORDER_TYPE_SCRBLT);
 	const size_t inf = update_approximate_scrblt_order(&orderInfo, scrblt);
-	update_check_flush(context, headerLength + inf);
+	if (!update_check_flush(context, headerLength + inf))
+		return FALSE;
+
 	wStream* s = update->us;
 
 	if (!s)
@@ -1622,20 +1645,20 @@ static BOOL update_send_scrblt(rdpContext* context, const SCRBLT_ORDER* scrblt)
 
 static BOOL update_send_opaque_rect(rdpContext* context, const OPAQUE_RECT_ORDER* opaque_rect)
 {
-	wStream* s = NULL;
 	size_t offset = 0;
-	int headerLength = 0;
-	ORDER_INFO orderInfo;
-	rdp_update_internal* update = NULL;
+	ORDER_INFO orderInfo = { 0 };
 
 	WINPR_ASSERT(context);
 	WINPR_ASSERT(opaque_rect);
-	update = update_cast(context->update);
+	rdp_update_internal* update = update_cast(context->update);
 
-	headerLength = update_prepare_order_info(context, &orderInfo, ORDER_TYPE_OPAQUE_RECT);
-	update_check_flush(context, headerLength +
-	                                update_approximate_opaque_rect_order(&orderInfo, opaque_rect));
-	s = update->us;
+	const size_t headerLength =
+	    update_prepare_order_info(context, &orderInfo, ORDER_TYPE_OPAQUE_RECT);
+	if (!update_check_flush(
+	        context, headerLength + update_approximate_opaque_rect_order(&orderInfo, opaque_rect)))
+		return FALSE;
+
+	wStream* s = update->us;
 
 	if (!s)
 		return FALSE;
@@ -1659,9 +1682,11 @@ static BOOL update_send_line_to(rdpContext* context, const LINE_TO_ORDER* line_t
 	WINPR_ASSERT(context);
 	WINPR_ASSERT(line_to);
 	rdp_update_internal* update = update_cast(context->update);
-	const int headerLength = update_prepare_order_info(context, &orderInfo, ORDER_TYPE_LINE_TO);
+	const size_t headerLength = update_prepare_order_info(context, &orderInfo, ORDER_TYPE_LINE_TO);
 	const size_t inf = update_approximate_line_to_order(&orderInfo, line_to);
-	update_check_flush(context, headerLength + inf);
+	if (!update_check_flush(context, headerLength + inf))
+		return FALSE;
+
 	wStream* s = update->us;
 
 	if (!s)
@@ -1681,18 +1706,18 @@ static BOOL update_send_line_to(rdpContext* context, const LINE_TO_ORDER* line_t
 
 static BOOL update_send_memblt(rdpContext* context, MEMBLT_ORDER* memblt)
 {
-	wStream* s = NULL;
 	size_t offset = 0;
-	int headerLength = 0;
-	ORDER_INFO orderInfo;
-	rdp_update_internal* update = NULL;
+	ORDER_INFO orderInfo = { 0 };
 
 	WINPR_ASSERT(context);
 	WINPR_ASSERT(memblt);
-	update = update_cast(context->update);
-	headerLength = update_prepare_order_info(context, &orderInfo, ORDER_TYPE_MEMBLT);
-	update_check_flush(context, headerLength + update_approximate_memblt_order(&orderInfo, memblt));
-	s = update->us;
+	rdp_update_internal* update = update_cast(context->update);
+	const size_t headerLength = update_prepare_order_info(context, &orderInfo, ORDER_TYPE_MEMBLT);
+	if (!update_check_flush(context,
+	                        headerLength + update_approximate_memblt_order(&orderInfo, memblt)))
+		return FALSE;
+
+	wStream* s = update->us;
 
 	if (!s)
 		return FALSE;
@@ -1717,9 +1742,12 @@ static BOOL update_send_glyph_index(rdpContext* context, GLYPH_INDEX_ORDER* glyp
 	WINPR_ASSERT(glyph_index);
 	rdp_update_internal* update = update_cast(context->update);
 
-	const int headerLength = update_prepare_order_info(context, &orderInfo, ORDER_TYPE_GLYPH_INDEX);
+	const size_t headerLength =
+	    update_prepare_order_info(context, &orderInfo, ORDER_TYPE_GLYPH_INDEX);
 	const size_t inf = update_approximate_glyph_index_order(&orderInfo, glyph_index);
-	update_check_flush(context, headerLength + inf);
+	if (!update_check_flush(context, headerLength + inf))
+		return FALSE;
+
 	wStream* s = update->us;
 
 	if (!s)
@@ -1754,7 +1782,9 @@ static BOOL update_send_cache_bitmap(rdpContext* context, const CACHE_BITMAP_ORD
 	                                                : ORDER_TYPE_BITMAP_UNCOMPRESSED;
 	const size_t inf =
 	    update_approximate_cache_bitmap_order(cache_bitmap, cache_bitmap->compressed, &extraFlags);
-	update_check_flush(context, headerLength + inf);
+	if (!update_check_flush(context, headerLength + inf))
+		return FALSE;
+
 	wStream* s = update->us;
 
 	if (!s)
@@ -1800,9 +1830,11 @@ static BOOL update_send_cache_bitmap_v2(rdpContext* context, CACHE_BITMAP_V2_ORD
 	if (context->settings->NoBitmapCompressionHeader)
 		cache_bitmap_v2->flags |= CBR2_NO_BITMAP_COMPRESSION_HDR;
 
-	update_check_flush(context, headerLength +
-	                                update_approximate_cache_bitmap_v2_order(
-	                                    cache_bitmap_v2, cache_bitmap_v2->compressed, &extraFlags));
+	if (!update_check_flush(
+	        context, headerLength + update_approximate_cache_bitmap_v2_order(
+	                                    cache_bitmap_v2, cache_bitmap_v2->compressed, &extraFlags)))
+		return FALSE;
+
 	wStream* s = update->us;
 
 	if (!s)
@@ -1844,8 +1876,10 @@ static BOOL update_send_cache_bitmap_v3(rdpContext* context, CACHE_BITMAP_V3_ORD
 	rdp_update_internal* update = update_cast(context->update);
 
 	const BYTE orderType = ORDER_TYPE_BITMAP_COMPRESSED_V3;
-	update_check_flush(context, headerLength + update_approximate_cache_bitmap_v3_order(
-	                                               cache_bitmap_v3, &extraFlags));
+	if (!update_check_flush(context, headerLength + update_approximate_cache_bitmap_v3_order(
+	                                                    cache_bitmap_v3, &extraFlags)))
+		return FALSE;
+
 	wStream* s = update->us;
 
 	if (!s)
@@ -1887,7 +1921,9 @@ static BOOL update_send_cache_color_table(rdpContext* context,
 	rdp_update_internal* update = update_cast(context->update);
 
 	const size_t inf = update_approximate_cache_color_table_order(cache_color_table, &flags);
-	update_check_flush(context, headerLength + inf);
+	if (!update_check_flush(context, headerLength + inf))
+		return FALSE;
+
 	wStream* s = update->us;
 
 	if (!s)
@@ -1927,7 +1963,9 @@ static BOOL update_send_cache_glyph(rdpContext* context, const CACHE_GLYPH_ORDER
 	rdp_update_internal* update = update_cast(context->update);
 
 	const size_t inf = update_approximate_cache_glyph_order(cache_glyph, &flags);
-	update_check_flush(context, headerLength + inf);
+	if (!update_check_flush(context, headerLength + inf))
+		return FALSE;
+
 	wStream* s = update->us;
 
 	if (!s)
@@ -1968,7 +2006,9 @@ static BOOL update_send_cache_glyph_v2(rdpContext* context,
 	rdp_update_internal* update = update_cast(context->update);
 
 	const size_t inf = update_approximate_cache_glyph_v2_order(cache_glyph_v2, &flags);
-	update_check_flush(context, headerLength + inf);
+	if (!update_check_flush(context, headerLength + inf))
+		return FALSE;
+
 	wStream* s = update->us;
 
 	if (!s)
@@ -2008,7 +2048,9 @@ static BOOL update_send_cache_brush(rdpContext* context, const CACHE_BRUSH_ORDER
 	rdp_update_internal* update = update_cast(context->update);
 
 	const size_t inf = update_approximate_cache_brush_order(cache_brush, &flags);
-	update_check_flush(context, headerLength + inf);
+	if (!update_check_flush(context, headerLength + inf))
+		return FALSE;
+
 	wStream* s = update->us;
 
 	if (!s)
@@ -2047,31 +2089,23 @@ static BOOL update_send_cache_brush(rdpContext* context, const CACHE_BRUSH_ORDER
 static BOOL update_send_create_offscreen_bitmap_order(
     rdpContext* context, const CREATE_OFFSCREEN_BITMAP_ORDER* create_offscreen_bitmap)
 {
-	wStream* s = NULL;
-	size_t bm = 0;
-	size_t em = 0;
-	size_t inf = 0;
-	BYTE orderType = 0;
-	BYTE controlFlags = 0;
-	size_t headerLength = 0;
-	rdp_update_internal* update = NULL;
-
 	WINPR_ASSERT(context);
 	WINPR_ASSERT(create_offscreen_bitmap);
-	update = update_cast(context->update);
+	rdp_update_internal* update = update_cast(context->update);
 
-	headerLength = 1;
-	orderType = ORDER_TYPE_CREATE_OFFSCREEN_BITMAP;
-	controlFlags = ORDER_SECONDARY | (orderType << 2);
-	inf = update_approximate_create_offscreen_bitmap_order(create_offscreen_bitmap);
-	update_check_flush(context, headerLength + inf);
+	const size_t headerLength = 1;
+	const size_t orderType = ORDER_TYPE_CREATE_OFFSCREEN_BITMAP;
+	const size_t controlFlags = ORDER_SECONDARY | (orderType << 2);
+	const size_t inf = update_approximate_create_offscreen_bitmap_order(create_offscreen_bitmap);
+	if (!update_check_flush(context, headerLength + inf))
+		return FALSE;
 
-	s = update->us;
+	wStream* s = update->us;
 
 	if (!s)
 		return FALSE;
 
-	bm = Stream_GetPosition(s);
+	const size_t bm = Stream_GetPosition(s);
 
 	if (!Stream_EnsureRemainingCapacity(s, headerLength))
 		return FALSE;
@@ -2081,7 +2115,7 @@ static BOOL update_send_create_offscreen_bitmap_order(
 	if (!update_write_create_offscreen_bitmap_order(s, create_offscreen_bitmap))
 		return FALSE;
 
-	em = Stream_GetPosition(s);
+	const size_t em = Stream_GetPosition(s);
 	Stream_SetPosition(s, bm);
 	Stream_Write_UINT8(s, controlFlags); /* controlFlags (1 byte) */
 	Stream_SetPosition(s, em);
@@ -2092,30 +2126,23 @@ static BOOL update_send_create_offscreen_bitmap_order(
 static BOOL update_send_switch_surface_order(rdpContext* context,
                                              const SWITCH_SURFACE_ORDER* switch_surface)
 {
-	wStream* s = NULL;
-	size_t bm = 0;
-	size_t em = 0;
-	size_t inf = 0;
-	BYTE orderType = 0;
-	BYTE controlFlags = 0;
-	size_t headerLength = 0;
-	rdp_update_internal* update = NULL;
-
 	WINPR_ASSERT(context);
 	WINPR_ASSERT(switch_surface);
-	update = update_cast(context->update);
+	rdp_update_internal* update = update_cast(context->update);
 
-	headerLength = 1;
-	orderType = ORDER_TYPE_SWITCH_SURFACE;
-	controlFlags = ORDER_SECONDARY | (orderType << 2);
-	inf = update_approximate_switch_surface_order(switch_surface);
-	update_check_flush(context, headerLength + inf);
-	s = update->us;
+	const size_t headerLength = 1;
+	const size_t orderType = ORDER_TYPE_SWITCH_SURFACE;
+	const size_t controlFlags = ORDER_SECONDARY | (orderType << 2);
+	const size_t inf = update_approximate_switch_surface_order(switch_surface);
+	if (!update_check_flush(context, headerLength + inf))
+		return FALSE;
+
+	wStream* s = update->us;
 
 	if (!s)
 		return FALSE;
 
-	bm = Stream_GetPosition(s);
+	const size_t bm = Stream_GetPosition(s);
 
 	if (!Stream_EnsureRemainingCapacity(s, headerLength))
 		return FALSE;
@@ -2125,7 +2152,7 @@ static BOOL update_send_switch_surface_order(rdpContext* context,
 	if (!update_write_switch_surface_order(s, switch_surface))
 		return FALSE;
 
-	em = Stream_GetPosition(s);
+	const size_t em = Stream_GetPosition(s);
 	Stream_SetPosition(s, bm);
 	Stream_Write_UINT8(s, controlFlags); /* controlFlags (1 byte) */
 	Stream_SetPosition(s, em);
@@ -2176,8 +2203,10 @@ static BOOL update_send_pointer_position(rdpContext* context,
 	if (!Stream_EnsureRemainingCapacity(s, 16))
 		goto out_fail;
 
-	Stream_Write_UINT16(s, pointerPosition->xPos); /* xPos (2 bytes) */
-	Stream_Write_UINT16(s, pointerPosition->yPos); /* yPos (2 bytes) */
+	Stream_Write_UINT16(
+	    s, WINPR_ASSERTING_INT_CAST(uint16_t, pointerPosition->xPos)); /* xPos (2 bytes) */
+	Stream_Write_UINT16(
+	    s, WINPR_ASSERTING_INT_CAST(uint16_t, pointerPosition->yPos)); /* yPos (2 bytes) */
 	ret = fastpath_send_update_pdu(rdp->fastpath, FASTPATH_UPDATETYPE_PTR_POSITION, s, FALSE);
 out_fail:
 	Stream_Release(s);
@@ -2296,7 +2325,8 @@ static BOOL update_send_pointer_new(rdpContext* context, const POINTER_NEW_UPDAT
 	if (!Stream_EnsureRemainingCapacity(s, 16))
 		goto out_fail;
 
-	Stream_Write_UINT16(s, pointer_new->xorBpp); /* xorBpp (2 bytes) */
+	Stream_Write_UINT16(
+	    s, WINPR_ASSERTING_INT_CAST(uint16_t, pointer_new->xorBpp)); /* xorBpp (2 bytes) */
 	update_write_pointer_color(s, &pointer_new->colorPtrAttr);
 	ret = fastpath_send_update_pdu(rdp->fastpath, FASTPATH_UPDATETYPE_POINTER, s, FALSE);
 out_fail:
@@ -2320,7 +2350,8 @@ static BOOL update_send_pointer_cached(rdpContext* context,
 	if (!s)
 		return FALSE;
 
-	Stream_Write_UINT16(s, pointer_cached->cacheIndex); /* cacheIndex (2 bytes) */
+	Stream_Write_UINT16(s, WINPR_ASSERTING_INT_CAST(
+	                           uint16_t, pointer_cached->cacheIndex)); /* cacheIndex (2 bytes) */
 	ret = fastpath_send_update_pdu(rdp->fastpath, FASTPATH_UPDATETYPE_CACHED, s, FALSE);
 	Stream_Release(s);
 	return ret;
@@ -2517,20 +2548,19 @@ static BOOL update_send_new_or_existing_window(rdpContext* context,
                                                const WINDOW_ORDER_INFO* orderInfo,
                                                const WINDOW_STATE_ORDER* stateOrder)
 {
-	wStream* s = NULL;
 	BYTE controlFlags = ORDER_SECONDARY | (ORDER_TYPE_WINDOW << 2);
 	UINT16 orderSize = update_calculate_new_or_existing_window(orderInfo, stateOrder);
-	rdp_update_internal* update = NULL;
 
 	WINPR_ASSERT(context);
 	WINPR_ASSERT(orderInfo);
 	WINPR_ASSERT(stateOrder);
 
-	update = update_cast(context->update);
+	rdp_update_internal* update = update_cast(context->update);
 
-	update_check_flush(context, orderSize);
+	if (!update_check_flush(context, orderSize))
+		return FALSE;
 
-	s = update->us;
+	wStream* s = update->us;
 
 	if (!s)
 		return FALSE;
@@ -2554,7 +2584,7 @@ static BOOL update_send_new_or_existing_window(rdpContext* context,
 
 	if ((orderInfo->fieldFlags & WINDOW_ORDER_FIELD_SHOW) != 0)
 	{
-		Stream_Write_UINT8(s, stateOrder->showState);
+		Stream_Write_UINT8(s, WINPR_ASSERTING_INT_CAST(uint8_t, stateOrder->showState));
 	}
 
 	if ((orderInfo->fieldFlags & WINDOW_ORDER_FIELD_TITLE) != 0)
@@ -2589,7 +2619,7 @@ static BOOL update_send_new_or_existing_window(rdpContext* context,
 
 	if ((orderInfo->fieldFlags & WINDOW_ORDER_FIELD_RP_CONTENT) != 0)
 	{
-		Stream_Write_UINT8(s, stateOrder->RPContent);
+		Stream_Write_UINT8(s, WINPR_ASSERTING_INT_CAST(uint8_t, stateOrder->RPContent));
 	}
 
 	if ((orderInfo->fieldFlags & WINDOW_ORDER_FIELD_ROOT_PARENT) != 0)
@@ -2617,19 +2647,19 @@ static BOOL update_send_new_or_existing_window(rdpContext* context,
 
 	if ((orderInfo->fieldFlags & WINDOW_ORDER_FIELD_WND_RECTS) != 0)
 	{
-		Stream_Write_UINT16(s, stateOrder->numWindowRects);
+		Stream_Write_UINT16(s, WINPR_ASSERTING_INT_CAST(uint16_t, stateOrder->numWindowRects));
 		Stream_Write(s, stateOrder->windowRects, stateOrder->numWindowRects * sizeof(RECTANGLE_16));
 	}
 
 	if ((orderInfo->fieldFlags & WINDOW_ORDER_FIELD_VIS_OFFSET) != 0)
 	{
-		Stream_Write_UINT32(s, stateOrder->visibleOffsetX);
-		Stream_Write_UINT32(s, stateOrder->visibleOffsetY);
+		Stream_Write_INT32(s, stateOrder->visibleOffsetX);
+		Stream_Write_INT32(s, stateOrder->visibleOffsetY);
 	}
 
 	if ((orderInfo->fieldFlags & WINDOW_ORDER_FIELD_VISIBILITY) != 0)
 	{
-		Stream_Write_UINT16(s, stateOrder->numVisibilityRects);
+		Stream_Write_UINT16(s, WINPR_ASSERTING_INT_CAST(uint16_t, stateOrder->numVisibilityRects));
 		Stream_Write(s, stateOrder->visibilityRects,
 		             stateOrder->numVisibilityRects * sizeof(RECTANGLE_16));
 	}
@@ -2697,23 +2727,22 @@ static UINT16 update_calculate_window_icon_order(const WINDOW_ORDER_INFO* orderI
 static BOOL update_send_window_icon(rdpContext* context, const WINDOW_ORDER_INFO* orderInfo,
                                     const WINDOW_ICON_ORDER* iconOrder)
 {
-	wStream* s = NULL;
 	BYTE controlFlags = ORDER_SECONDARY | (ORDER_TYPE_WINDOW << 2);
 
 	WINPR_ASSERT(iconOrder);
 	ICON_INFO* iconInfo = iconOrder->iconInfo;
 	UINT16 orderSize = update_calculate_window_icon_order(orderInfo, iconOrder);
-	rdp_update_internal* update = NULL;
 
 	WINPR_ASSERT(context);
 	WINPR_ASSERT(orderInfo);
 	WINPR_ASSERT(iconInfo);
 
-	update = update_cast(context->update);
+	rdp_update_internal* update = update_cast(context->update);
 
-	update_check_flush(context, orderSize);
+	if (!update_check_flush(context, orderSize))
+		return FALSE;
 
-	s = update->us;
+	wStream* s = update->us;
 
 	if (!s || !iconInfo)
 		return FALSE;
@@ -2727,19 +2756,26 @@ static BOOL update_send_window_icon(rdpContext* context, const WINDOW_ORDER_INFO
 	Stream_Write_UINT32(s, orderInfo->fieldFlags); /* FieldsPresentFlags (4 bytes) */
 	Stream_Write_UINT32(s, orderInfo->windowId);   /* WindowID (4 bytes) */
 	/* Write body */
-	Stream_Write_UINT16(s, iconInfo->cacheEntry); /* CacheEntry (2 bytes) */
-	Stream_Write_UINT8(s, iconInfo->cacheId);     /* CacheId (1 byte) */
-	Stream_Write_UINT8(s, iconInfo->bpp);         /* Bpp (1 byte) */
-	Stream_Write_UINT16(s, iconInfo->width);      /* Width (2 bytes) */
-	Stream_Write_UINT16(s, iconInfo->height);     /* Height (2 bytes) */
+	Stream_Write_UINT16(
+	    s, WINPR_ASSERTING_INT_CAST(uint16_t, iconInfo->cacheEntry)); /* CacheEntry (2 bytes) */
+	Stream_Write_UINT8(s,
+	                   WINPR_ASSERTING_INT_CAST(uint8_t, iconInfo->cacheId)); /* CacheId (1 byte) */
+	Stream_Write_UINT8(s, WINPR_ASSERTING_INT_CAST(uint8_t, iconInfo->bpp));  /* Bpp (1 byte) */
+	Stream_Write_UINT16(s,
+	                    WINPR_ASSERTING_INT_CAST(uint16_t, iconInfo->width)); /* Width (2 bytes) */
+	Stream_Write_UINT16(
+	    s, WINPR_ASSERTING_INT_CAST(uint16_t, iconInfo->height)); /* Height (2 bytes) */
 
 	if (iconInfo->bpp <= 8)
 	{
-		Stream_Write_UINT16(s, iconInfo->cbColorTable); /* CbColorTable (2 bytes) */
+		Stream_Write_UINT16(s, WINPR_ASSERTING_INT_CAST(
+		                           uint16_t, iconInfo->cbColorTable)); /* CbColorTable (2 bytes) */
 	}
 
-	Stream_Write_UINT16(s, iconInfo->cbBitsMask);              /* CbBitsMask (2 bytes) */
-	Stream_Write_UINT16(s, iconInfo->cbBitsColor);             /* CbBitsColor (2 bytes) */
+	Stream_Write_UINT16(
+	    s, WINPR_ASSERTING_INT_CAST(uint16_t, iconInfo->cbBitsMask)); /* CbBitsMask (2 bytes) */
+	Stream_Write_UINT16(
+	    s, WINPR_ASSERTING_INT_CAST(uint16_t, iconInfo->cbBitsColor)); /* CbBitsColor (2 bytes) */
 	Stream_Write(s, iconInfo->bitsMask, iconInfo->cbBitsMask); /* BitsMask (variable) */
 
 	if (iconInfo->bpp <= 8)
@@ -2756,23 +2792,22 @@ static BOOL update_send_window_icon(rdpContext* context, const WINDOW_ORDER_INFO
 static BOOL update_send_window_cached_icon(rdpContext* context, const WINDOW_ORDER_INFO* orderInfo,
                                            const WINDOW_CACHED_ICON_ORDER* cachedIconOrder)
 {
-	wStream* s = NULL;
 	BYTE controlFlags = ORDER_SECONDARY | (ORDER_TYPE_WINDOW << 2);
 	UINT16 orderSize = 14;
 
 	WINPR_ASSERT(cachedIconOrder);
 	const CACHED_ICON_INFO* cachedIcon = &cachedIconOrder->cachedIcon;
-	rdp_update_internal* update = NULL;
 
 	WINPR_ASSERT(context);
 	WINPR_ASSERT(orderInfo);
 	WINPR_ASSERT(cachedIcon);
 
-	update = update_cast(context->update);
+	rdp_update_internal* update = update_cast(context->update);
 
-	update_check_flush(context, orderSize);
+	if (!update_check_flush(context, orderSize))
+		return FALSE;
 
-	s = update->us;
+	wStream* s = update->us;
 	if (!s)
 		return FALSE;
 
@@ -2785,26 +2820,27 @@ static BOOL update_send_window_cached_icon(rdpContext* context, const WINDOW_ORD
 	Stream_Write_UINT32(s, orderInfo->fieldFlags); /* FieldsPresentFlags (4 bytes) */
 	Stream_Write_UINT32(s, orderInfo->windowId);   /* WindowID (4 bytes) */
 	/* Write body */
-	Stream_Write_UINT16(s, cachedIcon->cacheEntry); /* CacheEntry (2 bytes) */
-	Stream_Write_UINT8(s, cachedIcon->cacheId);     /* CacheId (1 byte) */
+	Stream_Write_UINT16(
+	    s, WINPR_ASSERTING_INT_CAST(uint16_t, cachedIcon->cacheEntry)); /* CacheEntry (2 bytes) */
+	Stream_Write_UINT8(
+	    s, WINPR_ASSERTING_INT_CAST(uint8_t, cachedIcon->cacheId)); /* CacheId (1 byte) */
 	update->numberOrders++;
 	return TRUE;
 }
 
 static BOOL update_send_window_delete(rdpContext* context, const WINDOW_ORDER_INFO* orderInfo)
 {
-	wStream* s = NULL;
 	BYTE controlFlags = ORDER_SECONDARY | (ORDER_TYPE_WINDOW << 2);
 	UINT16 orderSize = 11;
-	rdp_update_internal* update = NULL;
 
 	WINPR_ASSERT(context);
 	WINPR_ASSERT(orderInfo);
-	update = update_cast(context->update);
+	rdp_update_internal* update = update_cast(context->update);
 
-	update_check_flush(context, orderSize);
+	if (!update_check_flush(context, orderSize))
+		return FALSE;
 
-	s = update->us;
+	wStream* s = update->us;
 
 	if (!s)
 		return FALSE;
@@ -2871,21 +2907,20 @@ update_send_new_or_existing_notification_icons(rdpContext* context,
                                                const WINDOW_ORDER_INFO* orderInfo,
                                                const NOTIFY_ICON_STATE_ORDER* iconStateOrder)
 {
-	wStream* s = NULL;
 	BYTE controlFlags = ORDER_SECONDARY | (ORDER_TYPE_WINDOW << 2);
 	BOOL versionFieldPresent = FALSE;
 	const UINT16 orderSize =
 	    update_calculate_new_or_existing_notification_icons_order(orderInfo, iconStateOrder);
-	rdp_update_internal* update = NULL;
 
 	WINPR_ASSERT(context);
 	WINPR_ASSERT(orderInfo);
 	WINPR_ASSERT(iconStateOrder);
-	update = update_cast(context->update);
+	rdp_update_internal* update = update_cast(context->update);
 
-	update_check_flush(context, orderSize);
+	if (!update_check_flush(context, orderSize))
+		return FALSE;
 
-	s = update->us;
+	wStream* s = update->us;
 	if (!s)
 		return FALSE;
 
@@ -2939,20 +2974,29 @@ update_send_new_or_existing_notification_icons(rdpContext* context,
 
 	if ((orderInfo->fieldFlags & WINDOW_ORDER_ICON) != 0)
 	{
-		ICON_INFO iconInfo = iconStateOrder->icon;
-		Stream_Write_UINT16(s, iconInfo.cacheEntry); /* CacheEntry (2 bytes) */
-		Stream_Write_UINT8(s, iconInfo.cacheId);     /* CacheId (1 byte) */
-		Stream_Write_UINT8(s, iconInfo.bpp);         /* Bpp (1 byte) */
-		Stream_Write_UINT16(s, iconInfo.width);      /* Width (2 bytes) */
-		Stream_Write_UINT16(s, iconInfo.height);     /* Height (2 bytes) */
+		const ICON_INFO iconInfo = iconStateOrder->icon;
+
+		Stream_Write_UINT16(
+		    s, WINPR_ASSERTING_INT_CAST(uint16_t, iconInfo.cacheEntry)); /* CacheEntry (2 bytes) */
+		Stream_Write_UINT8(
+		    s, WINPR_ASSERTING_INT_CAST(uint8_t, iconInfo.cacheId)); /* CacheId (1 byte) */
+		Stream_Write_UINT8(s, WINPR_ASSERTING_INT_CAST(uint8_t, iconInfo.bpp)); /* Bpp (1 byte) */
+		Stream_Write_UINT16(
+		    s, WINPR_ASSERTING_INT_CAST(uint16_t, iconInfo.width)); /* Width (2 bytes) */
+		Stream_Write_UINT16(
+		    s, WINPR_ASSERTING_INT_CAST(uint16_t, iconInfo.height)); /* Height (2 bytes) */
 
 		if (iconInfo.bpp <= 8)
 		{
-			Stream_Write_UINT16(s, iconInfo.cbColorTable); /* CbColorTable (2 bytes) */
+			Stream_Write_UINT16(
+			    s, WINPR_ASSERTING_INT_CAST(uint16_t,
+			                                iconInfo.cbColorTable)); /* CbColorTable (2 bytes) */
 		}
 
-		Stream_Write_UINT16(s, iconInfo.cbBitsMask);             /* CbBitsMask (2 bytes) */
-		Stream_Write_UINT16(s, iconInfo.cbBitsColor);            /* CbBitsColor (2 bytes) */
+		Stream_Write_UINT16(
+		    s, WINPR_ASSERTING_INT_CAST(uint16_t, iconInfo.cbBitsMask)); /* CbBitsMask (2 bytes) */
+		Stream_Write_UINT16(s, WINPR_ASSERTING_INT_CAST(
+		                           uint16_t, iconInfo.cbBitsColor)); /* CbBitsColor (2 bytes) */
 		Stream_Write(s, iconInfo.bitsMask, iconInfo.cbBitsMask); /* BitsMask (variable) */
 
 		if (iconInfo.bpp <= 8)
@@ -2964,9 +3008,11 @@ update_send_new_or_existing_notification_icons(rdpContext* context,
 	}
 	else if ((orderInfo->fieldFlags & WINDOW_ORDER_CACHED_ICON) != 0)
 	{
-		CACHED_ICON_INFO cachedIcon = iconStateOrder->cachedIcon;
-		Stream_Write_UINT16(s, cachedIcon.cacheEntry); /* CacheEntry (2 bytes) */
-		Stream_Write_UINT8(s, cachedIcon.cacheId);     /* CacheId (1 byte) */
+		const CACHED_ICON_INFO cachedIcon = iconStateOrder->cachedIcon;
+		Stream_Write_UINT16(s, WINPR_ASSERTING_INT_CAST(
+		                           uint16_t, cachedIcon.cacheEntry)); /* CacheEntry (2 bytes) */
+		Stream_Write_UINT8(
+		    s, WINPR_ASSERTING_INT_CAST(uint8_t, cachedIcon.cacheId)); /* CacheId (1 byte) */
 	}
 
 	update->numberOrders++;
@@ -2987,18 +3033,17 @@ static BOOL update_send_notify_icon_update(rdpContext* context, const WINDOW_ORD
 
 static BOOL update_send_notify_icon_delete(rdpContext* context, const WINDOW_ORDER_INFO* orderInfo)
 {
-	wStream* s = NULL;
 	BYTE controlFlags = ORDER_SECONDARY | (ORDER_TYPE_WINDOW << 2);
 	UINT16 orderSize = 15;
-	rdp_update_internal* update = NULL;
 
 	WINPR_ASSERT(context);
 	WINPR_ASSERT(orderInfo);
-	update = update_cast(context->update);
+	rdp_update_internal* update = update_cast(context->update);
 
-	update_check_flush(context, orderSize);
+	if (!update_check_flush(context, orderSize))
+		return FALSE;
 
-	s = update->us;
+	wStream* s = update->us;
 
 	if (!s)
 		return FALSE;
@@ -3037,20 +3082,19 @@ static UINT16 update_calculate_monitored_desktop(const WINDOW_ORDER_INFO* orderI
 static BOOL update_send_monitored_desktop(rdpContext* context, const WINDOW_ORDER_INFO* orderInfo,
                                           const MONITORED_DESKTOP_ORDER* monitoredDesktop)
 {
-	wStream* s = NULL;
 	BYTE controlFlags = ORDER_SECONDARY | (ORDER_TYPE_WINDOW << 2);
 	UINT16 orderSize = update_calculate_monitored_desktop(orderInfo, monitoredDesktop);
-	rdp_update_internal* update = NULL;
 
 	WINPR_ASSERT(context);
 	WINPR_ASSERT(orderInfo);
 	WINPR_ASSERT(monitoredDesktop);
 
-	update = update_cast(context->update);
+	rdp_update_internal* update = update_cast(context->update);
 
-	update_check_flush(context, orderSize);
+	if (!update_check_flush(context, orderSize))
+		return FALSE;
 
-	s = update->us;
+	wStream* s = update->us;
 
 	if (!s)
 		return FALSE;
@@ -3066,12 +3110,15 @@ static BOOL update_send_monitored_desktop(rdpContext* context, const WINDOW_ORDE
 
 	if (orderInfo->fieldFlags & WINDOW_ORDER_FIELD_DESKTOP_ZORDER)
 	{
-		Stream_Write_UINT8(s, monitoredDesktop->numWindowIds); /* numWindowIds (1 byte) */
+		Stream_Write_UINT8(
+		    s, WINPR_ASSERTING_INT_CAST(
+		           uint8_t, monitoredDesktop->numWindowIds)); /* numWindowIds (1 byte) */
 
 		/* windowIds */
 		for (UINT32 i = 0; i < monitoredDesktop->numWindowIds; i++)
 		{
-			Stream_Write_UINT32(s, monitoredDesktop->windowIds[i]);
+			Stream_Write_UINT32(s,
+			                    WINPR_ASSERTING_INT_CAST(uint32_t, monitoredDesktop->windowIds[i]));
 		}
 	}
 
@@ -3082,18 +3129,17 @@ static BOOL update_send_monitored_desktop(rdpContext* context, const WINDOW_ORDE
 static BOOL update_send_non_monitored_desktop(rdpContext* context,
                                               const WINDOW_ORDER_INFO* orderInfo)
 {
-	wStream* s = NULL;
 	BYTE controlFlags = ORDER_SECONDARY | (ORDER_TYPE_WINDOW << 2);
 	UINT16 orderSize = 7;
-	rdp_update_internal* update = NULL;
 
 	WINPR_ASSERT(context);
 	WINPR_ASSERT(orderInfo);
-	update = update_cast(context->update);
+	rdp_update_internal* update = update_cast(context->update);
 
-	update_check_flush(context, orderSize);
+	if (!update_check_flush(context, orderSize))
+		return FALSE;
 
-	s = update->us;
+	wStream* s = update->us;
 
 	if (!s)
 		return FALSE;
@@ -3329,7 +3375,7 @@ BOOL update_begin_paint(rdpUpdate* update)
 	/* Reset the invalid regions, we start a new frame here. */
 	rdpGdi* gdi = update->context->gdi;
 	if (!gdi)
-		return FALSE;
+		return rc;
 
 	if (gdi->hdc && gdi->primary && gdi->primary->hdc)
 	{
