@@ -26,6 +26,7 @@
 
 #include <winpr/crt.h>
 #include <winpr/assert.h>
+#include <winpr/cast.h>
 
 #include <freerdp/api.h>
 #include <freerdp/log.h>
@@ -323,6 +324,14 @@ static const BYTE GDI_BS_HATCHED_PATTERNS[] = {
 	0x7E, 0xBD, 0xDB, 0xE7, 0xE7, 0xDB, 0xBD, 0x7E  /* HS_DIACROSS */
 };
 
+#define gdi_rop3_code_checked(code) gdi_rop3_code_checked_int((code), __FILE__, __func__, __LINE__)
+static inline DWORD gdi_rop3_code_checked_int(UINT32 code, const char* file, const char* fkt,
+                                              size_t line)
+{
+	WINPR_ASSERT_AT(code <= UINT8_MAX, file, fkt, line);
+	return gdi_rop3_code((UINT8)code);
+}
+
 BOOL gdi_decode_color(rdpGdi* gdi, const UINT32 srcColor, UINT32* color, UINT32* format)
 {
 	UINT32 SrcFormat = 0;
@@ -437,9 +446,13 @@ gdiBitmap* gdi_bitmap_new_ex(rdpGdi* gdi, int width, int height, int bpp, BYTE*
 	           bpp);
 
 	if (!data)
-		bitmap->bitmap = gdi_CreateCompatibleBitmap(gdi->hdc, width, height);
+		bitmap->bitmap =
+		    gdi_CreateCompatibleBitmap(gdi->hdc, WINPR_ASSERTING_INT_CAST(uint32_t, width),
+		                               WINPR_ASSERTING_INT_CAST(uint32_t, height));
 	else
-		bitmap->bitmap = gdi_create_bitmap(gdi, width, height, bpp, data);
+		bitmap->bitmap = gdi_create_bitmap(gdi, WINPR_ASSERTING_INT_CAST(uint32_t, width),
+		                                   WINPR_ASSERTING_INT_CAST(uint32_t, height),
+		                                   WINPR_ASSERTING_INT_CAST(uint32_t, bpp), data);
 
 	if (!bitmap->bitmap)
 		goto fail_bitmap_bitmap;
@@ -480,43 +493,39 @@ BOOL gdi_bitmap_update(rdpContext* context, const BITMAP_UPDATE* bitmapUpdate)
 
 	for (UINT32 index = 0; index < bitmapUpdate->number; index++)
 	{
+		BOOL rc = FALSE;
 		const BITMAP_DATA* bitmap = &(bitmapUpdate->rectangles[index]);
 		rdpBitmap* bmp = Bitmap_Alloc(context);
 
 		if (!bmp)
-		{
-			WLog_ERR(TAG, "Bitmap_Alloc failed");
-			return FALSE;
-		}
+			goto fail;
 
-		Bitmap_SetDimensions(bmp, bitmap->width, bitmap->height);
-		Bitmap_SetRectangle(bmp, bitmap->destLeft, bitmap->destTop, bitmap->destRight,
-		                    bitmap->destBottom);
+		if (!Bitmap_SetDimensions(bmp, WINPR_ASSERTING_INT_CAST(UINT16, bitmap->width),
+		                          WINPR_ASSERTING_INT_CAST(UINT16, bitmap->height)))
+			goto fail;
+
+		if (!Bitmap_SetRectangle(bmp, WINPR_ASSERTING_INT_CAST(UINT16, bitmap->destLeft),
+		                         WINPR_ASSERTING_INT_CAST(UINT16, bitmap->destTop),
+		                         WINPR_ASSERTING_INT_CAST(UINT16, bitmap->destRight),
+		                         WINPR_ASSERTING_INT_CAST(UINT16, bitmap->destBottom)))
+			goto fail;
 
 		if (!bmp->Decompress(context, bmp, bitmap->bitmapDataStream, bitmap->width, bitmap->height,
 		                     bitmap->bitsPerPixel, bitmap->bitmapLength, bitmap->compressed,
 		                     RDP_CODEC_ID_NONE))
-		{
-			WLog_ERR(TAG, "bmp->Decompress failed");
-			Bitmap_Free(context, bmp);
-			return FALSE;
-		}
+			goto fail;
 
 		if (!bmp->New(context, bmp))
-		{
-			WLog_ERR(TAG, "bmp->New failed");
-			Bitmap_Free(context, bmp);
-			return FALSE;
-		}
+			goto fail;
 
 		if (!bmp->Paint(context, bmp))
-		{
-			WLog_ERR(TAG, "bmp->Paint failed");
-			Bitmap_Free(context, bmp);
-			return FALSE;
-		}
+			goto fail;
 
+		rc = TRUE;
+	fail:
 		Bitmap_Free(context, bmp);
+		if (!rc)
+			return FALSE;
 	}
 
 	return TRUE;
@@ -571,7 +580,8 @@ static BOOL gdi_dstblt(rdpContext* context, const DSTBLT_ORDER* dstblt)
 
 	gdi = context->gdi;
 	return gdi_BitBlt(gdi->drawing->hdc, dstblt->nLeftRect, dstblt->nTopRect, dstblt->nWidth,
-	                  dstblt->nHeight, NULL, 0, 0, gdi_rop3_code(dstblt->bRop), &gdi->palette);
+	                  dstblt->nHeight, NULL, 0, 0, gdi_rop3_code_checked(dstblt->bRop),
+	                  &gdi->palette);
 }
 
 static BOOL gdi_patblt(rdpContext* context, PATBLT_ORDER* patblt)
@@ -584,7 +594,7 @@ static BOOL gdi_patblt(rdpContext* context, PATBLT_ORDER* patblt)
 	HGDI_BRUSH hbrush = NULL;
 	rdpGdi* gdi = context->gdi;
 	BOOL ret = FALSE;
-	const DWORD rop = gdi_rop3_code(patblt->bRop);
+	const DWORD rop = gdi_rop3_code_checked(patblt->bRop);
 	INT32 nXSrc = 0;
 	INT32 nYSrc = 0;
 	BYTE data[8 * 8 * 4];
@@ -608,7 +618,7 @@ static BOOL gdi_patblt(rdpContext* context, PATBLT_ORDER* patblt)
 		case GDI_BS_HATCHED:
 		{
 			const BYTE* hatched = NULL;
-			hatched = GDI_BS_HATCHED_PATTERNS + (8 * brush->hatch);
+			hatched = GDI_BS_HATCHED_PATTERNS + (8ULL * brush->hatch);
 
 			if (!freerdp_image_copy_from_monochrome(data, gdi->drawing->hdc->format, 0, 0, 0, 8, 8,
 			                                        hatched, backColor, foreColor, &gdi->palette))
@@ -666,8 +676,8 @@ static BOOL gdi_patblt(rdpContext* context, PATBLT_ORDER* patblt)
 
 	if (hbrush)
 	{
-		hbrush->nXOrg = brush->x;
-		hbrush->nYOrg = brush->y;
+		hbrush->nXOrg = WINPR_ASSERTING_INT_CAST(int32_t, brush->x);
+		hbrush->nYOrg = WINPR_ASSERTING_INT_CAST(int32_t, brush->y);
 		gdi->drawing->hdc->brush = hbrush;
 		ret = gdi_BitBlt(gdi->drawing->hdc, patblt->nLeftRect, patblt->nTopRect, patblt->nWidth,
 		                 patblt->nHeight, gdi->primary->hdc, nXSrc, nYSrc, rop, &gdi->palette);
@@ -691,7 +701,7 @@ static BOOL gdi_scrblt(rdpContext* context, const SCRBLT_ORDER* scrblt)
 	gdi = context->gdi;
 	return gdi_BitBlt(gdi->drawing->hdc, scrblt->nLeftRect, scrblt->nTopRect, scrblt->nWidth,
 	                  scrblt->nHeight, gdi->primary->hdc, scrblt->nXSrc, scrblt->nYSrc,
-	                  gdi_rop3_code(scrblt->bRop), &gdi->palette);
+	                  gdi_rop3_code_checked(scrblt->bRop), &gdi->palette);
 }
 
 static BOOL gdi_opaque_rect(rdpContext* context, const OPAQUE_RECT_ORDER* opaque_rect)
@@ -777,7 +787,7 @@ static BOOL gdi_line_to(rdpContext* context, const LINE_TO_ORDER* lineTo)
 		return FALSE;
 
 	gdi_SelectObject(gdi->drawing->hdc, (HGDIOBJECT)hPen);
-	gdi_SetROP2(gdi->drawing->hdc, lineTo->bRop2);
+	gdi_SetROP2(gdi->drawing->hdc, WINPR_ASSERTING_INT_CAST(int32_t, lineTo->bRop2));
 	gdi_MoveToEx(gdi->drawing->hdc, lineTo->nXStart, lineTo->nYStart, NULL);
 	gdi_LineTo(gdi->drawing->hdc, lineTo->nXEnd, lineTo->nYEnd);
 	gdi_DeleteObject((HGDIOBJECT)hPen);
@@ -802,7 +812,7 @@ static BOOL gdi_polyline(rdpContext* context, const POLYLINE_ORDER* polyline)
 		return FALSE;
 
 	gdi_SelectObject(gdi->drawing->hdc, (HGDIOBJECT)hPen);
-	gdi_SetROP2(gdi->drawing->hdc, polyline->bRop2);
+	gdi_SetROP2(gdi->drawing->hdc, WINPR_ASSERTING_INT_CAST(int32_t, polyline->bRop2));
 	x = polyline->xStart;
 	y = polyline->yStart;
 	gdi_ClipCoords(gdi->drawing->hdc, &x, &y, &w, &h, NULL, NULL);
@@ -834,7 +844,7 @@ static BOOL gdi_memblt(rdpContext* context, MEMBLT_ORDER* memblt)
 	gdi = context->gdi;
 	return gdi_BitBlt(gdi->drawing->hdc, memblt->nLeftRect, memblt->nTopRect, memblt->nWidth,
 	                  memblt->nHeight, bitmap->hdc, memblt->nXSrc, memblt->nYSrc,
-	                  gdi_rop3_code(memblt->bRop), &gdi->palette);
+	                  gdi_rop3_code_checked(memblt->bRop), &gdi->palette);
 }
 
 static BOOL gdi_mem3blt(rdpContext* context, MEM3BLT_ORDER* mem3blt)
@@ -870,7 +880,7 @@ static BOOL gdi_mem3blt(rdpContext* context, MEM3BLT_ORDER* mem3blt)
 
 			ret = gdi_BitBlt(gdi->drawing->hdc, mem3blt->nLeftRect, mem3blt->nTopRect,
 			                 mem3blt->nWidth, mem3blt->nHeight, bitmap->hdc, mem3blt->nXSrc,
-			                 mem3blt->nYSrc, gdi_rop3_code(mem3blt->bRop), &gdi->palette);
+			                 mem3blt->nYSrc, gdi_rop3_code_checked(mem3blt->bRop), &gdi->palette);
 			gdi_DeleteObject((HGDIOBJECT)gdi->drawing->hdc->brush);
 			gdi->drawing->hdc->brush = originalBrush;
 			break;
@@ -880,7 +890,7 @@ static BOOL gdi_mem3blt(rdpContext* context, MEM3BLT_ORDER* mem3blt)
 			HGDI_BITMAP hBmp = NULL;
 			UINT32 brushFormat = 0;
 			BYTE* data = (BYTE*)winpr_aligned_malloc(
-			    8 * 8 * FreeRDPGetBytesPerPixel(gdi->drawing->hdc->format), 16);
+			    8ULL * 8ULL * FreeRDPGetBytesPerPixel(gdi->drawing->hdc->format), 16);
 
 			if (!data)
 			{
@@ -938,11 +948,11 @@ static BOOL gdi_mem3blt(rdpContext* context, MEM3BLT_ORDER* mem3blt)
 				goto out_fail;
 			}
 
-			gdi->drawing->hdc->brush->nXOrg = brush->x;
-			gdi->drawing->hdc->brush->nYOrg = brush->y;
+			gdi->drawing->hdc->brush->nXOrg = WINPR_ASSERTING_INT_CAST(int32_t, brush->x);
+			gdi->drawing->hdc->brush->nYOrg = WINPR_ASSERTING_INT_CAST(int32_t, brush->y);
 			ret = gdi_BitBlt(gdi->drawing->hdc, mem3blt->nLeftRect, mem3blt->nTopRect,
 			                 mem3blt->nWidth, mem3blt->nHeight, bitmap->hdc, mem3blt->nXSrc,
-			                 mem3blt->nYSrc, gdi_rop3_code(mem3blt->bRop), &gdi->palette);
+			                 mem3blt->nYSrc, gdi_rop3_code_checked(mem3blt->bRop), &gdi->palette);
 			gdi_DeleteObject((HGDIOBJECT)gdi->drawing->hdc->brush);
 			gdi_DeleteObject((HGDIOBJECT)hBmp);
 			gdi->drawing->hdc->brush = originalBrush;
@@ -1006,6 +1016,8 @@ static BOOL gdi_surface_frame_marker(rdpContext* context,
 				       surfaceFrameMarker->frameId);
 			}
 
+			break;
+		default:
 			break;
 	}
 
@@ -1075,8 +1087,8 @@ static BOOL gdi_surface_bits(rdpContext* context, const SURFACE_BITS_COMMAND* cm
 		case RDP_CODEC_ID_IMAGE_REMOTEFX:
 			if (!rfx_process_message(context->codecs->rfx, cmd->bmp.bitmapData,
 			                         cmd->bmp.bitmapDataLength, cmdRect.left, cmdRect.top,
-			                         gdi->primary_buffer, gdi->dstFormat, gdi->stride, gdi->height,
-			                         &region))
+			                         gdi->primary_buffer, gdi->dstFormat, gdi->stride,
+			                         WINPR_ASSERTING_INT_CAST(uint32_t, gdi->height), &region))
 			{
 				WLog_ERR(TAG, "Failed to process RemoteFX message");
 				goto out;
@@ -1137,7 +1149,10 @@ static BOOL gdi_surface_bits(rdpContext* context, const SURFACE_BITS_COMMAND* cm
 		UINT32 width = rects[i].right - rects[i].left;
 		UINT32 height = rects[i].bottom - rects[i].top;
 
-		if (!gdi_InvalidateRegion(gdi->primary->hdc, left, top, width, height))
+		if (!gdi_InvalidateRegion(gdi->primary->hdc, WINPR_ASSERTING_INT_CAST(int32_t, left),
+		                          WINPR_ASSERTING_INT_CAST(int32_t, top),
+		                          WINPR_ASSERTING_INT_CAST(int32_t, width),
+		                          WINPR_ASSERTING_INT_CAST(int32_t, height)))
 		{
 			WLog_ERR(TAG, "Failed to update invalid region");
 			goto out;
@@ -1217,7 +1232,8 @@ static BOOL gdi_init_primary(rdpGdi* gdi, UINT32 stride, UINT32 format, BYTE* bu
 	if (stride > 0)
 		gdi->stride = stride;
 	else
-		gdi->stride = gdi->width * FreeRDPGetBytesPerPixel(gdi->dstFormat);
+		gdi->stride = WINPR_ASSERTING_INT_CAST(uint32_t, gdi->width) *
+		              FreeRDPGetBytesPerPixel(gdi->dstFormat);
 
 	if (!gdi->primary)
 		goto fail_primary;
@@ -1227,12 +1243,15 @@ static BOOL gdi_init_primary(rdpGdi* gdi, UINT32 stride, UINT32 format, BYTE* bu
 
 	if (!buffer)
 	{
-		gdi->primary->bitmap = gdi_CreateCompatibleBitmap(gdi->hdc, gdi->width, gdi->height);
+		gdi->primary->bitmap =
+		    gdi_CreateCompatibleBitmap(gdi->hdc, WINPR_ASSERTING_INT_CAST(uint32_t, gdi->width),
+		                               WINPR_ASSERTING_INT_CAST(uint32_t, gdi->height));
 	}
 	else
 	{
-		gdi->primary->bitmap =
-		    gdi_CreateBitmapEx(gdi->width, gdi->height, gdi->dstFormat, gdi->stride, buffer, pfree);
+		gdi->primary->bitmap = gdi_CreateBitmapEx(WINPR_ASSERTING_INT_CAST(uint32_t, gdi->width),
+		                                          WINPR_ASSERTING_INT_CAST(uint32_t, gdi->height),
+		                                          gdi->dstFormat, gdi->stride, buffer, pfree);
 	}
 
 	if (!gdi->primary->bitmap)
@@ -1295,6 +1314,10 @@ BOOL gdi_resize_ex(rdpGdi* gdi, UINT32 width, UINT32 height, UINT32 stride, UINT
 
 	WINPR_ASSERT(gdi->context);
 	WINPR_ASSERT(gdi->context->update);
+
+	/* EndPaint might not have been called, ensure the update lock is released */
+	if (!update_end_paint(gdi->context->update))
+		return FALSE;
 	rdp_update_lock(gdi->context->update);
 
 	if (gdi->drawing == gdi->primary)
@@ -1358,8 +1381,10 @@ BOOL gdi_init_ex(freerdp* instance, UINT32 format, UINT32 stride, BYTE* buffer,
 		goto fail;
 
 	gdi->context = context;
-	gdi->width = freerdp_settings_get_uint32(context->settings, FreeRDP_DesktopWidth);
-	gdi->height = freerdp_settings_get_uint32(context->settings, FreeRDP_DesktopHeight);
+	gdi->width = WINPR_ASSERTING_INT_CAST(
+	    int32_t, freerdp_settings_get_uint32(context->settings, FreeRDP_DesktopWidth));
+	gdi->height = WINPR_ASSERTING_INT_CAST(
+	    int32_t, freerdp_settings_get_uint32(context->settings, FreeRDP_DesktopHeight));
 	gdi->dstFormat = format;
 	/* default internal buffer format */
 	WLog_Print(gdi->log, WLOG_INFO, "Local framebuffer format  %s",
@@ -1435,7 +1460,10 @@ BOOL gdi_send_suppress_output(rdpGdi* gdi, BOOL suppress)
 	update = gdi->context->update;
 	rect.left = 0;
 	rect.top = 0;
-	rect.right = freerdp_settings_get_uint32(settings, FreeRDP_DesktopWidth);
-	rect.bottom = freerdp_settings_get_uint32(settings, FreeRDP_DesktopHeight);
+
+	const UINT32 w = freerdp_settings_get_uint32(settings, FreeRDP_DesktopWidth);
+	const UINT32 h = freerdp_settings_get_uint32(settings, FreeRDP_DesktopHeight);
+	rect.right = WINPR_ASSERTING_INT_CAST(UINT16, w);
+	rect.bottom = WINPR_ASSERTING_INT_CAST(UINT16, h);
 	return update->SuppressOutput(gdi->context, !suppress, &rect);
 }
