@@ -17,6 +17,8 @@
  * limitations under the License.
  */
 
+#include <winpr/environment.h>
+
 #include <freerdp/config.h>
 #include <freerdp/freerdp.h>
 
@@ -70,6 +72,7 @@ const char* freerdp_passphrase_read(rdpContext* context, const char* prompt, cha
 #include <stdio.h>
 #include <string.h>
 #include <sys/stat.h>
+#include <sys/wait.h>
 #include <termios.h>
 #include <unistd.h>
 #include <freerdp/utils/signal.h>
@@ -132,7 +135,6 @@ static const char* freerdp_passphrase_read_tty(rdpContext* context, const char*
 {
 	BOOL terminal_needs_reset = FALSE;
 	char term_name[L_ctermid] = { 0 };
-	int term_file = 0;
 
 	FILE* fout = NULL;
 
@@ -144,15 +146,29 @@ static const char* freerdp_passphrase_read_tty(rdpContext* context, const char*
 
 	ctermid(term_name);
 	int terminal_fildes = 0;
-	if (from_stdin || strcmp(term_name, "") == 0 || (term_file = open(term_name, O_RDWR)) == -1)
+	if (from_stdin || (strcmp(term_name, "") == 0))
 	{
 		fout = stdout;
 		terminal_fildes = STDIN_FILENO;
 	}
 	else
 	{
-		fout = fdopen(term_file, "w");
-		terminal_fildes = term_file;
+		const int term_file = open(term_name, O_RDWR);
+		if (term_file < 0)
+		{
+			fout = stdout;
+			terminal_fildes = STDIN_FILENO;
+		}
+		else
+		{
+			fout = fdopen(term_file, "w");
+			if (!fout)
+			{
+				close(term_file);
+				return NULL;
+			}
+			terminal_fildes = term_file;
+		}
 	}
 
 	struct termios orig_flags = { 0 };
@@ -160,7 +176,7 @@ static const char* freerdp_passphrase_read_tty(rdpContext* context, const char*
 	{
 		struct termios new_flags = { 0 };
 		new_flags = orig_flags;
-		new_flags.c_lflag &= ~ECHO;
+		new_flags.c_lflag &= (uint32_t)~ECHO;
 		new_flags.c_lflag |= ECHONL;
 		terminal_needs_reset = TRUE;
 		if (tcsetattr(terminal_fildes, TCSAFLUSH, &new_flags) == -1)
@@ -191,15 +207,13 @@ static const char* freerdp_passphrase_read_tty(rdpContext* context, const char*
 	}
 
 	if (terminal_fildes != STDIN_FILENO)
-	{
-		if (fclose(fp) == -1)
-			goto error;
-	}
+		(void)fclose(fp);
 
 	return buf;
 
 error:
 {
+	// NOLINTNEXTLINE(clang-analyzer-unix.Stream)
 	int saved_errno = errno;
 	if (terminal_needs_reset)
 		(void)tcsetattr(terminal_fildes, TCSAFLUSH, &orig_flags);
@@ -209,9 +223,11 @@ error:
 		if (fp)
 			(void)fclose(fp);
 	}
+	// NOLINTNEXTLINE(clang-analyzer-unix.Stream)
 	errno = saved_errno;
-	return NULL;
 }
+
+	return NULL;
 }
 
 static const char* freerdp_passphrase_read_askpass(const char* prompt, char* buf, size_t bufsiz,
@@ -221,10 +237,12 @@ static const char* freerdp_passphrase_read_askpass(const char* prompt, char* buf
 
 	(void)sprintf_s(command, sizeof(command), "%s 'FreeRDP authentication\n%s'", askpass_env,
 	                prompt);
+	// NOLINTNEXTLINE(clang-analyzer-optin.taint.GenericTaint)
 	FILE* askproc = popen(command, "r");
 	if (!askproc)
 		return NULL;
-	if (fgets(buf, bufsiz, askproc) != NULL)
+	WINPR_ASSERT(bufsiz <= INT32_MAX);
+	if (fgets(buf, (int)bufsiz, askproc) != NULL)
 		buf[strcspn(buf, "\r\n")] = '\0';
 	else
 		buf = NULL;
@@ -238,6 +256,7 @@ static const char* freerdp_passphrase_read_askpass(const char* prompt, char* buf
 const char* freerdp_passphrase_read(rdpContext* context, const char* prompt, char* buf,
                                     size_t bufsiz, int from_stdin)
 {
+	// NOLINTNEXTLINE(concurrency-mt-unsafe)
 	const char* askpass_env = getenv("FREERDP_ASKPASS");
 
 	if (askpass_env)
@@ -328,5 +347,5 @@ SSIZE_T freerdp_interruptible_get_line(rdpContext* context, char** plineptr, siz
 	}
 	*plineptr = ptr;
 	*psize = used;
-	return used;
+	return WINPR_ASSERTING_INT_CAST(SSIZE_T, used);
 }
