@@ -21,6 +21,9 @@
  * limitations under the License.
  */
 
+#include <winpr/assert.h>
+#include <winpr/cast.h>
+
 #include <freerdp/config.h>
 
 #include <freerdp/freerdp.h>
@@ -486,8 +489,9 @@ static BOOL pf_client_receive_channel_data_hook(freerdp* instance, UINT16 channe
 			if (channel->front_channel_id == 0)
 				return TRUE;
 
-			return ps->context.peer->SendChannelPacket(ps->context.peer, channel->front_channel_id,
-			                                           totalSize, flags, xdata, xsize);
+			return ps->context.peer->SendChannelPacket(
+			    ps->context.peer, WINPR_ASSERTING_INT_CAST(UINT16, channel->front_channel_id),
+			    totalSize, flags, xdata, xsize);
 		case PF_CHANNEL_RESULT_DROP:
 			return TRUE;
 		case PF_CHANNEL_RESULT_ERROR:
