@@ -104,6 +104,8 @@ static CK_ATTRIBUTE public_key_filter[] = {
 	{ CKA_KEY_TYPE, &object_ktype_rsa, sizeof(object_ktype_rsa) }
 };
 
+static const char* CK_RV_error_string(CK_RV rv);
+
 static SECURITY_STATUS NCryptP11StorageProvider_dtor(NCRYPT_HANDLE handle)
 {
 	NCryptP11ProviderHandle* provider = (NCryptP11ProviderHandle*)handle;
@@ -114,8 +116,8 @@ static SECURITY_STATUS NCryptP11StorageProvider_dtor(NCRYPT_HANDLE handle)
 		if (provider->p11 && provider->p11->C_Finalize)
 			rv = provider->p11->C_Finalize(NULL);
 		if (rv != CKR_OK)
-		{
-		}
+			WLog_WARN(TAG, "C_Finalize failed with %s [0x%08" PRIx32 "]", CK_RV_error_string(rv),
+			          rv);
 
 		free(provider->modulePath);
 
@@ -199,13 +201,11 @@ static CK_RV object_load_attributes(NCryptP11ProviderHandle* provider, CK_SESSIO
                                     CK_OBJECT_HANDLE object, CK_ATTRIBUTE_PTR attributes,
                                     CK_ULONG count)
 {
-	CK_RV rv = 0;
-
 	WINPR_ASSERT(provider);
 	WINPR_ASSERT(provider->p11);
 	WINPR_ASSERT(provider->p11->C_GetAttributeValue);
 
-	rv = provider->p11->C_GetAttributeValue(session, object, attributes, count);
+	CK_RV rv = provider->p11->C_GetAttributeValue(session, object, attributes, count);
 
 	switch (rv)
 	{
@@ -222,8 +222,13 @@ static CK_RV object_load_attributes(NCryptP11ProviderHandle* provider, CK_SESSIO
 				return CKR_HOST_MEMORY;
 
 			rv = provider->p11->C_GetAttributeValue(session, object, attributes, count);
+			if (rv != CKR_OK)
+				WLog_WARN(TAG, "C_GetAttributeValue failed with %s [0x%08" PRIx32 "]",
+				          CK_RV_error_string(rv), rv);
 			break;
 		default:
+			WLog_WARN(TAG, "C_GetAttributeValue failed with %s [0x%08" PRIx32 "]",
+			          CK_RV_error_string(rv), rv);
 			return rv;
 	}
 
@@ -232,7 +237,12 @@ static CK_RV object_load_attributes(NCryptP11ProviderHandle* provider, CK_SESSIO
 		case CKR_ATTRIBUTE_SENSITIVE:
 		case CKR_ATTRIBUTE_TYPE_INVALID:
 		case CKR_BUFFER_TOO_SMALL:
-			WLog_ERR(TAG, "C_GetAttributeValue return %d even after buffer allocation", rv);
+			WLog_ERR(TAG,
+			         "C_GetAttributeValue failed with %s [0x%08" PRIx32
+			         "] even after buffer allocation",
+			         CK_RV_error_string(rv), rv);
+			break;
+		default:
 			break;
 	}
 	return rv;
@@ -475,12 +485,13 @@ static SECURITY_STATUS collect_keys(NCryptP11ProviderHandle* provider, P11EnumKe
 
 static BOOL convertKeyType(CK_KEY_TYPE k, LPWSTR dest, DWORD len, DWORD* outlen)
 {
-	DWORD retLen = 0;
 	const WCHAR* r = NULL;
+	size_t retLen = 0;
 
-#define ALGO_CASE(V, S) \
-	case V:             \
-		r = S;          \
+#define ALGO_CASE(V, S)                         \
+	case V:                                     \
+		r = S;                                  \
+		retLen = _wcsnlen((S), ARRAYSIZE((S))); \
 		break
 	switch (k)
 	{
@@ -513,9 +524,11 @@ static BOOL convertKeyType(CK_KEY_TYPE k, LPWSTR dest, DWORD len, DWORD* outlen)
 	}
 #undef ALGO_CASE
 
-	retLen = _wcslen(r);
+	if (retLen > UINT32_MAX)
+		return FALSE;
+
 	if (outlen)
-		*outlen = retLen;
+		*outlen = (UINT32)retLen;
 
 	if (!r)
 	{
@@ -525,7 +538,7 @@ static BOOL convertKeyType(CK_KEY_TYPE k, LPWSTR dest, DWORD len, DWORD* outlen)
 	}
 	else
 	{
-		if (retLen + 1 < len)
+		if (retLen + 1 > len)
 		{
 			WLog_ERR(TAG, "target buffer is too small for algo name");
 			return FALSE;
@@ -560,17 +573,17 @@ static void wprintKeyName(LPWSTR str, CK_SLOT_ID slotId, CK_BYTE* id, CK_ULONG i
 	for (CK_ULONG i = 0; i < idLen; i++, id++, ptr += 2)
 		(void)snprintf(ptr, 3, "%.2x", *id);
 
-	ConvertUtf8NToWChar(asciiName, ARRAYSIZE(asciiName), str,
-	                    strnlen(asciiName, ARRAYSIZE(asciiName)) + 1);
+	(void)ConvertUtf8NToWChar(asciiName, ARRAYSIZE(asciiName), str,
+	                          strnlen(asciiName, ARRAYSIZE(asciiName)) + 1);
 }
 
 static size_t parseHex(const char* str, const char* end, CK_BYTE* target)
 {
-	int ret = 0;
+	size_t ret = 0;
 
 	for (; str != end && *str; str++, ret++, target++)
 	{
-		CK_BYTE v = 0;
+		int v = 0;
 		if (*str <= '9' && *str >= '0')
 		{
 			v = (*str - '0');
@@ -610,7 +623,7 @@ static size_t parseHex(const char* str, const char* end, CK_BYTE* target)
 			return 0;
 		}
 
-		*target = v;
+		*target = v & 0xFF;
 	}
 	return ret;
 }
@@ -707,8 +720,10 @@ static SECURITY_STATUS NCryptP11EnumKeys(NCRYPT_PROV_HANDLE hProvider, LPCWSTR p
 		rv = provider->p11->C_GetSlotList(CK_TRUE, NULL, &state->nslots);
 		if (rv != CKR_OK)
 		{
+			free(state);
 			/* TODO: perhaps convert rv to NTE_*** errors */
-			WLog_WARN(TAG, "C_GetSlotList failed with %u", rv);
+			WLog_WARN(TAG, "C_GetSlotList failed with %s [0x%08" PRIx32 "]", CK_RV_error_string(rv),
+			          rv);
 			return NTE_FAIL;
 		}
 
@@ -720,7 +735,8 @@ static SECURITY_STATUS NCryptP11EnumKeys(NCRYPT_PROV_HANDLE hProvider, LPCWSTR p
 		{
 			free(state);
 			/* TODO: perhaps convert rv to NTE_*** errors */
-			WLog_WARN(TAG, "C_GetSlotList failed with %u", rv);
+			WLog_WARN(TAG, "C_GetSlotList failed with %s [0x%08" PRIx32 "]", CK_RV_error_string(rv),
+			          rv);
 			return NTE_FAIL;
 		}
 
@@ -758,6 +774,9 @@ static SECURITY_STATUS NCryptP11EnumKeys(NCRYPT_PROV_HANDLE hProvider, LPCWSTR p
 			{
 				WINPR_ASSERT(provider->p11->C_CloseSession);
 				rv = provider->p11->C_CloseSession(currentSession);
+				if (rv != CKR_OK)
+					WLog_WARN(TAG, "C_CloseSession failed with %s [0x%08" PRIx32 "]",
+					          CK_RV_error_string(rv), rv);
 				currentSession = (CK_SESSION_HANDLE)NULL;
 			}
 
@@ -766,7 +785,8 @@ static SECURITY_STATUS NCryptP11EnumKeys(NCRYPT_PROV_HANDLE hProvider, LPCWSTR p
 			                                  &currentSession);
 			if (rv != CKR_OK)
 			{
-				WLog_ERR(TAG, "unable to openSession for slot %d", key->slotId);
+				WLog_ERR(TAG, "C_OpenSession failed with %s [0x%08" PRIx32 "] for slot %d",
+				         CK_RV_error_string(rv), rv, key->slotId);
 				continue;
 			}
 			currentSlot = key->slotId;
@@ -778,7 +798,8 @@ static SECURITY_STATUS NCryptP11EnumKeys(NCRYPT_PROV_HANDLE hProvider, LPCWSTR p
 		                                      ARRAYSIZE(certificateFilter));
 		if (rv != CKR_OK)
 		{
-			WLog_ERR(TAG, "unable to initiate search for slot %d", key->slotId);
+			WLog_ERR(TAG, "C_FindObjectsInit failed with %s [0x%08" PRIx32 "] for slot %d",
+			         CK_RV_error_string(rv), rv, key->slotId);
 			continue;
 		}
 
@@ -786,7 +807,8 @@ static SECURITY_STATUS NCryptP11EnumKeys(NCRYPT_PROV_HANDLE hProvider, LPCWSTR p
 		rv = provider->p11->C_FindObjects(currentSession, &certObject, 1, &ncertObjects);
 		if (rv != CKR_OK)
 		{
-			WLog_ERR(TAG, "unable to findObjects for slot %d", currentSlot);
+			WLog_ERR(TAG, "C_FindObjects failed with %s [0x%08" PRIx32 "] for slot %d",
+			         CK_RV_error_string(rv), rv, currentSlot);
 			goto cleanup_FindObjects;
 		}
 
@@ -818,6 +840,9 @@ static SECURITY_STATUS NCryptP11EnumKeys(NCRYPT_PROV_HANDLE hProvider, LPCWSTR p
 	cleanup_FindObjects:
 		WINPR_ASSERT(provider->p11->C_FindObjectsFinal);
 		rv = provider->p11->C_FindObjectsFinal(currentSession);
+		if (rv != CKR_OK)
+			WLog_ERR(TAG, "C_FindObjectsFinal failed with %s [0x%08" PRIx32 "]",
+			         CK_RV_error_string(rv), rv);
 
 		if (keyName)
 		{
@@ -992,7 +1017,10 @@ static SECURITY_STATUS NCryptP11KeyGetProperties(NCryptP11KeyHandle* keyHandle,
 
 #define SLOT_DESC_SZ sizeof(slotInfo.slotDescription)
 			fix_padded_string((char*)slotInfo.slotDescription, SLOT_DESC_SZ);
-			*pcbResult = 2 * (strnlen((char*)slotInfo.slotDescription, SLOT_DESC_SZ) + 1);
+			const size_t len = 2ULL * (strnlen((char*)slotInfo.slotDescription, SLOT_DESC_SZ) + 1);
+			if (len > UINT32_MAX)
+				return NTE_BAD_DATA;
+			*pcbResult = (UINT32)len;
 			if (pbOutput)
 			{
 				union
@@ -1019,8 +1047,12 @@ static SECURITY_STATUS NCryptP11KeyGetProperties(NCryptP11KeyHandle* keyHandle,
 
 				if (cbOutput < 4)
 					return NTE_NO_MEMORY;
-
-				*ptr = keyHandle->slotId;
+				if (keyHandle->slotId > UINT32_MAX)
+				{
+					ret = NTE_BAD_DATA;
+					goto out_final;
+				}
+				*ptr = (UINT32)keyHandle->slotId;
 			}
 			return ERROR_SUCCESS;
 		}
@@ -1071,7 +1103,12 @@ static SECURITY_STATUS NCryptP11KeyGetProperties(NCryptP11KeyHandle* keyHandle,
 				// TODO: do a kind of translation from CKR_* to NTE_*
 			}
 
-			*pcbResult = certValue.ulValueLen;
+			if (certValue.ulValueLen > UINT32_MAX)
+			{
+				ret = NTE_BAD_DATA;
+				goto out_final;
+			}
+			*pcbResult = (UINT32)certValue.ulValueLen;
 			ret = ERROR_SUCCESS;
 			break;
 		}
@@ -1252,7 +1289,6 @@ SECURITY_STATUS NCryptOpenP11StorageProviderEx(NCRYPT_PROV_HANDLE* phProvider,
 		const char* modulePath = *modulePaths++;
 		HANDLE library = LoadLibrary(modulePath);
 		typedef CK_RV (*c_get_function_list_t)(CK_FUNCTION_LIST_PTR_PTR);
-		c_get_function_list_t c_get_function_list = NULL;
 		NCryptP11ProviderHandle* provider = NULL;
 
 		WLog_DBG(TAG, "Trying pkcs11 module '%s'", modulePath);
@@ -1262,7 +1298,9 @@ SECURITY_STATUS NCryptOpenP11StorageProviderEx(NCRYPT_PROV_HANDLE* phProvider,
 			goto out_load_library;
 		}
 
-		c_get_function_list = (c_get_function_list_t)GetProcAddress(library, "C_GetFunctionList");
+		c_get_function_list_t c_get_function_list =
+		    GetProcAddressAs(library, "C_GetFunctionList", c_get_function_list_t);
+
 		if (!c_get_function_list)
 		{
 			status = NTE_PROV_TYPE_ENTRY_BAD;
