@@ -19,6 +19,7 @@
  */
 
 #include <winpr/config.h>
+#include <winpr/build-config.h>
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -42,6 +43,7 @@
 #endif
 
 #if defined(WIN32)
+#include <windows.h>
 #include <shlobj.h>
 #else
 #include <errno.h>
@@ -215,23 +217,45 @@ static char* GetPath_XDG_CONFIG_HOME(void)
 	return path;
 }
 
+static char* GetPath_SYSTEM_CONFIG_HOME(void)
+{
+	char* path = NULL;
+#if defined(WIN32) && !defined(_UWP)
+
+	WCHAR* wpath = NULL;
+	if (FAILED(SHGetKnownFolderPath(&FOLDERID_ProgramData, 0, (HANDLE)-1, &wpath)))
+		return NULL;
+
+	if (wpath)
+		path = ConvertWCharToUtf8Alloc(wpath, NULL);
+	CoTaskMemFree(wpath);
+
+#elif defined(__IOS__)
+	path = ios_get_data();
+#else
+	path = _strdup(WINPR_INSTALL_SYSCONFDIR);
+#endif
+	return path;
+}
+
 static char* GetPath_XDG_CACHE_HOME(void)
 {
 	char* path = NULL;
-	char* home = NULL;
 #if defined(WIN32)
-	home = GetPath_XDG_RUNTIME_DIR();
-
-	if (home)
 	{
-		path = GetCombinedPath(home, "cache");
+		char* home = GetPath_XDG_RUNTIME_DIR();
 
-		if (!winpr_PathFileExists(path))
-			if (!CreateDirectoryA(path, NULL))
-				path = NULL;
+		if (home)
+		{
+			path = GetCombinedPath(home, "cache");
+
+			if (!winpr_PathFileExists(path))
+				if (!CreateDirectoryA(path, NULL))
+					path = NULL;
+		}
+
+		free(home);
 	}
-
-	free(home);
 #elif defined(__IOS__)
 	path = ios_get_cache();
 #else
@@ -249,7 +273,7 @@ static char* GetPath_XDG_CACHE_HOME(void)
 	if (path)
 		return path;
 
-	home = GetPath_HOME();
+	char* home = GetPath_HOME();
 
 	if (!home)
 		return NULL;
@@ -326,7 +350,7 @@ char* GetPath_XDG_RUNTIME_DIR(void)
 	return path;
 }
 
-char* GetKnownPath(int id)
+char* GetKnownPath(eKnownPathTypes id)
 {
 	char* path = NULL;
 
@@ -356,26 +380,28 @@ char* GetKnownPath(int id)
 			path = GetPath_XDG_RUNTIME_DIR();
 			break;
 
+		case KNOWN_PATH_SYSTEM_CONFIG_HOME:
+			path = GetPath_SYSTEM_CONFIG_HOME();
+			break;
+
 		default:
 			path = NULL;
 			break;
 	}
 
 	if (!path)
-		WLog_WARN(TAG, "Path %s is %p", GetKnownPathIdString(id), path);
+		WLog_WARN(TAG, "Path %s is %p", GetKnownPathIdString(WINPR_ASSERTING_INT_CAST(int, id)),
+		          path);
 	return path;
 }
 
-char* GetKnownSubPath(int id, const char* path)
+char* GetKnownSubPath(eKnownPathTypes id, const char* path)
 {
-	char* subPath = NULL;
-	char* knownPath = NULL;
-	knownPath = GetKnownPath(id);
-
+	char* knownPath = GetKnownPath(id);
 	if (!knownPath)
 		return NULL;
 
-	subPath = GetCombinedPath(knownPath, path);
+	char* subPath = GetCombinedPath(knownPath, path);
 	free(knownPath);
 	return subPath;
 }
@@ -642,6 +668,7 @@ BOOL PathIsDirectoryEmptyA(LPCSTR pszPath)
 	if (dir == NULL) /* Not a directory or doesn't exist */
 		return 1;
 
+	// NOLINTNEXTLINE(concurrency-mt-unsafe)
 	while ((dp = readdir(dir)) != NULL)
 	{
 		if (strcmp(dp->d_name, ".") == 0 || strcmp(dp->d_name, "..") == 0)
